@@ -23,6 +23,7 @@
 #define ARRAY_H
 
 #define C_ARRAY_SIZE(a) (sizeof (a) / sizeof *(a))
+#define C_ARRAY_ITEMSIZE(a) (sizeof *(a))
 
 #ifdef __cplusplus
 extern "C" {
@@ -34,6 +35,9 @@ array_reverse_bytes (unsigned char data[], unsigned int size);
 void
 array_reverse_bits (unsigned char data[], unsigned int size);
 
+void
+array_reverse_nibbles (unsigned char data[], unsigned int size);
+
 int
 array_isequal (const unsigned char data[], unsigned int size, unsigned char value);
 
@@ -126,6 +130,9 @@ dec2bcd (unsigned char value);
 unsigned int
 signextend (unsigned int value, unsigned int nbits);
 
+unsigned int
+popcount (unsigned int value);
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
