@@ -51,8 +51,8 @@
 #include "context-private.h"
 #include "iostream-private.h"
 #include "iterator-private.h"
-#include "descriptor-private.h"
 #include "platform.h"
+#include "array.h"
 
 #ifdef _WIN32
 #define DC_ADDRESS_FORMAT "%012I64X"
@@ -60,8 +60,6 @@
 #define DC_ADDRESS_FORMAT "%012llX"
 #endif
 
-#define C_ARRAY_SIZE(array) (sizeof (array) / sizeof *(array))
-
 #define MAX_DEVICES 255
 #define MAX_PERIODS 8
 
@@ -455,7 +453,7 @@ dc_bluetooth_iterator_next (dc_iterator_t *abstract, void *out)
 		INFO (abstract->context, "Discover: address=" DC_ADDRESS_FORMAT ", name=%s",
 			address, name ? name : "");
 
-		if (!dc_descriptor_filter (iterator->descriptor, DC_TRANSPORT_BLUETOOTH, name, NULL)) {
+		if (!dc_descriptor_filter (iterator->descriptor, DC_TRANSPORT_BLUETOOTH, name)) {
 			continue;
 		}
 
