@@ -23,6 +23,8 @@
 #include <stdlib.h> // malloc, free
 #include <assert.h> // assert
 
+#include <libdivecomputer/ble.h>
+
 #include "cressi_goa.h"
 #include "context-private.h"
 #include "device-private.h"
@@ -32,9 +34,15 @@
 
 #define ISINSTANCE(device) dc_device_isinstance((device), &cressi_goa_device_vtable)
 
-#define CMD_VERSION 0x00
-#define CMD_LOGBOOK 0x21
-#define CMD_DIVE    0x22
+#define CMD_VERSION        0x00
+#define CMD_SET_TIME       0x13
+#define CMD_EXIT_PCLINK    0x1D
+#define CMD_LOGBOOK        0x21
+#define CMD_DIVE           0x22
+#define CMD_LOGBOOK_V4     0x23
+
+#define CMD_LOGBOOK_BLE 0x02
+#define CMD_DIVE_BLE    0x03
 
 #define HEADER  0xAA
 #define TRAILER 0x55
@@ -42,10 +50,8 @@
 #define ACK     0x06
 
 #define SZ_DATA   512
-#define SZ_PACKET 10
-#define SZ_HEADER 23
+#define SZ_PACKET 12
 
-#define FP_OFFSET 0x11
 #define FP_SIZE   6
 
 #define NSTEPS    1000
@@ -57,8 +63,17 @@ typedef struct cressi_goa_device_t {
 	unsigned char fingerprint[FP_SIZE];
 } cressi_goa_device_t;
 
+typedef struct cressi_goa_conf_t {
+	unsigned int logbook_cmd;
+	unsigned int logbook_len;
+	unsigned int logbook_fp_offset;
+	unsigned int dive_fp_offset;
+} cressi_goa_conf_t;
+
 static dc_status_t cressi_goa_device_set_fingerprint (dc_device_t *abstract, const unsigned char data[], unsigned int size);
 static dc_status_t cressi_goa_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback, void *userdata);
+static dc_status_t cressi_goa_device_timesync (dc_device_t *abstract, const dc_datetime_t *datetime);
+static dc_status_t cressi_goa_device_close (dc_device_t *abstract);
 
 static const dc_device_vtable_t cressi_goa_device_vtable = {
 	sizeof(cressi_goa_device_t),
@@ -68,8 +83,13 @@ static const dc_device_vtable_t cressi_goa_device_vtable = {
 	NULL, /* write */
 	NULL, /* dump */
 	cressi_goa_device_foreach, /* foreach */
-	NULL, /* timesync */
-	NULL /* close */
+	cressi_goa_device_timesync, /* timesync */
+	cressi_goa_device_close /* close */
+};
+
+static const cressi_goa_conf_t version_conf[] = {
+	{ CMD_LOGBOOK,    23, 17, 12 },
+	{ CMD_LOGBOOK_V4, 15,  3,  4 },
 };
 
 static dc_status_t
@@ -77,6 +97,7 @@ cressi_goa_device_send (cressi_goa_device_t *device, unsigned char cmd, const un
 {
 	dc_status_t status = DC_STATUS_SUCCESS;
 	dc_device_t *abstract = (dc_device_t *) device;
+	dc_transport_t transport = dc_iostream_get_transport (device->iostream);
 
 	if (size > SZ_PACKET) {
 		ERROR (abstract->context, "Unexpected payload size (%u).", size);
@@ -100,10 +121,15 @@ cressi_goa_device_send (cressi_goa_device_t *device, unsigned char cmd, const un
 
 	// Wait a small amount of time before sending the command. Without
 	// this delay, the transfer will fail most of the time.
-	dc_iostream_sleep (device->iostream, 100);
+	unsigned int delay = transport == DC_TRANSPORT_BLE ? 2000 : 100;
+	dc_iostream_sleep (device->iostream, delay);
 
 	// Send the command to the device.
-	status = dc_iostream_write (device->iostream, packet, size + 8, NULL);
+	if (transport == DC_TRANSPORT_BLE) {
+		status = dc_iostream_write (device->iostream, packet + 4, size + 1, NULL);
+	} else {
+		status = dc_iostream_write (device->iostream, packet, size + 8, NULL);
+	}
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to send the command.");
 		return status;
@@ -113,13 +139,25 @@ cressi_goa_device_send (cressi_goa_device_t *device, unsigned char cmd, const un
 }
 
 static dc_status_t
-cressi_goa_device_receive (cressi_goa_device_t *device, unsigned char data[], unsigned int size)
+cressi_goa_device_receive (cressi_goa_device_t *device, dc_buffer_t *output)
 {
 	dc_status_t status = DC_STATUS_SUCCESS;
 	dc_device_t *abstract = (dc_device_t *) device;
+	dc_transport_t transport = dc_iostream_get_transport (device->iostream);
 
 	unsigned char packet[SZ_PACKET + 8];
 
+	if (transport == DC_TRANSPORT_BLE) {
+		if (output) {
+			return DC_STATUS_INVALIDARGS;
+		} else {
+			return DC_STATUS_SUCCESS;
+		}
+	}
+
+	// Clear the output buffer.
+	dc_buffer_clear (output);
+
 	// Read the header of the data packet.
 	status = dc_iostream_read (device->iostream, packet, 4, NULL);
 	if (status != DC_STATUS_SUCCESS) {
@@ -161,14 +199,11 @@ cressi_goa_device_receive (cressi_goa_device_t *device, unsigned char data[], un
 		return DC_STATUS_PROTOCOL;
 	}
 
-	// Verify the payload length.
-	if (length != size) {
-		ERROR (abstract->context, "Unexpected payload size (%u).", length);
-		return DC_STATUS_PROTOCOL;
-	}
-
-	if (length) {
-		memcpy (data, packet + 5, length);
+	if (length && output) {
+		if (!dc_buffer_append (output, packet + 5, length)) {
+			ERROR (abstract->context, "Could not append received data.");
+			return DC_STATUS_NOMEMORY;
+		}
 	}
 
 	return status;
@@ -179,6 +214,7 @@ cressi_goa_device_download (cressi_goa_device_t *device, dc_buffer_t *buffer, dc
 {
 	dc_status_t status = DC_STATUS_SUCCESS;
 	dc_device_t *abstract = (dc_device_t *) device;
+	dc_transport_t transport = dc_iostream_get_transport (device->iostream);
 
 	const unsigned char ack[] = {ACK};
 	const unsigned int initial = progress ? progress->current : 0;
@@ -193,27 +229,43 @@ cressi_goa_device_download (cressi_goa_device_t *device, dc_buffer_t *buffer, dc
 	unsigned int size = 2;
 	unsigned int nbytes = 0;
 	while (nbytes < size) {
-		// Read the data packet.
 		unsigned char packet[3 + SZ_DATA + 2];
-		status = dc_iostream_read (device->iostream, packet, sizeof(packet), NULL);
-		if (status != DC_STATUS_SUCCESS) {
-			ERROR (abstract->context, "Failed to receive the answer.");
-			return status;
-		}
 
-		// Verify the checksum of the packet.
-		unsigned short crc = array_uint16_le (packet + sizeof(packet) - 2);
-		unsigned short ccrc = checksum_crc16_ccitt (packet + 3, sizeof(packet) - 5, 0x0000, 0x0000);
-		if (crc != ccrc) {
-			ERROR (abstract->context, "Unexpected answer checksum.");
-			return DC_STATUS_PROTOCOL;
-		}
+		if (transport == DC_TRANSPORT_BLE) {
+			// Read the data packet.
+			unsigned int packetsize = 0;
+			while (packetsize < SZ_DATA) {
+				size_t len = 0;
+				status = dc_iostream_read (device->iostream, packet + 3 + packetsize, SZ_DATA - packetsize, &len);
+				if (status != DC_STATUS_SUCCESS) {
+					ERROR (abstract->context, "Failed to receive the answer.");
+					return status;
+				}
 
-		// Send the ack byte to the device.
-		status = dc_iostream_write (device->iostream, ack, sizeof(ack), NULL);
-		if (status != DC_STATUS_SUCCESS) {
-			ERROR (abstract->context, "Failed to send the ack byte.");
-			return status;
+				packetsize += len;
+			}
+		} else {
+			// Read the data packet.
+			status = dc_iostream_read (device->iostream, packet, sizeof(packet), NULL);
+			if (status != DC_STATUS_SUCCESS) {
+				ERROR (abstract->context, "Failed to receive the answer.");
+				return status;
+			}
+
+			// Verify the checksum of the packet.
+			unsigned short crc = array_uint16_le (packet + sizeof(packet) - 2);
+			unsigned short ccrc = checksum_crc16_ccitt (packet + 3, sizeof(packet) - 5, 0x0000, 0x0000);
+			if (crc != ccrc) {
+				ERROR (abstract->context, "Unexpected answer checksum.");
+				return DC_STATUS_PROTOCOL;
+			}
+
+			// Send the ack byte to the device.
+			status = dc_iostream_write (device->iostream, ack, sizeof(ack), NULL);
+			if (status != DC_STATUS_SUCCESS) {
+				ERROR (abstract->context, "Failed to send the ack byte.");
+				return status;
+			}
 		}
 
 		// Get the total size from the first data packet.
@@ -243,25 +295,45 @@ cressi_goa_device_download (cressi_goa_device_t *device, dc_buffer_t *buffer, dc
 		}
 	}
 
-	// Read the end byte.
-	unsigned char end = 0;
-	status = dc_iostream_read (device->iostream, &end, 1, NULL);
-	if (status != DC_STATUS_SUCCESS) {
-		ERROR (abstract->context, "Failed to receive the end byte.");
-		return status;
-	}
+	if (transport == DC_TRANSPORT_BLE) {
+		// Read the end bytes.
+		unsigned char end[16] = {0};
+		size_t len = 0;
+		status = dc_iostream_read (device->iostream, end, sizeof(end), &len);
+		if (status != DC_STATUS_SUCCESS) {
+			ERROR (abstract->context, "Failed to receive the end bytes.");
+			return status;
+		}
 
-	// Verify the end byte.
-	if (end != END) {
-		ERROR (abstract->context, "Unexpected end byte (%02x).", end);
-		return DC_STATUS_PROTOCOL;
-	}
+		// Verify the end bytes ("EOT xmodem").
+		const unsigned char validate[16] = {
+			0x45, 0x4F, 0x54, 0x20, 0x78, 0x6D, 0x6F, 0x64,
+			0x65, 0x6D, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
+		if (memcmp (end, validate, sizeof(validate)) != 0) {
+			ERROR (abstract->context, "Unexpected end bytes.");
+			return DC_STATUS_PROTOCOL;
+		}
+	} else {
+		// Read the end byte.
+		unsigned char end = 0;
+		status = dc_iostream_read (device->iostream, &end, 1, NULL);
+		if (status != DC_STATUS_SUCCESS) {
+			ERROR (abstract->context, "Failed to receive the end byte.");
+			return status;
+		}
 
-	// Send the ack byte to the device.
-	status = dc_iostream_write (device->iostream, ack, sizeof(ack), NULL);
-	if (status != DC_STATUS_SUCCESS) {
-		ERROR (abstract->context, "Failed to send the ack byte.");
-		return status;
+		// Verify the end byte.
+		if (end != END) {
+			ERROR (abstract->context, "Unexpected end byte (%02x).", end);
+			return DC_STATUS_PROTOCOL;
+		}
+
+		// Send the ack byte to the device.
+		status = dc_iostream_write (device->iostream, ack, sizeof(ack), NULL);
+		if (status != DC_STATUS_SUCCESS) {
+			ERROR (abstract->context, "Failed to send the ack byte.");
+			return status;
+		}
 	}
 
 	return status;
@@ -271,7 +343,7 @@ static dc_status_t
 cressi_goa_device_transfer (cressi_goa_device_t *device,
                             unsigned char cmd,
                             const unsigned char input[], unsigned int isize,
-                            unsigned char output[], unsigned int osize,
+                            dc_buffer_t *output,
                             dc_buffer_t *buffer,
                             dc_event_progress_t *progress)
 {
@@ -284,7 +356,7 @@ cressi_goa_device_transfer (cressi_goa_device_t *device,
 	}
 
 	// Receive the answer from the dive computer.
-	status = cressi_goa_device_receive (device, output, osize);
+	status = cressi_goa_device_receive (device, output);
 	if (status != DC_STATUS_SUCCESS) {
 		return status;
 	}
@@ -328,8 +400,10 @@ cressi_goa_device_open (dc_device_t **out, dc_context_t *context, dc_iostream_t
 		goto error_free;
 	}
 
-	// Set the timeout for receiving data (3000 ms).
-	status = dc_iostream_set_timeout (device->iostream, 3000);
+	// Set the timeout for receiving data (3000 - 5000 ms).
+	dc_transport_t transport = dc_iostream_get_transport (device->iostream);
+	int timeout = transport == DC_TRANSPORT_BLE ? 5000 : 3000;
+	status = dc_iostream_set_timeout (device->iostream, timeout);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the timeout.");
 		goto error_free;
@@ -382,6 +456,7 @@ cressi_goa_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback, v
 {
 	dc_status_t status = DC_STATUS_SUCCESS;
 	cressi_goa_device_t *device = (cressi_goa_device_t *) abstract;
+	dc_transport_t transport = dc_iostream_get_transport (device->iostream);
 	dc_buffer_t *logbook = NULL;
 	dc_buffer_t *dive = NULL;
 
@@ -389,36 +464,131 @@ cressi_goa_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback, v
 	dc_event_progress_t progress = EVENT_PROGRESS_INITIALIZER;
 	device_event_emit (abstract, DC_EVENT_PROGRESS, &progress);
 
-	// Read the version information.
-	unsigned char id[9] = {0};
-	status = cressi_goa_device_transfer (device, CMD_VERSION, NULL, 0, id, sizeof(id), NULL, NULL);
-	if (status != DC_STATUS_SUCCESS) {
-		ERROR (abstract->context, "Failed to read the version information.");
+	dc_buffer_t *id = dc_buffer_new(11);
+	if (id == NULL) {
+		ERROR (abstract->context, "Failed to allocate memory for the ID.");
+		status = DC_STATUS_NOMEMORY;
 		goto error_exit;
 	}
 
+	// Read the version information.
+	if (transport == DC_TRANSPORT_BLE) {
+		/*
+		 * With the BLE communication, there is no variant of the CMD_VERSION
+		 * command available. The corresponding information must be obtained by
+		 * reading some secondary characteristics instead:
+		 *     6E400003-B5A3-F393-E0A9-E50E24DC10B8 - 5 bytes
+		 *     6E400004-B5A3-F393-E0A9-E50E24DC10B8 - 2 bytes
+		 *     6E400005-B5A3-F393-E0A9-E50E24DC10B8 - 2 bytes
+		 */
+		const dc_ble_uuid_t characteristics[] = {
+			{0x6E, 0x40, 0x00, 0x03, 0xB5, 0xA3, 0xF3, 0x93, 0xE0, 0xA9, 0xE5, 0x0E, 0x24, 0xDC, 0x10, 0xB8},
+			{0x6E, 0x40, 0x00, 0x04, 0xB5, 0xA3, 0xF3, 0x93, 0xE0, 0xA9, 0xE5, 0x0E, 0x24, 0xDC, 0x10, 0xB8},
+			{0x6E, 0x40, 0x00, 0x05, 0xB5, 0xA3, 0xF3, 0x93, 0xE0, 0xA9, 0xE5, 0x0E, 0x24, 0xDC, 0x10, 0xB8},
+		};
+		const size_t sizes[] = {5, 2, 2};
+
+		for (size_t i = 0; i < C_ARRAY_SIZE(characteristics); ++i) {
+			unsigned char request[sizeof(dc_ble_uuid_t) + 5] = {0};
+
+			// Setup the request.
+			memcpy (request, characteristics[i], sizeof(dc_ble_uuid_t));
+			memset (request + sizeof(dc_ble_uuid_t), 0, sizes[i]);
+
+			// Read the characteristic.
+			status = dc_iostream_ioctl (device->iostream, DC_IOCTL_BLE_CHARACTERISTIC_READ, request, sizeof(dc_ble_uuid_t) + sizes[i]);
+			if (status != DC_STATUS_SUCCESS) {
+				char uuidstr[DC_BLE_UUID_SIZE] = {0};
+				ERROR (abstract->context, "Failed to read the characteristic '%s'.",
+					dc_ble_uuid2str(characteristics[i], uuidstr, sizeof(uuidstr)));
+				goto error_free_id;
+			}
+
+			// Copy the payload data.
+			if (!dc_buffer_append(id, request + sizeof(dc_ble_uuid_t), sizes[i])) {
+				ERROR (abstract->context, "Insufficient buffer space available.");
+				status = DC_STATUS_NOMEMORY;
+				goto error_free_id;
+			}
+		}
+	} else {
+		status = cressi_goa_device_transfer (device, CMD_VERSION, NULL, 0, id, NULL, NULL);
+		if (status != DC_STATUS_SUCCESS) {
+			ERROR (abstract->context, "Failed to read the version information.");
+			goto error_free_id;
+		}
+	}
+
+	const unsigned char *id_data = dc_buffer_get_data(id);
+	size_t id_size = dc_buffer_get_size(id);
+
+	HEXDUMP (abstract->context, DC_LOGLEVEL_DEBUG, "Version", id_data, id_size);
+
+	if (id_size < 9) {
+		ERROR (abstract->context, "Unexpected version length (" DC_PRINTF_SIZE ").", id_size);
+		status = DC_STATUS_DATAFORMAT;
+		goto error_free_id;
+	}
+
+	// Get the device info.
+	unsigned int model = id_data[4];
+	unsigned int firmware = array_uint16_le (id_data + 5);
+	unsigned int serial = array_uint32_le (id_data + 0);
+
+	// Get the data format version.
+	unsigned int version = 0;
+	if (id_size == 11) {
+		version = array_uint16_le (id_data + 9);
+	} else {
+		if (firmware >= 161 && firmware <= 165) {
+			version = 0;
+		} else if (firmware >= 166 && firmware <= 169) {
+			version = 1;
+		} else if (firmware >= 170 && firmware <= 179) {
+			version = 2;
+		} else if (firmware >= 100 && firmware <= 110) {
+			version = 3;
+		} else if (firmware >= 200 && firmware <= 299) {
+			version = 4;
+		} else if (firmware >= 300) {
+			version = 5;
+		} else {
+			ERROR (abstract->context, "Unknown firmware version (%u).", firmware);
+			status = DC_STATUS_DATAFORMAT;
+			goto error_free_id;
+		}
+	}
+
+	const cressi_goa_conf_t *conf = &version_conf[version >= 4];
+
 	// Emit a vendor event.
 	dc_event_vendor_t vendor;
-	vendor.data = id;
-	vendor.size = sizeof (id);
+	vendor.data = id_data;
+	vendor.size = id_size;
 	device_event_emit (abstract, DC_EVENT_VENDOR, &vendor);
 
 	// Emit a device info event.
 	dc_event_devinfo_t devinfo;
-	devinfo.model = id[4];
-	devinfo.firmware = array_uint16_le (id + 5);
-	devinfo.serial = array_uint32_le (id + 0);
+	devinfo.model = model;
+	devinfo.firmware = firmware;
+	devinfo.serial = serial;
 	device_event_emit (abstract, DC_EVENT_DEVINFO, &devinfo);
 
 	// Allocate memory for the logbook data.
 	logbook = dc_buffer_new(4096);
 	if (logbook == NULL) {
-		ERROR (abstract->context, "Failed to allocate memory.");
-		goto error_exit;
+		ERROR (abstract->context, "Failed to allocate memory for the logbook.");
+		status = DC_STATUS_NOMEMORY;
+		goto error_free_id;
 	}
 
 	// Read the logbook data.
-	status = cressi_goa_device_transfer (device, CMD_LOGBOOK, NULL, 0, NULL, 0, logbook, &progress);
+	if (transport == DC_TRANSPORT_BLE) {
+		unsigned char args[] = {0x00};
+		status = cressi_goa_device_transfer (device, CMD_LOGBOOK_BLE, args, sizeof(args), NULL, logbook, &progress);
+	} else {
+		status = cressi_goa_device_transfer (device, conf->logbook_cmd, NULL, 0, NULL, logbook, &progress);
+	}
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to read the logbook data.");
 		goto error_free_logbook;
@@ -430,9 +600,9 @@ cressi_goa_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback, v
 	// Count the number of dives.
 	unsigned int count = 0;
 	unsigned int offset = logbook_size;
-	while (offset > SZ_HEADER) {
+	while (offset >= conf->logbook_len) {
 		// Move to the start of the logbook entry.
-		offset -= SZ_HEADER;
+		offset -= conf->logbook_len;
 
 		// Get the dive number.
 		unsigned int number= array_uint16_le (logbook_data + offset);
@@ -440,7 +610,7 @@ cressi_goa_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback, v
 			break;
 
 		// Compare the fingerprint to identify previously downloaded entries.
-		if (memcmp (logbook_data + offset + FP_OFFSET, device->fingerprint, sizeof(device->fingerprint)) == 0) {
+		if (memcmp (logbook_data + offset + conf->logbook_fp_offset, device->fingerprint, sizeof(device->fingerprint)) == 0) {
 			break;
 		}
 
@@ -454,7 +624,8 @@ cressi_goa_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback, v
 	// Allocate memory for the dive data.
 	dive = dc_buffer_new(4096);
 	if (dive == NULL) {
-		ERROR (abstract->context, "Failed to allocate memory.");
+		ERROR (abstract->context, "Failed to allocate memory for the dive.");
+		status = DC_STATUS_NOMEMORY;
 		goto error_free_logbook;
 	}
 
@@ -462,10 +633,17 @@ cressi_goa_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback, v
 	offset = logbook_size;
 	for (unsigned int i = 0; i < count; ++i) {
 		// Move to the start of the logbook entry.
-		offset -= SZ_HEADER;
+		offset -= conf->logbook_len;
 
 		// Read the dive data.
-		status = cressi_goa_device_transfer (device, CMD_DIVE, logbook_data + offset, 2, NULL, 0, dive, &progress);
+		if (transport == DC_TRANSPORT_BLE) {
+			unsigned char number[2] = {
+				logbook_data[offset + 1],
+				logbook_data[offset + 0]};
+			status = cressi_goa_device_transfer (device, CMD_DIVE_BLE, number, 2, NULL, dive, &progress);
+		} else {
+			status = cressi_goa_device_transfer (device, CMD_DIVE, logbook_data + offset, 2, NULL, dive, &progress);
+		}
 		if (status != DC_STATUS_SUCCESS) {
 			ERROR (abstract->context, "Failed to read the dive data.");
 			goto error_free_dive;
@@ -474,21 +652,29 @@ cressi_goa_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback, v
 		const unsigned char *dive_data = dc_buffer_get_data (dive);
 		size_t dive_size = dc_buffer_get_size (dive);
 
-		// Verify the header in the logbook and dive data are identical.
-		// After the 2 byte dive number, the logbook header has 5 bytes
-		// extra, which are not present in the dive header.
-		if (dive_size < SZ_HEADER - 5 ||
-			memcmp (dive_data + 0, logbook_data + offset + 0, 2) != 0 ||
-			memcmp (dive_data + 2, logbook_data + offset + 7, SZ_HEADER - 7) != 0) {
+		// Verify the dive number and the fingerprint in the logbook and dive
+		// data are identical.
+		if (dive_size < conf->dive_fp_offset + FP_SIZE ||
+			memcmp (dive_data, logbook_data + offset, 2) != 0 ||
+			memcmp (dive_data + conf->dive_fp_offset, logbook_data + offset + conf->logbook_fp_offset, FP_SIZE) != 0) {
 			ERROR (abstract->context, "Unexpected dive header.");
 			status = DC_STATUS_DATAFORMAT;
 			goto error_free_dive;
 		}
 
-		// Those 5 extra bytes contain the dive mode, which is required for
-		// parsing the dive data. Therefore, insert all 5 bytes again. The
-		// remaining 4 bytes appear to be some 32 bit address.
-		if (!dc_buffer_insert (dive, 2, logbook_data + offset + 2, 5)) {
+		// The dive computer id data and the logbook entry contain important
+		// information that is required for parsing the dive data, but is
+		// not present in the dive data itself. Therefore, both pieces of data
+		// are prepended to the dive data, along with a small header containing
+		// their size.
+		const unsigned char header[] = {
+			id_size,
+			conf->logbook_len,
+		};
+		unsigned int headersize = sizeof(header) + id_size + conf->logbook_len;
+		if (!dc_buffer_prepend(dive, logbook_data + offset, conf->logbook_len) ||
+			!dc_buffer_prepend(dive, id_data, id_size) ||
+			!dc_buffer_prepend(dive, header, sizeof(header))) {
 			ERROR (abstract->context, "Out of memory.");
 			status = DC_STATUS_NOMEMORY;
 			goto error_free_dive;
@@ -497,7 +683,7 @@ cressi_goa_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback, v
 		dive_data = dc_buffer_get_data (dive);
 		dive_size = dc_buffer_get_size (dive);
 
-		if (callback && !callback(dive_data, dive_size, dive_data + FP_OFFSET, sizeof(device->fingerprint), userdata))
+		if (callback && !callback(dive_data, dive_size, dive_data + headersize + conf->dive_fp_offset, sizeof(device->fingerprint), userdata))
 			break;
 	}
 
@@ -505,6 +691,55 @@ error_free_dive:
 	dc_buffer_free(dive);
 error_free_logbook:
 	dc_buffer_free(logbook);
+error_free_id:
+	dc_buffer_free(id);
 error_exit:
 	return status;
 }
+
+static dc_status_t
+cressi_goa_device_timesync (dc_device_t *abstract, const dc_datetime_t *datetime)
+{
+	dc_status_t status = DC_STATUS_SUCCESS;
+	cressi_goa_device_t *device = (cressi_goa_device_t *) abstract;
+	dc_transport_t transport = dc_iostream_get_transport (device->iostream);
+
+	if (transport == DC_TRANSPORT_BLE) {
+		return DC_STATUS_UNSUPPORTED;
+	}
+
+	unsigned char new_time[7];
+	array_uint16_le_set(new_time, datetime->year);
+	new_time[2] = datetime->month;
+	new_time[3] = datetime->day;
+	new_time[4] = datetime->hour;
+	new_time[5] = datetime->minute;
+	new_time[6] = datetime->second;
+	status = cressi_goa_device_transfer (device, CMD_SET_TIME, new_time, sizeof(new_time), NULL, NULL, NULL);
+	if (status != DC_STATUS_SUCCESS) {
+		ERROR (abstract->context, "Failed to set the new time.");
+		return status;
+	}
+
+	return status;
+}
+
+static dc_status_t
+cressi_goa_device_close (dc_device_t *abstract)
+{
+	dc_status_t status = DC_STATUS_SUCCESS;
+	cressi_goa_device_t *device = (cressi_goa_device_t *) abstract;
+	dc_transport_t transport = dc_iostream_get_transport (device->iostream);
+
+	if (transport == DC_TRANSPORT_BLE) {
+		return DC_STATUS_SUCCESS;
+	}
+
+	status = cressi_goa_device_transfer (device, CMD_EXIT_PCLINK, NULL, 0, NULL, NULL, NULL);
+	if (status != DC_STATUS_SUCCESS) {
+		ERROR (abstract->context, "Failed to exit PC Link.");
+		return status;
+	}
+
+	return status;
+}
