@@ -35,7 +35,7 @@ dc_status_t
 cressi_leonardo_device_open (dc_device_t **device, dc_context_t *context, dc_iostream_t *iostream);
 
 dc_status_t
-cressi_leonardo_parser_create (dc_parser_t **parser, dc_context_t *context, unsigned int model);
+cressi_leonardo_parser_create (dc_parser_t **parser, dc_context_t *context, const unsigned char data[], size_t size, unsigned int model);
 
 #ifdef __cplusplus
 }
