@@ -23,12 +23,13 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "descriptor-private.h"
+#include <libdivecomputer/descriptor.h>
+#include <libdivecomputer/usbhid.h>
+#include <libdivecomputer/usb.h>
+
 #include "iterator-private.h"
 #include "platform.h"
-
-#define C_ARRAY_SIZE(array) (sizeof (array) / sizeof *(array))
-#define C_ARRAY_ITEMSIZE(array) (sizeof *(array))
+#include "array.h"
 
 #define DC_FILTER_INTERNAL(key, values, isnullterminated, match) \
 	dc_filter_internal( \
@@ -36,36 +37,32 @@
 		values, \
 		C_ARRAY_SIZE(values) - isnullterminated, \
 		C_ARRAY_ITEMSIZE(values), \
-		match, \
-		NULL, NULL, 0)
+		match)
 
-#define DC_FILTER_INTERNAL_WITH_PARAMS(key, values, isnullterminated, match, params_dst, params_src) \
-	dc_filter_internal( \
-		key, \
-		values, \
-		C_ARRAY_SIZE(values) - isnullterminated, \
-		C_ARRAY_ITEMSIZE(values), \
-		match, \
-		params_dst, params_src, sizeof *(params_src))
+#undef dc_descriptor_iterator
+
+dc_status_t dc_descriptor_iterator (dc_iterator_t **out);
 
 typedef int (*dc_match_t)(const void *, const void *);
 
-typedef int (*dc_filter_t) (dc_transport_t transport, const void *userdata, void *params);
+typedef int (*dc_filter_t) (const dc_descriptor_t *descriptor, dc_transport_t transport, const void *userdata);
 
-static int dc_filter_uwatec (dc_transport_t transport, const void *userdata, void *params);
-static int dc_filter_suunto (dc_transport_t transport, const void *userdata, void *params);
-static int dc_filter_shearwater (dc_transport_t transport, const void *userdata, void *params);
-static int dc_filter_hw (dc_transport_t transport, const void *userdata, void *params);
-static int dc_filter_tecdiving (dc_transport_t transport, const void *userdata, void *params);
-static int dc_filter_mares (dc_transport_t transport, const void *userdata, void *params);
-static int dc_filter_divesystem (dc_transport_t transport, const void *userdata, void *params);
-static int dc_filter_oceanic (dc_transport_t transport, const void *userdata, void *params);
-static int dc_filter_mclean (dc_transport_t transport, const void *userdata, void *params);
-static int dc_filter_atomic (dc_transport_t transport, const void *userdata, void *params);
-static int dc_filter_deepsix (dc_transport_t transport, const void *userdata, void *params);
-static int dc_filter_deepblu (dc_transport_t transport, const void *userdata, void *params);
-static int dc_filter_oceans (dc_transport_t transport, const void *userdata, void *params);
-static int dc_filter_divesoft (dc_transport_t transport, const void *userdata, void *params);
+static int dc_filter_uwatec (const dc_descriptor_t *descriptor, dc_transport_t transport, const void *userdata);
+static int dc_filter_suunto (const dc_descriptor_t *descriptor, dc_transport_t transport, const void *userdata);
+static int dc_filter_shearwater (const dc_descriptor_t *descriptor, dc_transport_t transport, const void *userdata);
+static int dc_filter_hw (const dc_descriptor_t *descriptor, dc_transport_t transport, const void *userdata);
+static int dc_filter_tecdiving (const dc_descriptor_t *descriptor, dc_transport_t transport, const void *userdata);
+static int dc_filter_mares (const dc_descriptor_t *descriptor, dc_transport_t transport, const void *userdata);
+static int dc_filter_divesystem (const dc_descriptor_t *descriptor, dc_transport_t transport, const void *userdata);
+static int dc_filter_oceanic (const dc_descriptor_t *descriptor, dc_transport_t transport, const void *userdata);
+static int dc_filter_mclean (const dc_descriptor_t *descriptor, dc_transport_t transport, const void *userdata);
+static int dc_filter_atomic (const dc_descriptor_t *descriptor, dc_transport_t transport, const void *userdata);
+static int dc_filter_deepsix (const dc_descriptor_t *descriptor, dc_transport_t transport, const void *userdata);
+static int dc_filter_deepblu (const dc_descriptor_t *descriptor, dc_transport_t transport, const void *userdata);
+static int dc_filter_oceans (const dc_descriptor_t *descriptor, dc_transport_t transport, const void *userdata);
+static int dc_filter_divesoft (const dc_descriptor_t *descriptor, dc_transport_t transport, const void *userdata);
+static int dc_filter_cressi (const dc_descriptor_t *descriptor, dc_transport_t transport, const void *userdata);
+static int dc_filter_halcyon (const dc_descriptor_t *descriptor, dc_transport_t transport, const void *userdata);
 
 static dc_status_t dc_descriptor_iterator_next (dc_iterator_t *iterator, void *item);
 
@@ -153,6 +150,7 @@ static const dc_descriptor_t g_descriptors[] = {
 	{"Uwatec",   "Galileo Terra",       DC_FAMILY_UWATEC_SMART, 0x11, DC_TRANSPORT_IRDA, dc_filter_uwatec},
 	{"Uwatec",   "Aladin Tec",          DC_FAMILY_UWATEC_SMART, 0x12, DC_TRANSPORT_IRDA, dc_filter_uwatec},
 	{"Uwatec",   "Aladin Prime",        DC_FAMILY_UWATEC_SMART, 0x12, DC_TRANSPORT_IRDA, dc_filter_uwatec},
+	{"Uwatec",   "Aladin One",          DC_FAMILY_UWATEC_SMART, 0x12, DC_TRANSPORT_IRDA, dc_filter_uwatec},
 	{"Uwatec",   "Aladin Tec 2G",       DC_FAMILY_UWATEC_SMART, 0x13, DC_TRANSPORT_IRDA, dc_filter_uwatec},
 	{"Uwatec",   "Aladin 2G",           DC_FAMILY_UWATEC_SMART, 0x13, DC_TRANSPORT_IRDA, dc_filter_uwatec},
 	{"Subgear",  "XP-10",               DC_FAMILY_UWATEC_SMART, 0x13, DC_TRANSPORT_IRDA, dc_filter_uwatec},
@@ -177,7 +175,10 @@ static const dc_descriptor_t g_descriptors[] = {
 	{"Scubapro", "G2 TEK",              DC_FAMILY_UWATEC_SMART, 0x31, DC_TRANSPORT_USBHID | DC_TRANSPORT_BLE, dc_filter_uwatec},
 	{"Scubapro", "G2",                  DC_FAMILY_UWATEC_SMART, 0x32, DC_TRANSPORT_USBHID | DC_TRANSPORT_BLE, dc_filter_uwatec},
 	{"Scubapro", "G2 Console",          DC_FAMILY_UWATEC_SMART, 0x32, DC_TRANSPORT_USBHID | DC_TRANSPORT_BLE, dc_filter_uwatec},
+	{"Scubapro", "G3",                  DC_FAMILY_UWATEC_SMART, 0x34, DC_TRANSPORT_USBHID | DC_TRANSPORT_BLE, dc_filter_uwatec},
 	{"Scubapro", "G2 HUD",              DC_FAMILY_UWATEC_SMART, 0x42, DC_TRANSPORT_USBHID | DC_TRANSPORT_BLE, dc_filter_uwatec},
+	{"Scubapro", "Luna 2.0 AI",         DC_FAMILY_UWATEC_SMART, 0x50, DC_TRANSPORT_BLE, dc_filter_uwatec},
+	{"Scubapro", "Luna 2.0",            DC_FAMILY_UWATEC_SMART, 0x51, DC_TRANSPORT_BLE, dc_filter_uwatec},
 	/* Reefnet */
 	{"Reefnet", "Sensus",       DC_FAMILY_REEFNET_SENSUS, 1, DC_TRANSPORT_SERIAL, NULL},
 	{"Reefnet", "Sensus Pro",   DC_FAMILY_REEFNET_SENSUSPRO, 2, DC_TRANSPORT_SERIAL, NULL},
@@ -274,8 +275,13 @@ static const dc_descriptor_t g_descriptors[] = {
 	{"Sherwood", "Amphos Air 2.0",      DC_FAMILY_OCEANIC_ATOM2, 0x4658, DC_TRANSPORT_SERIAL, NULL},
 	{"Sherwood", "Beacon",              DC_FAMILY_OCEANIC_ATOM2, 0x4742, DC_TRANSPORT_SERIAL | DC_TRANSPORT_BLE, dc_filter_oceanic},
 	{"Aqualung", "i470TC",              DC_FAMILY_OCEANIC_ATOM2, 0x4743, DC_TRANSPORT_SERIAL | DC_TRANSPORT_BLE, dc_filter_oceanic},
+	{"Aqualung", "i100",                DC_FAMILY_OCEANIC_ATOM2, 0x4745, DC_TRANSPORT_SERIAL, NULL},
 	{"Aqualung", "i200C",               DC_FAMILY_OCEANIC_ATOM2, 0x4749, DC_TRANSPORT_SERIAL | DC_TRANSPORT_BLE, dc_filter_oceanic},
 	{"Oceanic",  "Geo Air",             DC_FAMILY_OCEANIC_ATOM2, 0x474B, DC_TRANSPORT_SERIAL | DC_TRANSPORT_BLE, dc_filter_oceanic},
+	/* Pelagic I330R */
+	{"Apeks",    "DSX",                 DC_FAMILY_PELAGIC_I330R, 0x4741, DC_TRANSPORT_BLE, dc_filter_oceanic},
+	{"Aqualung", "i330R",               DC_FAMILY_PELAGIC_I330R, 0x4744, DC_TRANSPORT_BLE, dc_filter_oceanic},
+	{"Aqualung", "i330R Console",       DC_FAMILY_PELAGIC_I330R, 0x474D, DC_TRANSPORT_BLE, dc_filter_oceanic},
 	/* Mares Nemo */
 	{"Mares", "Nemo",         DC_FAMILY_MARES_NEMO, 0, DC_TRANSPORT_SERIAL, NULL},
 	{"Mares", "Nemo Steel",   DC_FAMILY_MARES_NEMO, 0, DC_TRANSPORT_SERIAL, NULL},
@@ -308,6 +314,11 @@ static const dc_descriptor_t g_descriptors[] = {
 	{"Mares", "Smart Air",         DC_FAMILY_MARES_ICONHD , 0x24, DC_TRANSPORT_SERIAL | DC_TRANSPORT_BLE, dc_filter_mares},
 	{"Mares", "Quad",              DC_FAMILY_MARES_ICONHD , 0x29, DC_TRANSPORT_SERIAL | DC_TRANSPORT_BLE, dc_filter_mares},
 	{"Mares", "Horizon",           DC_FAMILY_MARES_ICONHD , 0x2C, DC_TRANSPORT_SERIAL, NULL},
+	{"Mares", "Puck Air 2",        DC_FAMILY_MARES_ICONHD , 0x2D, DC_TRANSPORT_BLE, dc_filter_mares},
+	{"Mares", "Sirius",            DC_FAMILY_MARES_ICONHD , 0x2F, DC_TRANSPORT_BLE, dc_filter_mares},
+	{"Mares", "Quad Ci",           DC_FAMILY_MARES_ICONHD , 0x31, DC_TRANSPORT_BLE, dc_filter_mares},
+	{"Mares", "Puck 4",            DC_FAMILY_MARES_ICONHD , 0x35, DC_TRANSPORT_BLE, dc_filter_mares},
+	{"Mares", "Puck Lite",         DC_FAMILY_MARES_ICONHD , 0x35, DC_TRANSPORT_BLE, dc_filter_mares},
 	/* Heinrichs Weikamp */
 	{"Heinrichs Weikamp", "OSTC",     DC_FAMILY_HW_OSTC, 0, DC_TRANSPORT_SERIAL, NULL},
 	{"Heinrichs Weikamp", "OSTC Mk2", DC_FAMILY_HW_OSTC, 1, DC_TRANSPORT_SERIAL, NULL},
@@ -321,12 +332,14 @@ static const dc_descriptor_t g_descriptors[] = {
 	{"Heinrichs Weikamp", "OSTC Plus",  DC_FAMILY_HW_OSTC3, 0x13, DC_TRANSPORT_SERIAL | DC_TRANSPORT_BLUETOOTH | DC_TRANSPORT_BLE, dc_filter_hw},
 	{"Heinrichs Weikamp", "OSTC Plus",  DC_FAMILY_HW_OSTC3, 0x1A, DC_TRANSPORT_SERIAL | DC_TRANSPORT_BLUETOOTH | DC_TRANSPORT_BLE, dc_filter_hw},
 	{"Heinrichs Weikamp", "OSTC 4",     DC_FAMILY_HW_OSTC3, 0x3B, DC_TRANSPORT_SERIAL | DC_TRANSPORT_BLUETOOTH | DC_TRANSPORT_BLE, dc_filter_hw},
+	{"Heinrichs Weikamp", "OSTC 5",     DC_FAMILY_HW_OSTC3, 0x3B, DC_TRANSPORT_SERIAL | DC_TRANSPORT_BLUETOOTH | DC_TRANSPORT_BLE, dc_filter_hw},
 	{"Heinrichs Weikamp", "OSTC cR",    DC_FAMILY_HW_OSTC3, 0x05, DC_TRANSPORT_SERIAL, NULL},
 	{"Heinrichs Weikamp", "OSTC cR",    DC_FAMILY_HW_OSTC3, 0x07, DC_TRANSPORT_SERIAL, NULL},
 	{"Heinrichs Weikamp", "OSTC Sport", DC_FAMILY_HW_OSTC3, 0x12, DC_TRANSPORT_SERIAL | DC_TRANSPORT_BLUETOOTH | DC_TRANSPORT_BLE, dc_filter_hw},
 	{"Heinrichs Weikamp", "OSTC Sport", DC_FAMILY_HW_OSTC3, 0x13, DC_TRANSPORT_SERIAL | DC_TRANSPORT_BLUETOOTH | DC_TRANSPORT_BLE, dc_filter_hw},
 	{"Heinrichs Weikamp", "OSTC 2 TR",  DC_FAMILY_HW_OSTC3, 0x33, DC_TRANSPORT_SERIAL | DC_TRANSPORT_BLUETOOTH | DC_TRANSPORT_BLE, dc_filter_hw},
 	/* Cressi Edy */
+	{"Cressi", "Archimede", DC_FAMILY_CRESSI_EDY, 0x01, DC_TRANSPORT_SERIAL, NULL},
 	{"Tusa",   "IQ-700", DC_FAMILY_CRESSI_EDY, 0x05, DC_TRANSPORT_SERIAL, NULL},
 	{"Cressi", "Edy",    DC_FAMILY_CRESSI_EDY, 0x08, DC_TRANSPORT_SERIAL, NULL},
 	/* Cressi Leonardo */
@@ -335,11 +348,13 @@ static const dc_descriptor_t g_descriptors[] = {
 	{"Cressi", "Newton",   DC_FAMILY_CRESSI_LEONARDO, 5, DC_TRANSPORT_SERIAL, NULL},
 	{"Cressi", "Drake",    DC_FAMILY_CRESSI_LEONARDO, 6, DC_TRANSPORT_SERIAL, NULL},
 	/* Cressi Goa */
-	{"Cressi", "Cartesio", DC_FAMILY_CRESSI_GOA, 1, DC_TRANSPORT_SERIAL, NULL},
-	{"Cressi", "Goa",      DC_FAMILY_CRESSI_GOA, 2, DC_TRANSPORT_SERIAL, NULL},
-	{"Cressi", "Donatello",    DC_FAMILY_CRESSI_GOA, 4, DC_TRANSPORT_SERIAL, NULL},
-	{"Cressi", "Michelangelo", DC_FAMILY_CRESSI_GOA, 5, DC_TRANSPORT_SERIAL, NULL},
-	{"Cressi", "Neon",     DC_FAMILY_CRESSI_GOA, 9, DC_TRANSPORT_SERIAL, NULL},
+	{"Cressi", "Cartesio", DC_FAMILY_CRESSI_GOA, 1, DC_TRANSPORT_SERIAL | DC_TRANSPORT_BLE, dc_filter_cressi},
+	{"Cressi", "Goa",      DC_FAMILY_CRESSI_GOA, 2, DC_TRANSPORT_SERIAL | DC_TRANSPORT_BLE, dc_filter_cressi},
+	{"Cressi", "Leonardo 2.0", DC_FAMILY_CRESSI_GOA, 3, DC_TRANSPORT_SERIAL | DC_TRANSPORT_BLE, dc_filter_cressi},
+	{"Cressi", "Donatello",    DC_FAMILY_CRESSI_GOA, 4, DC_TRANSPORT_SERIAL | DC_TRANSPORT_BLE, dc_filter_cressi},
+	{"Cressi", "Michelangelo", DC_FAMILY_CRESSI_GOA, 5, DC_TRANSPORT_SERIAL | DC_TRANSPORT_BLE, dc_filter_cressi},
+	{"Cressi", "Neon",     DC_FAMILY_CRESSI_GOA, 9, DC_TRANSPORT_SERIAL | DC_TRANSPORT_BLE, dc_filter_cressi},
+	{"Cressi", "Nepto",    DC_FAMILY_CRESSI_GOA, 10, DC_TRANSPORT_SERIAL | DC_TRANSPORT_BLE, dc_filter_cressi},
 	/* Zeagle N2iTiON3 */
 	{"Zeagle",    "N2iTiON3",   DC_FAMILY_ZEAGLE_N2ITION3, 0, DC_TRANSPORT_SERIAL, NULL},
 	{"Apeks",     "Quantum X",  DC_FAMILY_ZEAGLE_N2ITION3, 0, DC_TRANSPORT_SERIAL, NULL},
@@ -361,6 +376,9 @@ static const dc_descriptor_t g_descriptors[] = {
 	{"Shearwater", "Peregrine", DC_FAMILY_SHEARWATER_PETREL, 9, DC_TRANSPORT_BLE, dc_filter_shearwater},
 	{"Shearwater", "Petrel 3",  DC_FAMILY_SHEARWATER_PETREL, 10, DC_TRANSPORT_BLE, dc_filter_shearwater},
 	{"Shearwater", "Perdix 2",  DC_FAMILY_SHEARWATER_PETREL, 11, DC_TRANSPORT_BLE, dc_filter_shearwater},
+	{"Shearwater", "Tern",      DC_FAMILY_SHEARWATER_PETREL, 12, DC_TRANSPORT_BLE, dc_filter_shearwater},
+	{"Shearwater", "Tern TX",   DC_FAMILY_SHEARWATER_PETREL, 12, DC_TRANSPORT_BLE, dc_filter_shearwater},
+	{"Shearwater", "Peregrine TX", DC_FAMILY_SHEARWATER_PETREL, 13, DC_TRANSPORT_BLE, dc_filter_shearwater},
 	/* Dive Rite NiTek Q */
 	{"Dive Rite", "NiTek Q",   DC_FAMILY_DIVERITE_NITEKQ, 0, DC_TRANSPORT_SERIAL, NULL},
 	/* Citizen Hyper Aqualand */
@@ -414,6 +432,20 @@ static const dc_descriptor_t g_descriptors[] = {
 	{"Ratio",      "iX3M 2021 Pro Deep",  DC_FAMILY_DIVESYSTEM_IDIVE, 0x73, DC_TRANSPORT_SERIAL, NULL},
 	{"Ratio",      "iX3M 2021 Pro Tech+", DC_FAMILY_DIVESYSTEM_IDIVE, 0x74, DC_TRANSPORT_SERIAL, NULL},
 	{"Ratio",      "iX3M 2021 Pro Reb",   DC_FAMILY_DIVESYSTEM_IDIVE, 0x75, DC_TRANSPORT_SERIAL, NULL},
+	{"Ratio",      "iDive 2 Free",        DC_FAMILY_DIVESYSTEM_IDIVE, 0x80, DC_TRANSPORT_SERIAL | DC_TRANSPORT_BLE, dc_filter_divesystem},
+	{"Ratio",      "iDive 2 Fancy",       DC_FAMILY_DIVESYSTEM_IDIVE, 0x81, DC_TRANSPORT_SERIAL | DC_TRANSPORT_BLE, dc_filter_divesystem},
+	{"Ratio",      "iDive 2 Easy",        DC_FAMILY_DIVESYSTEM_IDIVE, 0x82, DC_TRANSPORT_SERIAL | DC_TRANSPORT_BLE, dc_filter_divesystem},
+	{"Ratio",      "iDive 2 Pro",         DC_FAMILY_DIVESYSTEM_IDIVE, 0x83, DC_TRANSPORT_SERIAL | DC_TRANSPORT_BLE, dc_filter_divesystem},
+	{"Ratio",      "iDive 2 Deep",        DC_FAMILY_DIVESYSTEM_IDIVE, 0x84, DC_TRANSPORT_SERIAL | DC_TRANSPORT_BLE, dc_filter_divesystem},
+	{"Ratio",      "iDive 2 Tech",        DC_FAMILY_DIVESYSTEM_IDIVE, 0x85, DC_TRANSPORT_SERIAL | DC_TRANSPORT_BLE, dc_filter_divesystem},
+	{"Ratio",      "iDive 2 Reb",         DC_FAMILY_DIVESYSTEM_IDIVE, 0x86, DC_TRANSPORT_SERIAL | DC_TRANSPORT_BLE, dc_filter_divesystem},
+	{"Ratio",      "iX3M 2 GPS Gauge",    DC_FAMILY_DIVESYSTEM_IDIVE, 0x90, DC_TRANSPORT_SERIAL | DC_TRANSPORT_BLE, dc_filter_divesystem},
+	{"Ratio",      "iX3M 2 GPS Easy",     DC_FAMILY_DIVESYSTEM_IDIVE, 0x91, DC_TRANSPORT_SERIAL | DC_TRANSPORT_BLE, dc_filter_divesystem},
+	{"Ratio",      "iX3M 2 GPS Pro",      DC_FAMILY_DIVESYSTEM_IDIVE, 0x92, DC_TRANSPORT_SERIAL | DC_TRANSPORT_BLE, dc_filter_divesystem},
+	{"Ratio",      "iX3M 2 GPS Deep",     DC_FAMILY_DIVESYSTEM_IDIVE, 0x93, DC_TRANSPORT_SERIAL | DC_TRANSPORT_BLE, dc_filter_divesystem},
+	{"Ratio",      "iX3M 2 GPS Tech",     DC_FAMILY_DIVESYSTEM_IDIVE, 0x94, DC_TRANSPORT_SERIAL | DC_TRANSPORT_BLE, dc_filter_divesystem},
+	{"Ratio",      "iX3M 2 GPS Reb",      DC_FAMILY_DIVESYSTEM_IDIVE, 0x95, DC_TRANSPORT_SERIAL | DC_TRANSPORT_BLE, dc_filter_divesystem},
+	{"Ratio",      "ATOM",                DC_FAMILY_DIVESYSTEM_IDIVE, 0x96, DC_TRANSPORT_SERIAL | DC_TRANSPORT_BLE, dc_filter_divesystem},
 	{"Ratio",      "iX3M 2 Gauge",        DC_FAMILY_DIVESYSTEM_IDIVE, 0x100, DC_TRANSPORT_SERIAL | DC_TRANSPORT_BLE, dc_filter_divesystem},
 	{"Ratio",      "iX3M 2 Easy",         DC_FAMILY_DIVESYSTEM_IDIVE, 0x101, DC_TRANSPORT_SERIAL | DC_TRANSPORT_BLE, dc_filter_divesystem},
 	{"Ratio",      "iX3M 2 Pro",          DC_FAMILY_DIVESYSTEM_IDIVE, 0x102, DC_TRANSPORT_SERIAL | DC_TRANSPORT_BLE, dc_filter_divesystem},
@@ -443,11 +475,11 @@ static const dc_descriptor_t g_descriptors[] = {
 	{"Deep Six", "Excursion", DC_FAMILY_DEEPSIX_EXCURSION, 0, DC_TRANSPORT_BLE, dc_filter_deepsix},
 	{"Crest",    "CR-4",      DC_FAMILY_DEEPSIX_EXCURSION, 0, DC_TRANSPORT_BLE, dc_filter_deepsix},
 	{"Genesis",  "Centauri",  DC_FAMILY_DEEPSIX_EXCURSION, 0, DC_TRANSPORT_BLE, dc_filter_deepsix},
-	{"Tusa",     "TC1",       DC_FAMILY_DEEPSIX_EXCURSION, 0, DC_TRANSPORT_BLE, dc_filter_deepsix},
 	{"Scorpena", "Alpha",     DC_FAMILY_DEEPSIX_EXCURSION, 0, DC_TRANSPORT_BLE, dc_filter_deepsix},
 	/* Seac Screen */
-	{"Seac", "Screen", DC_FAMILY_SEAC_SCREEN, 0, DC_TRANSPORT_SERIAL, NULL},
-	{"Seac", "Action", DC_FAMILY_SEAC_SCREEN, 0, DC_TRANSPORT_SERIAL, NULL},
+	{"Seac", "Action", DC_FAMILY_SEAC_SCREEN, 0x01, DC_TRANSPORT_SERIAL, NULL},
+	{"Seac", "Screen", DC_FAMILY_SEAC_SCREEN, 0x02, DC_TRANSPORT_SERIAL, NULL},
+	{"Seac", "Tablet", DC_FAMILY_SEAC_SCREEN, 0x10, DC_TRANSPORT_SERIAL, NULL},
 	/* Deepblu Cosmiq */
 	{"Deepblu", "Cosmiq+", DC_FAMILY_DEEPBLU_COSMIQ, 0, DC_TRANSPORT_BLE, dc_filter_deepblu},
 	/* Oceans S1 */
@@ -455,8 +487,29 @@ static const dc_descriptor_t g_descriptors[] = {
 	/* Divesoft Freedom */
 	{"Divesoft", "Freedom", DC_FAMILY_DIVESOFT_FREEDOM, 19, DC_TRANSPORT_BLE, dc_filter_divesoft},
 	{"Divesoft", "Liberty", DC_FAMILY_DIVESOFT_FREEDOM, 10, DC_TRANSPORT_BLE, dc_filter_divesoft},
+	/* Halcyon Symbios */
+	{"Halcyon", "Symbios HUD",     DC_FAMILY_HALCYON_SYMBIOS, 1, DC_TRANSPORT_BLE, dc_filter_halcyon},
+	{"Halcyon", "Symbios Handset", DC_FAMILY_HALCYON_SYMBIOS, 7, DC_TRANSPORT_BLE, dc_filter_halcyon},
 };
 
+static int
+dc_match_usb (const void *key, const void *value)
+{
+	const dc_usb_desc_t *k = (const dc_usb_desc_t *) key;
+	const dc_usb_desc_t *v = (const dc_usb_desc_t *) value;
+
+	return k->vid == v->vid && k->pid == v->pid;
+}
+
+static int
+dc_match_usbhid (const void *key, const void *value)
+{
+	const dc_usbhid_desc_t *k = (const dc_usbhid_desc_t *) key;
+	const dc_usbhid_desc_t *v = (const dc_usbhid_desc_t *) value;
+
+	return k->vid == v->vid && k->pid == v->pid;
+}
+
 static int
 dc_match_name (const void *key, const void *value)
 {
@@ -476,32 +529,14 @@ dc_match_prefix (const void *key, const void *value)
 }
 
 static int
-dc_match_devname (const void *key, const void *value)
-{
-	const char *k = (const char *) key;
-	const char *v = *(const char * const *) value;
-
-	return strncmp (k, v, strlen (v)) == 0;
-}
-
-static int
-dc_match_usb (const void *key, const void *value)
-{
-	const dc_usb_desc_t *k = (const dc_usb_desc_t *) key;
-	const dc_usb_desc_t *v = (const dc_usb_desc_t *) value;
-
-	return k->vid == v->vid && k->pid == v->pid;
-}
-
-static int
-dc_match_number_with_prefix (const void *key, const void *value)
+dc_match_prefix_with_number (const void *key, const void *value)
 {
 	const char *str = (const char *) key;
 	const char *prefix = *(const char * const *) value;
 
 	size_t n = strlen (prefix);
 
-	if (strncmp (str, prefix, n) != 0) {
+	if (strncasecmp (str, prefix, n) != 0) {
 		return 0;
 	}
 
@@ -516,6 +551,31 @@ dc_match_number_with_prefix (const void *key, const void *value)
 	return 1;
 }
 
+static int
+dc_match_prefix_with_hex (const void *key, const void *value)
+{
+	const char *str = (const char *) key;
+	const char *prefix = *(const char * const *) value;
+
+	size_t n = strlen (prefix);
+
+	if (strncasecmp (str, prefix, n) != 0) {
+		return 0;
+	}
+
+	while (str[n] != 0) {
+		const char c = str[n];
+		if ((c < '0' || c > '9') &&
+			(c < 'A' || c > 'F') &&
+			(c < 'a' || c > 'f')) {
+			return 0;
+		}
+		n++;
+	}
+
+	return 1;
+}
+
 static int
 dc_match_oceanic (const void *key, const void *value)
 {
@@ -529,20 +589,60 @@ dc_match_oceanic (const void *key, const void *value)
 
 	const char *p = prefix;
 
-	return dc_match_number_with_prefix (key, &p);
+	return dc_match_prefix_with_number (key, &p);
 }
 
 static int
-dc_filter_internal (const void *key, const void *values, size_t count, size_t size, dc_match_t match, void *params_dst, const void *params_src, size_t params_size)
+dc_match_cressi (const void *key, const void *value)
+{
+	unsigned int model = *(const unsigned int *) value;
+
+	char prefix[16] = {0};
+
+	dc_platform_snprintf(prefix, sizeof(prefix), "%x_", model);
+
+	const char *p = prefix;
+
+	return dc_match_prefix_with_hex (key, &p);
+}
+
+static int
+dc_match_halcyon (const void *key, const void *value)
+{
+	const char *str = (const char *) key;
+	unsigned int model = *(const unsigned int *) value;
+
+	char prefix[16] = {0};
+	dc_platform_snprintf(prefix, sizeof(prefix), "H%02u", model);
+
+	if (strncasecmp (str, prefix, 3) == 0) {
+		return 1;
+	}
+
+	size_t n = 0;
+	while (str[n] != 0) {
+		const char c = str[n];
+		if (c < '0' || c > '9') {
+			return 0;
+		}
+		n++;
+	}
+
+	if (n < 10) {
+		return 0;
+	}
+
+	return strncasecmp (str + 4, prefix + 1, 2) == 0;
+}
+
+static int
+dc_filter_internal (const void *key, const void *values, size_t count, size_t size, dc_match_t match)
 {
 	if (key == NULL)
 		return 1;
 
 	for (size_t i = 0; i < count; ++i) {
 		if (match (key, (const unsigned char *) values + i * size)) {
-			if (params_src && params_dst) {
-				memcpy (params_dst, params_src, params_size);
-			}
 			return 1;
 		}
 	}
@@ -550,14 +650,8 @@ dc_filter_internal (const void *key, const void *values, size_t count, size_t si
 	return count == 0;
 }
 
-static const char * const rfcomm[] = {
-#if defined (__linux__)
-	"/dev/rfcomm",
-#endif
-	NULL
-};
-
-static int dc_filter_uwatec (dc_transport_t transport, const void *userdata, void *params)
+static int
+dc_filter_uwatec (const dc_descriptor_t *descriptor, dc_transport_t transport, const void *userdata)
 {
 	static const char * const irda[] = {
 		"Aladin Smart Com",
@@ -568,7 +662,7 @@ static int dc_filter_uwatec (dc_transport_t transport, const void *userdata, voi
 		"UWATEC Galileo",
 		"UWATEC Galileo Sol",
 	};
-	static const dc_usb_desc_t usbhid[] = {
+	static const dc_usbhid_desc_t usbhid[] = {
 		{0x2e6c, 0x3201}, // G2, G2 TEK
 		{0x2e6c, 0x3211}, // G2 Console
 		{0x2e6c, 0x4201}, // G2 HUD
@@ -581,12 +675,15 @@ static int dc_filter_uwatec (dc_transport_t transport, const void *userdata, voi
 		"A1",
 		"A2",
 		"G2 TEK",
+		"Galileo 3",
+		"Luna 2.0 AI",
+		"Luna 2.0",
 	};
 
 	if (transport == DC_TRANSPORT_IRDA) {
 		return DC_FILTER_INTERNAL (userdata, irda, 0, dc_match_name);
 	} else if (transport == DC_TRANSPORT_USBHID) {
-		return DC_FILTER_INTERNAL (userdata, usbhid, 0, dc_match_usb);
+		return DC_FILTER_INTERNAL (userdata, usbhid, 0, dc_match_usbhid);
 	} else if (transport == DC_TRANSPORT_BLE) {
 		return DC_FILTER_INTERNAL (userdata, bluetooth, 0, dc_match_name);
 	}
@@ -594,9 +691,10 @@ static int dc_filter_uwatec (dc_transport_t transport, const void *userdata, voi
 	return 1;
 }
 
-static int dc_filter_suunto (dc_transport_t transport, const void *userdata, void *params)
+static int
+dc_filter_suunto (const dc_descriptor_t *descriptor, dc_transport_t transport, const void *userdata)
 {
-	static const dc_usb_desc_t usbhid[] = {
+	static const dc_usbhid_desc_t usbhid[] = {
 		{0x1493, 0x0030}, // Eon Steel
 		{0x1493, 0x0033}, // Eon Core
 		{0x1493, 0x0035}, // D5
@@ -610,7 +708,7 @@ static int dc_filter_suunto (dc_transport_t transport, const void *userdata, voi
 	};
 
 	if (transport == DC_TRANSPORT_USBHID) {
-		return DC_FILTER_INTERNAL (userdata, usbhid, 0, dc_match_usb);
+		return DC_FILTER_INTERNAL (userdata, usbhid, 0, dc_match_usbhid);
 	} else if (transport == DC_TRANSPORT_BLE) {
 		return DC_FILTER_INTERNAL (userdata, bluetooth, 0, dc_match_prefix);
 	}
@@ -618,7 +716,8 @@ static int dc_filter_suunto (dc_transport_t transport, const void *userdata, voi
 	return 1;
 }
 
-static int dc_filter_hw (dc_transport_t transport, const void *userdata, void *params)
+static int
+dc_filter_hw (const dc_descriptor_t *descriptor, dc_transport_t transport, const void *userdata)
 {
 	static const char * const bluetooth[] = {
 		"OSTC",
@@ -627,14 +726,13 @@ static int dc_filter_hw (dc_transport_t transport, const void *userdata, void *p
 
 	if (transport == DC_TRANSPORT_BLUETOOTH || transport == DC_TRANSPORT_BLE) {
 		return DC_FILTER_INTERNAL (userdata, bluetooth, 0, dc_match_prefix);
-	} else if (transport == DC_TRANSPORT_SERIAL) {
-		return DC_FILTER_INTERNAL (userdata, rfcomm, 1, dc_match_devname);
 	}
 
 	return 1;
 }
 
-static int dc_filter_shearwater (dc_transport_t transport, const void *userdata, void *params)
+static int
+dc_filter_shearwater (const dc_descriptor_t *descriptor, dc_transport_t transport, const void *userdata)
 {
 	static const char * const bluetooth[] = {
 		"Predator",
@@ -646,18 +744,19 @@ static int dc_filter_shearwater (dc_transport_t transport, const void *userdata,
 		"Perdix 2",
 		"Teric",
 		"Peregrine",
+		"Peregrine TX",
+		"Tern"
 	};
 
 	if (transport == DC_TRANSPORT_BLUETOOTH || transport == DC_TRANSPORT_BLE) {
 		return DC_FILTER_INTERNAL (userdata, bluetooth, 0, dc_match_name);
-	} else if (transport == DC_TRANSPORT_SERIAL) {
-		return DC_FILTER_INTERNAL (userdata, rfcomm, 1, dc_match_devname);
 	}
 
 	return 1;
 }
 
-static int dc_filter_tecdiving (dc_transport_t transport, const void *userdata, void *params)
+static int
+dc_filter_tecdiving (const dc_descriptor_t *descriptor, dc_transport_t transport, const void *userdata)
 {
 	static const char * const bluetooth[] = {
 		"DiveComputer",
@@ -665,18 +764,21 @@ static int dc_filter_tecdiving (dc_transport_t transport, const void *userdata,
 
 	if (transport == DC_TRANSPORT_BLUETOOTH) {
 		return DC_FILTER_INTERNAL (userdata, bluetooth, 0, dc_match_name);
-	} else if (transport == DC_TRANSPORT_SERIAL) {
-		return DC_FILTER_INTERNAL (userdata, rfcomm, 1, dc_match_devname);
 	}
 
 	return 1;
 }
 
-static int dc_filter_mares (dc_transport_t transport, const void *userdata, void *params)
+static int
+dc_filter_mares (const dc_descriptor_t *descriptor, dc_transport_t transport, const void *userdata)
 {
 	static const char * const bluetooth[] = {
 		"Mares bluelink pro",
 		"Mares Genius",
+		"Sirius",
+		"Quad Ci",
+		"Puck4",
+		"Puck Lite",
 	};
 
 	if (transport == DC_TRANSPORT_BLE) {
@@ -686,21 +788,24 @@ static int dc_filter_mares (dc_transport_t transport, const void *userdata, void
 	return 1;
 }
 
-static int dc_filter_divesystem (dc_transport_t transport, const void *userdata, void *params)
+static int
+dc_filter_divesystem (const dc_descriptor_t *descriptor, dc_transport_t transport, const void *userdata)
 {
 	static const char * const bluetooth[] = {
 		"DS",
 		"IX5M",
+		"RATIO-",
 	};
 
 	if (transport == DC_TRANSPORT_BLUETOOTH || transport == DC_TRANSPORT_BLE) {
-		return DC_FILTER_INTERNAL (userdata, bluetooth, 0, dc_match_number_with_prefix);
+		return DC_FILTER_INTERNAL (userdata, bluetooth, 0, dc_match_prefix_with_number);
 	}
 
 	return 1;
 }
 
-static int dc_filter_oceanic (dc_transport_t transport, const void *userdata, void *params)
+static int
+dc_filter_oceanic (const dc_descriptor_t *descriptor, dc_transport_t transport, const void *userdata)
 {
 	static const unsigned int model[] = {
 		0x4552, // Oceanic Pro Plus X
@@ -714,10 +819,13 @@ static int dc_filter_oceanic (dc_transport_t transport, const void *userdata, vo
 		0x4654, // Oceanic Veo 4.0
 		0x4655, // Sherwood Wisdom 4
 		0x4656, // Oceanic Pro Plus 4
+		0x4741, // Apeks DSX
 		0x4742, // Sherwood Beacon
 		0x4743, // Aqualung i470TC
+		0x4744, // Aqualung i330R
 		0x4749, // Aqualung i200C
 		0x474B, // Oceanic Geo Air
+		0x474D, // Aqualung i330R Console
 	};
 
 	if (transport == DC_TRANSPORT_BLE) {
@@ -727,7 +835,8 @@ static int dc_filter_oceanic (dc_transport_t transport, const void *userdata, vo
 	return 1;
 }
 
-static int dc_filter_mclean(dc_transport_t transport, const void *userdata, void *params)
+static int
+dc_filter_mclean(const dc_descriptor_t *descriptor, dc_transport_t transport, const void *userdata)
 {
 	static const char * const bluetooth[] = {
 		"McLean Extreme",
@@ -735,37 +844,32 @@ static int dc_filter_mclean(dc_transport_t transport, const void *userdata, void
 
 	if (transport == DC_TRANSPORT_BLUETOOTH || transport == DC_TRANSPORT_BLE) {
 		return DC_FILTER_INTERNAL (userdata, bluetooth, 0, dc_match_name);
-	} else if (transport == DC_TRANSPORT_SERIAL) {
-		return DC_FILTER_INTERNAL(userdata, rfcomm, 1, dc_match_devname);
 	}
 
 	return 1;
 }
 
-static int dc_filter_atomic (dc_transport_t transport, const void *userdata, void *params)
+static int
+dc_filter_atomic (const dc_descriptor_t *descriptor, dc_transport_t transport, const void *userdata)
 {
 	static const dc_usb_desc_t usb[] = {
 		{0x0471, 0x0888}, // Atomic Aquatics Cobalt
 	};
 
-	static const dc_usb_params_t usb_params = {
-		0, 0x82, 0x02
-	};
-
 	if (transport == DC_TRANSPORT_USB) {
-		return DC_FILTER_INTERNAL_WITH_PARAMS (userdata, usb, 0, dc_match_usb, params, &usb_params);
+		return DC_FILTER_INTERNAL (userdata, usb, 0, dc_match_usb);
 	}
 
 	return 1;
 }
 
-static int dc_filter_deepsix (dc_transport_t transport, const void *userdata, void *params)
+static int
+dc_filter_deepsix (const dc_descriptor_t *descriptor, dc_transport_t transport, const void *userdata)
 {
 	static const char * const bluetooth[] = {
 		"EXCURSION",
 		"Crest-CR4",
 		"CENTAURI",
-		"TC1",
 		"ALPHA",
 	};
 
@@ -776,7 +880,8 @@ static int dc_filter_deepsix (dc_transport_t transport, const void *userdata, vo
 	return 1;
 }
 
-static int dc_filter_deepblu (dc_transport_t transport, const void *userdata, void *params)
+static int
+dc_filter_deepblu (const dc_descriptor_t *descriptor, dc_transport_t transport, const void *userdata)
 {
 	static const char * const bluetooth[] = {
 		"COSMIQ",
@@ -789,7 +894,8 @@ static int dc_filter_deepblu (dc_transport_t transport, const void *userdata, vo
 	return 1;
 }
 
-static int dc_filter_oceans (dc_transport_t transport, const void *userdata, void *params)
+static int
+dc_filter_oceans (const dc_descriptor_t *descriptor, dc_transport_t transport, const void *userdata)
 {
 	static const char * const bluetooth[] = {
 		"S1",
@@ -802,7 +908,8 @@ static int dc_filter_oceans (dc_transport_t transport, const void *userdata, voi
 	return 1;
 }
 
-static int dc_filter_divesoft (dc_transport_t transport, const void *userdata, void *params)
+static int
+dc_filter_divesoft (const dc_descriptor_t *descriptor, dc_transport_t transport, const void *userdata)
 {
 	static const char * const bluetooth[] = {
 		"Freedom",
@@ -816,8 +923,43 @@ static int dc_filter_divesoft (dc_transport_t transport, const void *userdata, v
 	return 1;
 }
 
+static int
+dc_filter_cressi (const dc_descriptor_t *descriptor, dc_transport_t transport, const void *userdata)
+{
+	static const unsigned int model[] = {
+		1,  // Cartesio
+		2,  // Goa
+		3,  // Leonardo 2.0
+		4,  // Donatello
+		5,  // Michelangelo
+		9,  // Neon
+		10, // Nepto
+	};
+
+	if (transport == DC_TRANSPORT_BLE) {
+		return DC_FILTER_INTERNAL (userdata, model, 0, dc_match_cressi);
+	}
+
+	return 1;
+}
+
+static int
+dc_filter_halcyon (const dc_descriptor_t *descriptor, dc_transport_t transport, const void *userdata)
+{
+	static const unsigned int model[] = {
+		1, // Symbios HUD
+		7, // Symbios Handset
+	};
+
+	if (transport == DC_TRANSPORT_BLE) {
+		return DC_FILTER_INTERNAL (userdata, model, 0, dc_match_halcyon);
+	}
+
+	return 1;
+}
+
 dc_status_t
-dc_descriptor_iterator (dc_iterator_t **out)
+dc_descriptor_iterator_new (dc_iterator_t **out, dc_context_t *context)
 {
 	dc_descriptor_iterator_t *iterator = NULL;
 
@@ -835,6 +977,12 @@ dc_descriptor_iterator (dc_iterator_t **out)
 	return DC_STATUS_SUCCESS;
 }
 
+dc_status_t
+dc_descriptor_iterator (dc_iterator_t **out)
+{
+	return dc_descriptor_iterator_new (out, NULL);
+}
+
 static dc_status_t
 dc_descriptor_iterator_next (dc_iterator_t *abstract, void *out)
 {
@@ -863,7 +1011,7 @@ dc_descriptor_free (dc_descriptor_t *descriptor)
 }
 
 const char *
-dc_descriptor_get_vendor (dc_descriptor_t *descriptor)
+dc_descriptor_get_vendor (const dc_descriptor_t *descriptor)
 {
 	if (descriptor == NULL)
 		return NULL;
@@ -872,7 +1020,7 @@ dc_descriptor_get_vendor (dc_descriptor_t *descriptor)
 }
 
 const char *
-dc_descriptor_get_product (dc_descriptor_t *descriptor)
+dc_descriptor_get_product (const dc_descriptor_t *descriptor)
 {
 	if (descriptor == NULL)
 		return NULL;
@@ -881,7 +1029,7 @@ dc_descriptor_get_product (dc_descriptor_t *descriptor)
 }
 
 dc_family_t
-dc_descriptor_get_type (dc_descriptor_t *descriptor)
+dc_descriptor_get_type (const dc_descriptor_t *descriptor)
 {
 	if (descriptor == NULL)
 		return DC_FAMILY_NULL;
@@ -890,7 +1038,7 @@ dc_descriptor_get_type (dc_descriptor_t *descriptor)
 }
 
 unsigned int
-dc_descriptor_get_model (dc_descriptor_t *descriptor)
+dc_descriptor_get_model (const dc_descriptor_t *descriptor)
 {
 	if (descriptor == NULL)
 		return 0;
@@ -899,7 +1047,7 @@ dc_descriptor_get_model (dc_descriptor_t *descriptor)
 }
 
 unsigned int
-dc_descriptor_get_transports (dc_descriptor_t *descriptor)
+dc_descriptor_get_transports (const dc_descriptor_t *descriptor)
 {
 	if (descriptor == NULL)
 		return DC_TRANSPORT_NONE;
@@ -908,10 +1056,16 @@ dc_descriptor_get_transports (dc_descriptor_t *descriptor)
 }
 
 int
-dc_descriptor_filter (dc_descriptor_t *descriptor, dc_transport_t transport, const void *userdata, void *params)
+dc_descriptor_filter (const dc_descriptor_t *descriptor, dc_transport_t transport, const void *userdata)
 {
-	if (descriptor == NULL || descriptor->filter == NULL || userdata == NULL)
+	if (descriptor == NULL)
 		return 1;
 
-	return descriptor->filter (transport, userdata, params);
+	if ((descriptor->transports & transport) == 0)
+		return 0;
+
+	if (descriptor->filter == NULL || userdata == NULL)
+		return 1;
+
+	return descriptor->filter (descriptor, transport, userdata);
 }
