@@ -38,6 +38,7 @@
 #include "oceanic_atom2.h"
 #include "oceanic_veo250.h"
 #include "oceanic_vtpro.h"
+#include "pelagic_i330r.h"
 #include "mares_darwin.h"
 #include "mares_iconhd.h"
 #include "mares_nemo.h"
@@ -65,6 +66,7 @@
 #include "deepblu_cosmiq.h"
 #include "oceans_s1.h"
 #include "divesoft_freedom.h"
+#include "halcyon_symbios.h"
 
 #include "device-private.h"
 #include "context-private.h"
@@ -162,6 +164,9 @@ dc_device_open (dc_device_t **out, dc_context_t *context, dc_descriptor_t *descr
 	case DC_FAMILY_OCEANIC_ATOM2:
 		rc = oceanic_atom2_device_open (&device, context, iostream, dc_descriptor_get_model (descriptor));
 		break;
+	case DC_FAMILY_PELAGIC_I330R:
+		rc = pelagic_i330r_device_open (&device, context, iostream, dc_descriptor_get_model (descriptor));
+		break;
 	case DC_FAMILY_MARES_NEMO:
 		rc = mares_nemo_device_open (&device, context, iostream);
 		break;
@@ -172,7 +177,7 @@ dc_device_open (dc_device_t **out, dc_context_t *context, dc_descriptor_t *descr
 		rc = mares_darwin_device_open (&device, context, iostream, dc_descriptor_get_model (descriptor));
 		break;
 	case DC_FAMILY_MARES_ICONHD:
-		rc = mares_iconhd_device_open (&device, context, iostream);
+		rc = mares_iconhd_device_open (&device, context, iostream, dc_descriptor_get_model (descriptor));
 		break;
 	case DC_FAMILY_HW_OSTC:
 		rc = hw_ostc_device_open (&device, context, iostream);
@@ -243,6 +248,9 @@ dc_device_open (dc_device_t **out, dc_context_t *context, dc_descriptor_t *descr
 	case DC_FAMILY_DIVESOFT_FREEDOM:
 		rc = divesoft_freedom_device_open (&device, context, iostream);
 		break;
+	case DC_FAMILY_HALCYON_SYMBIOS:
+		rc = halcyon_symbios_device_open (&device, context, iostream);
+		break;
 	default:
 		return DC_STATUS_INVALIDARGS;
 	}
@@ -309,6 +317,8 @@ dc_device_set_fingerprint (dc_device_t *device, const unsigned char data[], unsi
 	if (device->vtable->set_fingerprint == NULL)
 		return DC_STATUS_UNSUPPORTED;
 
+	HEXDUMP (device->context, DC_LOGLEVEL_INFO, "Fingerprint", data, size);
+
 	return device->vtable->set_fingerprint (device, data, size);
 }
 
