@@ -23,18 +23,20 @@
 #define DIVESYSTEM_IDIVE_H
 
 #include <libdivecomputer/context.h>
+#include <libdivecomputer/iostream.h>
 #include <libdivecomputer/device.h>
 #include <libdivecomputer/parser.h>
+#include <libdivecomputer/divesystem_idive.h>
 
 #ifdef __cplusplus
 extern "C" {
 #endif /* __cplusplus */
 
 dc_status_t
-divesystem_idive_device_open (dc_device_t **device, dc_context_t *context, const char *name, unsigned int model);
+divesystem_idive_device_open (dc_device_t **device, dc_context_t *context, dc_iostream_t *iostream, unsigned int model);
 
 dc_status_t
-divesystem_idive_parser_create (dc_parser_t **parser, dc_context_t *context, unsigned int model);
+divesystem_idive_parser_create (dc_parser_t **parser, dc_context_t *context, const unsigned char data[], size_t size, unsigned int model);
 
 #ifdef __cplusplus
 }
