@@ -58,6 +58,9 @@
 #define IX3M2_ZHL16C   2
 #define IX3M2_VPM      3
 
+#define REC_SAMPLE 0
+#define REC_INFO   1
+
 typedef struct divesystem_idive_parser_t divesystem_idive_parser_t;
 
 typedef struct divesystem_idive_gasmix_t {
@@ -87,9 +90,12 @@ struct divesystem_idive_parser_t {
 	unsigned int algorithm;
 	unsigned int gf_low;
 	unsigned int gf_high;
+	unsigned int have_location;
+	int latitude;
+	int longitude;
+	int altitude;
 };
 
-static dc_status_t divesystem_idive_parser_set_data (dc_parser_t *abstract, const unsigned char *data, unsigned int size);
 static dc_status_t divesystem_idive_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *datetime);
 static dc_status_t divesystem_idive_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsigned int flags, void *value);
 static dc_status_t divesystem_idive_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t callback, void *userdata);
@@ -97,7 +103,6 @@ static dc_status_t divesystem_idive_parser_samples_foreach (dc_parser_t *abstrac
 static const dc_parser_vtable_t divesystem_idive_parser_vtable = {
 	sizeof(divesystem_idive_parser_t),
 	DC_FAMILY_DIVESYSTEM_IDIVE,
-	divesystem_idive_parser_set_data, /* set_data */
 	NULL, /* set_clock */
 	NULL, /* set_atmospheric */
 	NULL, /* set_density */
@@ -109,7 +114,7 @@ static const dc_parser_vtable_t divesystem_idive_parser_vtable = {
 
 
 dc_status_t
-divesystem_idive_parser_create (dc_parser_t **out, dc_context_t *context, unsigned int model)
+divesystem_idive_parser_create (dc_parser_t **out, dc_context_t *context, const unsigned char data[], size_t size, unsigned int model)
 {
 	divesystem_idive_parser_t *parser = NULL;
 
@@ -117,7 +122,7 @@ divesystem_idive_parser_create (dc_parser_t **out, dc_context_t *context, unsign
 		return DC_STATUS_INVALIDARGS;
 
 	// Allocate memory.
-	parser = (divesystem_idive_parser_t *) dc_parser_allocate (context, &divesystem_idive_parser_vtable);
+	parser = (divesystem_idive_parser_t *) dc_parser_allocate (context, &divesystem_idive_parser_vtable, data, size);
 	if (parser == NULL) {
 		ERROR (context, "Failed to allocate memory.");
 		return DC_STATUS_NOMEMORY;
@@ -148,7 +153,10 @@ divesystem_idive_parser_create (dc_parser_t **out, dc_context_t *context, unsign
 	parser->algorithm = INVALID;
 	parser->gf_low = INVALID;
 	parser->gf_high = INVALID;
-
+	parser->have_location = 0;
+	parser->latitude = 0;
+	parser->longitude = 0;
+	parser->altitude = 0;
 
 	*out = (dc_parser_t*) parser;
 
@@ -156,35 +164,6 @@ divesystem_idive_parser_create (dc_parser_t **out, dc_context_t *context, unsign
 }
 
 
-static dc_status_t
-divesystem_idive_parser_set_data (dc_parser_t *abstract, const unsigned char *data, unsigned int size)
-{
-	divesystem_idive_parser_t *parser = (divesystem_idive_parser_t *) abstract;
-
-	// Reset the cache.
-	parser->cached = 0;
-	parser->divemode = INVALID;
-	parser->divetime = 0;
-	parser->maxdepth = 0;
-	parser->ngasmixes = 0;
-	parser->ntanks = 0;
-	for (unsigned int i = 0; i < NGASMIXES; ++i) {
-		parser->gasmix[i].oxygen = 0;
-		parser->gasmix[i].helium = 0;
-	}
-	for (unsigned int i = 0; i < NTANKS; ++i) {
-		parser->tank[i].id = 0;
-		parser->tank[i].beginpressure = 0;
-		parser->tank[i].endpressure = 0;
-	}
-	parser->algorithm = INVALID;
-	parser->gf_low = INVALID;
-	parser->gf_high = INVALID;
-
-	return DC_STATUS_SUCCESS;
-}
-
-
 static dc_status_t
 divesystem_idive_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *datetime)
 {
@@ -304,6 +283,7 @@ divesystem_idive_parser_get_field (dc_parser_t *abstract, dc_field_type_t type,
 	dc_tank_t *tank = (dc_tank_t *) value;
 	dc_salinity_t *water = (dc_salinity_t *) value;
 	dc_decomodel_t *decomodel = (dc_decomodel_t *) value;
+	dc_location_t *location = (dc_location_t *) value;
 
 	if (value) {
 		switch (type) {
@@ -317,6 +297,7 @@ divesystem_idive_parser_get_field (dc_parser_t *abstract, dc_field_type_t type,
 			*((unsigned int *) value) = parser->ngasmixes;
 			break;
 		case DC_FIELD_GASMIX:
+			gasmix->usage = DC_USAGE_NONE;
 			gasmix->helium = parser->gasmix[flags].helium / 100.0;
 			gasmix->oxygen = parser->gasmix[flags].oxygen / 100.0;
 			gasmix->nitrogen = 1.0 - gasmix->oxygen - gasmix->helium;
@@ -331,6 +312,7 @@ divesystem_idive_parser_get_field (dc_parser_t *abstract, dc_field_type_t type,
 			tank->beginpressure = parser->tank[flags].beginpressure;
 			tank->endpressure   = parser->tank[flags].endpressure;
 			tank->gasmix = DC_GASMIX_UNKNOWN;
+			tank->usage = DC_USAGE_NONE;
 			break;
 		case DC_FIELD_ATMOSPHERIC:
 			if (ISIX3M(parser->model)) {
@@ -407,6 +389,13 @@ divesystem_idive_parser_get_field (dc_parser_t *abstract, dc_field_type_t type,
 				}
 			}
 			break;
+		case DC_FIELD_LOCATION:
+			if (!parser->have_location)
+				return DC_STATUS_UNSUPPORTED;
+			location->latitude  = parser->latitude  / 10000000.0;
+			location->longitude = parser->longitude / 10000000.0;
+			location->altitude  = parser->altitude  / 1000.0;
+			break;
 		default:
 			return DC_STATUS_UNSUPPORTED;
 		}
@@ -439,6 +428,7 @@ divesystem_idive_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callba
 	unsigned int algorithm_previous = INVALID;
 	unsigned int gf_low = INVALID;
 	unsigned int gf_high = INVALID;
+	unsigned int have_bearing = 0;
 
 	unsigned int firmware = 0;
 	unsigned int apos4 = 0;
@@ -463,31 +453,55 @@ divesystem_idive_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callba
 		firmware = array_uint32_le(data + 0x2E);
 	}
 
+	unsigned int have_location = 0;
+	int altitude = 0, longitude = 0, latitude = 0;
+
 	unsigned int offset = parser->headersize;
 	while (offset + samplesize <= size) {
 		dc_sample_value_t sample = {0};
 
+		// Get the record type.
+		unsigned int type = ISIX3M(parser->model) ?
+			array_uint16_le (data + offset + 52) :
+			REC_SAMPLE;
+		if (type != REC_SAMPLE) {
+			if (type == REC_INFO) {
+				if (!have_location) {
+					altitude  = (signed int) array_uint32_le (data + offset + 40);
+					longitude = (signed int) array_uint32_le (data + offset + 44);
+					latitude  = (signed int) array_uint32_le (data + offset + 48);
+					have_location = 1;
+				} else {
+					WARNING (abstract->context, "Multiple GPS locations present.");
+				}
+			}
+
+			// Skip non-sample records.
+			offset += samplesize;
+			continue;
+		}
+
 		// Time (seconds).
 		unsigned int timestamp = array_uint32_le (data + offset + 2);
-		if (timestamp <= time) {
-			ERROR (abstract->context, "Timestamp moved backwards.");
+		if (timestamp <= time && time != 0) {
+			ERROR (abstract->context, "Timestamp moved backwards (%u %u).", timestamp, time);
 			return DC_STATUS_DATAFORMAT;
 		}
 		time = timestamp;
-		sample.time = timestamp;
-		if (callback) callback (DC_SAMPLE_TIME, sample, userdata);
+		sample.time = timestamp * 1000;
+		if (callback) callback (DC_SAMPLE_TIME, &sample, userdata);
 
 		// Depth (1/10 m).
 		unsigned int depth = array_uint16_le (data + offset + 6);
 		if (maxdepth < depth)
 			maxdepth = depth;
 		sample.depth = depth / 10.0;
-		if (callback) callback (DC_SAMPLE_DEPTH, sample, userdata);
+		if (callback) callback (DC_SAMPLE_DEPTH, &sample, userdata);
 
 		// Temperature (Celsius).
 		signed int temperature = (signed short) array_uint16_le (data + offset + 8);
 		sample.temperature = temperature / 10.0;
-		if (callback) callback (DC_SAMPLE_TEMPERATURE, sample, userdata);
+		if (callback) callback (DC_SAMPLE_TEMPERATURE, &sample, userdata);
 
 		// Dive mode
 		unsigned int mode = data[offset + 18];
@@ -521,7 +535,7 @@ divesystem_idive_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callba
 		if (mode == SCR || mode == CCR) {
 			unsigned int setpoint = array_uint16_le (data + offset + 19);
 			sample.setpoint = setpoint / 1000.0;
-			if (callback) callback (DC_SAMPLE_SETPOINT, sample, userdata);
+			if (callback) callback (DC_SAMPLE_SETPOINT, &sample, userdata);
 		}
 
 		// Gaschange.
@@ -548,7 +562,7 @@ divesystem_idive_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callba
 			}
 
 			sample.gasmix = i;
-			if (callback) callback (DC_SAMPLE_GASMIX, sample, userdata);
+			if (callback) callback (DC_SAMPLE_GASMIX, &sample, userdata);
 			o2_previous = o2;
 			he_previous = he;
 		}
@@ -566,18 +580,20 @@ divesystem_idive_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callba
 		if (decostop) {
 			sample.deco.type = DC_DECO_DECOSTOP;
 			sample.deco.depth = decostop / 10.0;
-			sample.deco.time = apos4 ? decotime : tts;
+			sample.deco.time = decotime;
+			sample.deco.tts = tts;
 		} else {
 			sample.deco.type = DC_DECO_NDL;
 			sample.deco.depth = 0.0;
 			sample.deco.time = tts;
+			sample.deco.tts = 0;
 		}
-		if (callback) callback (DC_SAMPLE_DECO, sample, userdata);
+		if (callback) callback (DC_SAMPLE_DECO, &sample, userdata);
 
 		// CNS
 		unsigned int cns = array_uint16_le (data + offset + 29);
 		sample.cns = cns / 100.0;
-		if (callback) callback (DC_SAMPLE_CNS, sample, userdata);
+		if (callback) callback (DC_SAMPLE_CNS, &sample, userdata);
 
 		// Tank Pressure
 		if (samplesize == SZ_SAMPLE_IX3M_APOS4) {
@@ -598,7 +614,7 @@ divesystem_idive_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callba
 				sample.event.time = 0;
 				sample.event.flags = 0;
 				sample.event.value = 0;
-				if (callback) callback (DC_SAMPLE_EVENT, sample, userdata);
+				if (callback) callback (DC_SAMPLE_EVENT, &sample, userdata);
 			} else {
 				// Get the index of the tank.
 				if (id != tank_previous) {
@@ -628,10 +644,20 @@ divesystem_idive_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callba
 				if (tank_idx < ntanks) {
 					sample.pressure.tank = tank_idx;
 					sample.pressure.value = pressure;
-					if (callback) callback (DC_SAMPLE_PRESSURE, sample, userdata);
+					if (callback) callback (DC_SAMPLE_PRESSURE, &sample, userdata);
 					tank[tank_idx].endpressure = pressure;
 				}
 			}
+
+			// Compass bearing
+			unsigned int bearing = array_uint16_le (data + offset + 50);
+			if (bearing != 0) {
+				have_bearing = 1; // Stop ignoring zero values.
+			}
+			if (have_bearing && bearing != 0xFFFF) {
+				sample.bearing = bearing;
+				if (callback) callback (DC_SAMPLE_BEARING, &sample, userdata);
+			}
 		}
 
 		offset += samplesize;
@@ -652,6 +678,10 @@ divesystem_idive_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callba
 	parser->algorithm = algorithm;
 	parser->gf_low = gf_low;
 	parser->gf_high = gf_high;
+	parser->have_location = have_location;
+	parser->latitude = latitude;
+	parser->longitude = longitude;
+	parser->altitude = altitude;
 	parser->cached = 1;
 
 	return DC_STATUS_SUCCESS;
