@@ -25,15 +25,12 @@
 #include "hw_ostc.h"
 #include "context-private.h"
 #include "device-private.h"
-#include "serial.h"
 #include "checksum.h"
 #include "array.h"
 #include "ihex.h"
 
 #define ISINSTANCE(device) dc_device_isinstance((device), &hw_ostc_device_vtable)
 
-#define C_ARRAY_SIZE(a) (sizeof(a) / sizeof(*(a)))
-
 #define MAXRETRIES 9
 
 #define FW_190 0x015A
@@ -58,7 +55,7 @@
 
 typedef struct hw_ostc_device_t {
 	dc_device_t base;
-	dc_serial_t *port;
+	dc_iostream_t *iostream;
 	unsigned char fingerprint[5];
 } hw_ostc_device_t;
 
@@ -71,7 +68,6 @@ static dc_status_t hw_ostc_device_set_fingerprint (dc_device_t *abstract, const
 static dc_status_t hw_ostc_device_dump (dc_device_t *abstract, dc_buffer_t *buffer);
 static dc_status_t hw_ostc_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback, void *userdata);
 static dc_status_t hw_ostc_device_timesync (dc_device_t *abstract, const dc_datetime_t *datetime);
-static dc_status_t hw_ostc_device_close (dc_device_t *abstract);
 
 static const dc_device_vtable_t hw_ostc_device_vtable = {
 	sizeof(hw_ostc_device_t),
@@ -82,7 +78,7 @@ static const dc_device_vtable_t hw_ostc_device_vtable = {
 	hw_ostc_device_dump, /* dump */
 	hw_ostc_device_foreach, /* foreach */
 	hw_ostc_device_timesync, /* timesync */
-	hw_ostc_device_close /* close */
+	NULL /* close */
 };
 
 static dc_status_t
@@ -96,7 +92,7 @@ hw_ostc_send (hw_ostc_device_t *device, unsigned char cmd, unsigned int echo)
 
 	// Send the command.
 	unsigned char command[1] = {cmd};
-	status = dc_serial_write (device->port, command, sizeof (command), NULL);
+	status = dc_iostream_write (device->iostream, command, sizeof (command), NULL);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to send the command.");
 		return status;
@@ -105,7 +101,7 @@ hw_ostc_send (hw_ostc_device_t *device, unsigned char cmd, unsigned int echo)
 	if (echo) {
 		// Read the echo.
 		unsigned char answer[1] = {0};
-		status = dc_serial_read (device->port, answer, sizeof (answer), NULL);
+		status = dc_iostream_read (device->iostream, answer, sizeof (answer), NULL);
 		if (status != DC_STATUS_SUCCESS) {
 			ERROR (abstract->context, "Failed to receive the echo.");
 			return status;
@@ -123,7 +119,7 @@ hw_ostc_send (hw_ostc_device_t *device, unsigned char cmd, unsigned int echo)
 
 
 dc_status_t
-hw_ostc_device_open (dc_device_t **out, dc_context_t *context, const char *name)
+hw_ostc_device_open (dc_device_t **out, dc_context_t *context, dc_iostream_t *iostream)
 {
 	dc_status_t status = DC_STATUS_SUCCESS;
 	hw_ostc_device_t *device = NULL;
@@ -139,63 +135,37 @@ hw_ostc_device_open (dc_device_t **out, dc_context_t *context, const char *name)
 	}
 
 	// Set the default values.
-	device->port = NULL;
+	device->iostream = iostream;
 	memset (device->fingerprint, 0, sizeof (device->fingerprint));
 
-	// Open the device.
-	status = dc_serial_open (&device->port, context, name);
+	// Set the serial communication protocol (115200 8N1).
+	status = dc_iostream_configure (device->iostream, 115200, 8, DC_PARITY_NONE, DC_STOPBITS_ONE, DC_FLOWCONTROL_NONE);
 	if (status != DC_STATUS_SUCCESS) {
-		ERROR (context, "Failed to open the serial port.");
+		ERROR (context, "Failed to set the terminal attributes.");
 		goto error_free;
 	}
 
-	// Set the serial communication protocol (115200 8N1).
-	status = dc_serial_configure (device->port, 115200, 8, DC_PARITY_NONE, DC_STOPBITS_ONE, DC_FLOWCONTROL_NONE);
-	if (status != DC_STATUS_SUCCESS) {
-		ERROR (context, "Failed to set the terminal attributes.");
-		goto error_close;
-	}
-
 	// Set the timeout for receiving data.
-	status = dc_serial_set_timeout (device->port, 4000);
+	status = dc_iostream_set_timeout (device->iostream, 4000);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the timeout.");
-		goto error_close;
+		goto error_free;
 	}
 
 	// Make sure everything is in a sane state.
-	dc_serial_sleep (device->port, 100);
-	dc_serial_purge (device->port, DC_DIRECTION_ALL);
+	dc_iostream_sleep (device->iostream, 100);
+	dc_iostream_purge (device->iostream, DC_DIRECTION_ALL);
 
 	*out = (dc_device_t*) device;
 
 	return DC_STATUS_SUCCESS;
 
-error_close:
-	dc_serial_close (device->port);
 error_free:
 	dc_device_deallocate ((dc_device_t *) device);
 	return status;
 }
 
 
-static dc_status_t
-hw_ostc_device_close (dc_device_t *abstract)
-{
-	dc_status_t status = DC_STATUS_SUCCESS;
-	hw_ostc_device_t *device = (hw_ostc_device_t*) abstract;
-	dc_status_t rc = DC_STATUS_SUCCESS;
-
-	// Close the device.
-	rc = dc_serial_close (device->port);
-	if (rc != DC_STATUS_SUCCESS) {
-		dc_status_set_error(&status, rc);
-	}
-
-	return status;
-}
-
-
 static dc_status_t
 hw_ostc_device_set_fingerprint (dc_device_t *abstract, const unsigned char data[], unsigned int size)
 {
@@ -219,12 +189,6 @@ hw_ostc_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 	dc_status_t status = DC_STATUS_SUCCESS;
 	hw_ostc_device_t *device = (hw_ostc_device_t*) abstract;
 
-	// Erase the current contents of the buffer.
-	if (!dc_buffer_clear (buffer)) {
-		ERROR (abstract->context, "Insufficient buffer space available.");
-		return DC_STATUS_NOMEMORY;
-	}
-
 	// Enable progress notifications.
 	dc_event_progress_t progress = EVENT_PROGRESS_INITIALIZER;
 	progress.maximum = SZ_HEADER + SZ_FW_NEW;
@@ -232,7 +196,7 @@ hw_ostc_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 
 	// Send the command.
 	unsigned char command[1] = {'a'};
-	status = dc_serial_write (device->port, command, sizeof (command), NULL);
+	status = dc_iostream_write (device->iostream, command, sizeof (command), NULL);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to send the command.");
 		return status;
@@ -240,7 +204,7 @@ hw_ostc_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 
 	// Read the header.
 	unsigned char header[SZ_HEADER] = {0};
-	status = dc_serial_read (device->port, header, sizeof (header), NULL);
+	status = dc_iostream_read (device->iostream, header, sizeof (header), NULL);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to receive the header.");
 		return status;
@@ -286,7 +250,7 @@ hw_ostc_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 
 		// Increase the packet size if more data is immediately available.
 		size_t available = 0;
-		status = dc_serial_get_available (device->port, &available);
+		status = dc_iostream_get_available (device->iostream, &available);
 		if (status == DC_STATUS_SUCCESS && available > len)
 			len = available;
 
@@ -295,7 +259,7 @@ hw_ostc_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 			len = size - nbytes;
 
 		// Read the packet.
-		status = dc_serial_read (device->port, data + nbytes, len, NULL);
+		status = dc_iostream_read (device->iostream, data + nbytes, len, NULL);
 		if (status != DC_STATUS_SUCCESS) {
 			ERROR (abstract->context, "Failed to receive the answer.");
 			return status;
@@ -308,6 +272,20 @@ hw_ostc_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 		nbytes += len;
 	}
 
+	// Emit a device info event.
+	dc_event_devinfo_t devinfo;
+	devinfo.firmware = array_uint16_be (data + 264);
+	devinfo.serial = array_uint16_le (data + 6);
+	if (devinfo.serial > 7000)
+		devinfo.model = 3; // OSTC 2C
+	else if (devinfo.serial > 2048)
+		devinfo.model = 2; // OSTC 2N
+	else if (devinfo.serial > 300)
+		devinfo.model = 1; // OSTC Mk2
+	else
+		devinfo.model = 0; // OSTC
+	device_event_emit (abstract, DC_EVENT_DEVINFO, &devinfo);
+
 	return DC_STATUS_SUCCESS;
 }
 
@@ -325,21 +303,6 @@ hw_ostc_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback, void
 		return rc;
 	}
 
-	// Emit a device info event.
-	unsigned char *data = dc_buffer_get_data (buffer);
-	dc_event_devinfo_t devinfo;
-	devinfo.firmware = array_uint16_be (data + 264);
-	devinfo.serial = array_uint16_le (data + 6);
-	if (devinfo.serial > 7000)
-		devinfo.model = 3; // OSTC 2C
-	else if (devinfo.serial > 2048)
-		devinfo.model = 2; // OSTC 2N
-	else if (devinfo.serial > 300)
-		devinfo.model = 1; // OSTC Mk2
-	else
-		devinfo.model = 0; // OSTC
-	device_event_emit (abstract, DC_EVENT_DEVINFO, &devinfo);
-
 	rc = hw_ostc_extract_dives (abstract, dc_buffer_get_data (buffer),
 		dc_buffer_get_size (buffer), callback, userdata);
 
@@ -369,7 +332,7 @@ hw_ostc_device_md2hash (dc_device_t *abstract, unsigned char data[], unsigned in
 		return rc;
 
 	// Read the answer.
-	status = dc_serial_read (device->port, data, SZ_MD2HASH, NULL);
+	status = dc_iostream_read (device->iostream, data, SZ_MD2HASH, NULL);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to receive the answer.");
 		return status;
@@ -385,11 +348,6 @@ hw_ostc_device_timesync (dc_device_t *abstract, const dc_datetime_t *datetime)
 	dc_status_t status = DC_STATUS_SUCCESS;
 	hw_ostc_device_t *device = (hw_ostc_device_t *) abstract;
 
-	if (datetime == NULL) {
-		ERROR (abstract->context, "Invalid parameter specified.");
-		return DC_STATUS_INVALIDARGS;
-	}
-
 	// Send the command.
 	dc_status_t rc = hw_ostc_send (device, 'b', 1);
 	if (rc != DC_STATUS_SUCCESS)
@@ -399,7 +357,7 @@ hw_ostc_device_timesync (dc_device_t *abstract, const dc_datetime_t *datetime)
 	unsigned char packet[6] = {
 		datetime->hour, datetime->minute, datetime->second,
 		datetime->month, datetime->day, datetime->year - 2000};
-	status = dc_serial_write (device->port, packet, sizeof (packet), NULL);
+	status = dc_iostream_write (device->iostream, packet, sizeof (packet), NULL);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to send the data packet.");
 		return status;
@@ -435,7 +393,7 @@ hw_ostc_device_eeprom_read (dc_device_t *abstract, unsigned int bank, unsigned c
 		return rc;
 
 	// Read the answer.
-	status = dc_serial_read (device->port, data, SZ_EEPROM, NULL);
+	status = dc_iostream_read (device->iostream, data, SZ_EEPROM, NULL);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to receive the answer.");
 		return status;
@@ -556,7 +514,7 @@ hw_ostc_device_screenshot (dc_device_t *abstract, dc_buffer_t *buffer, hw_ostc_f
 	unsigned int npixels = 0;
 	while (npixels < WIDTH * HEIGHT) {
 		unsigned char raw[3] = {0};
-		status = dc_serial_read (device->port, raw, 1, NULL);
+		status = dc_iostream_read (device->iostream, raw, 1, NULL);
 		if (status != DC_STATUS_SUCCESS) {
 			ERROR (abstract->context, "Failed to receive the packet.");
 			return status;
@@ -574,7 +532,7 @@ hw_ostc_device_screenshot (dc_device_t *abstract, dc_buffer_t *buffer, hw_ostc_f
 			count &= 0x3F;
 		} else {
 			// Color pixel.
-			status = dc_serial_read (device->port, raw + 1, 2, NULL);
+			status = dc_iostream_read (device->iostream, raw + 1, 2, NULL);
 			if (status != DC_STATUS_SUCCESS) {
 				ERROR (abstract->context, "Failed to receive the packet.");
 				return status;
@@ -593,7 +551,10 @@ hw_ostc_device_screenshot (dc_device_t *abstract, dc_buffer_t *buffer, hw_ostc_f
 
 		if (format == HW_OSTC_FORMAT_RAW) {
 			// Append the raw data to the output buffer.
-			dc_buffer_append (buffer, raw, nbytes);
+			if (!dc_buffer_append (buffer, raw, nbytes)) {
+				ERROR (abstract->context, "Insufficient buffer space available.");
+				return DC_STATUS_NOMEMORY;
+			}
 		} else {
 			// Store the decompressed data in the output buffer.
 			for (unsigned int i = 0; i < count; ++i) {
@@ -774,7 +735,7 @@ hw_ostc_firmware_setup_internal (hw_ostc_device_t *device)
 
 	// Send the command.
 	unsigned char command[1] = {0xC1};
-	status = dc_serial_write (device->port, command, sizeof (command), NULL);
+	status = dc_iostream_write (device->iostream, command, sizeof (command), NULL);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to send the command.");
 		return status;
@@ -782,7 +743,7 @@ hw_ostc_firmware_setup_internal (hw_ostc_device_t *device)
 
 	// Read the response.
 	unsigned char answer[2] = {0};
-	status = dc_serial_read (device->port, answer, sizeof (answer), NULL);
+	status = dc_iostream_read (device->iostream, answer, sizeof (answer), NULL);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to receive the response.");
 		return status;
@@ -825,7 +786,7 @@ hw_ostc_firmware_write_internal (hw_ostc_device_t *device, unsigned char *data,
 	dc_device_t *abstract = (dc_device_t *) device;
 
 	// Send the packet.
-	status = dc_serial_write (device->port, data, size, NULL);
+	status = dc_iostream_write (device->iostream, data, size, NULL);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to send the packet.");
 		return status;
@@ -833,7 +794,7 @@ hw_ostc_firmware_write_internal (hw_ostc_device_t *device, unsigned char *data,
 
 	// Read the response.
 	unsigned char answer[1] = {0};
-	status = dc_serial_read (device->port, answer, sizeof (answer), NULL);
+	status = dc_iostream_read (device->iostream, answer, sizeof (answer), NULL);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to receive the response.");
 		return status;
@@ -905,13 +866,18 @@ hw_ostc_device_fwupdate (dc_device_t *abstract, const char *filename)
 	// bootloader needs to be send repeatedly, until the response packet is
 	// received. Thus the time between each two attempts is directly controlled
 	// by the timeout value.
-	dc_serial_set_timeout (device->port, 300);
+	rc = dc_iostream_set_timeout (device->iostream, 300);
+	if (rc != DC_STATUS_SUCCESS) {
+		ERROR (context, "Failed to set the timeout.");
+		free (firmware);
+		return rc;
+	}
 
 	// Setup the bootloader.
 	const unsigned int baudrates[] = {19200, 115200};
 	for (unsigned int i = 0; i < C_ARRAY_SIZE(baudrates); ++i) {
 		// Adjust the baudrate.
-		rc = dc_serial_configure (device->port, baudrates[i], 8, DC_PARITY_NONE, DC_STOPBITS_ONE, DC_FLOWCONTROL_NONE);
+		rc = dc_iostream_configure (device->iostream, baudrates[i], 8, DC_PARITY_NONE, DC_STOPBITS_ONE, DC_FLOWCONTROL_NONE);
 		if (rc != DC_STATUS_SUCCESS) {
 			ERROR (abstract->context, "Failed to set the terminal attributes.");
 			free (firmware);
@@ -931,7 +897,12 @@ hw_ostc_device_fwupdate (dc_device_t *abstract, const char *filename)
 	}
 
 	// Increase the timeout again.
-	dc_serial_set_timeout (device->port, 1000);
+	rc = dc_iostream_set_timeout (device->iostream, 1000);
+	if (rc != DC_STATUS_SUCCESS) {
+		ERROR (context, "Failed to set the timeout.");
+		free (firmware);
+		return rc;
+	}
 
 	// Enable progress notifications.
 	dc_event_progress_t progress = EVENT_PROGRESS_INITIALIZER;
