@@ -240,7 +240,7 @@ hw_ostc3_write (hw_ostc3_device_t *device, dc_event_progress_t *progress, const
 	size_t nbytes = 0;
 	while (nbytes < size) {
 		// Set the maximum packet size.
-		size_t length = 1024;
+		size_t length = (device->hardware == OSTC4) ? 64 : 1024;
 
 		// Limit the packet size to the total size.
 		if (nbytes + length > size)
@@ -605,6 +605,8 @@ hw_ostc3_device_init (hw_ostc3_device_t *device, hw_ostc3_state_t state)
 		return rc;
 	}
 
+	HEXDUMP (abstract->context, DC_LOGLEVEL_DEBUG, "Hardware", hardware, sizeof(hardware));
+
 	// Read the version information.
 	unsigned char version[SZ_VERSION] = {0};
 	rc = hw_ostc3_transfer (device, NULL, IDENTITY, NULL, 0, version, sizeof(version), NULL, NODELAY);
@@ -613,6 +615,8 @@ hw_ostc3_device_init (hw_ostc3_device_t *device, hw_ostc3_state_t state)
 		return rc;
 	}
 
+	HEXDUMP (abstract->context, DC_LOGLEVEL_DEBUG, "Version", version, sizeof(version));
+
 	// Cache the descriptor.
 	device->hardware = array_uint16_be(hardware + 0);
 	device->feature = array_uint16_be(hardware + 2);
@@ -1715,6 +1719,10 @@ hw_ostc3_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 		return rc;
 	}
 
+	if (device->hardware == OSTC4) {
+		return DC_STATUS_UNSUPPORTED;
+	}
+
 	// Emit a device info event.
 	dc_event_devinfo_t devinfo;
 	devinfo.firmware = device->firmware;
