@@ -60,7 +60,6 @@ typedef struct mares_darwin_device_t {
 static dc_status_t mares_darwin_device_set_fingerprint (dc_device_t *abstract, const unsigned char data[], unsigned int size);
 static dc_status_t mares_darwin_device_dump (dc_device_t *abstract, dc_buffer_t *buffer);
 static dc_status_t mares_darwin_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback, void *userdata);
-static dc_status_t mares_darwin_device_close (dc_device_t *abstract);
 
 static const dc_device_vtable_t mares_darwin_device_vtable = {
 	sizeof(mares_darwin_device_t),
@@ -71,7 +70,7 @@ static const dc_device_vtable_t mares_darwin_device_vtable = {
 	mares_darwin_device_dump, /* dump */
 	mares_darwin_device_foreach, /* foreach */
 	NULL, /* timesync */
-	mares_darwin_device_close /* close */
+	NULL /* close */
 };
 
 static const mares_darwin_layout_t mares_darwin_layout = {
@@ -98,7 +97,7 @@ static dc_status_t
 mares_darwin_extract_dives (dc_device_t *device, const unsigned char data[], unsigned int size, dc_dive_callback_t callback, void *userdata);
 
 dc_status_t
-mares_darwin_device_open (dc_device_t **out, dc_context_t *context, const char *name, unsigned int model)
+mares_darwin_device_open (dc_device_t **out, dc_context_t *context, dc_iostream_t *iostream, unsigned int model)
 {
 	dc_status_t status = DC_STATUS_SUCCESS;
 	mares_darwin_device_t *device = NULL;
@@ -114,7 +113,7 @@ mares_darwin_device_open (dc_device_t **out, dc_context_t *context, const char *
 	}
 
 	// Initialize the base class.
-	mares_common_device_init (&device->base);
+	mares_common_device_init (&device->base, iostream);
 
 	// Set the default values.
 	memset (device->fingerprint, 0, sizeof (device->fingerprint));
@@ -124,44 +123,37 @@ mares_darwin_device_open (dc_device_t **out, dc_context_t *context, const char *
 	else
 		device->layout = &mares_darwin_layout;
 
-	// Open the device.
-	status = dc_serial_open (&device->base.port, context, name);
+	// Set the serial communication protocol (9600 8N1).
+	status = dc_iostream_configure (device->base.iostream, 9600, 8, DC_PARITY_NONE, DC_STOPBITS_ONE, DC_FLOWCONTROL_NONE);
 	if (status != DC_STATUS_SUCCESS) {
-		ERROR (context, "Failed to open the serial port.");
+		ERROR (context, "Failed to set the terminal attributes.");
 		goto error_free;
 	}
 
-	// Set the serial communication protocol (9600 8N1).
-	status = dc_serial_configure (device->base.port, 9600, 8, DC_PARITY_NONE, DC_STOPBITS_ONE, DC_FLOWCONTROL_NONE);
-	if (status != DC_STATUS_SUCCESS) {
-		ERROR (context, "Failed to set the terminal attributes.");
-		goto error_close;
-	}
-
 	// Set the timeout for receiving data (1000 ms).
-	status = dc_serial_set_timeout (device->base.port, 1000);
+	status = dc_iostream_set_timeout (device->base.iostream, 1000);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the timeout.");
-		goto error_close;
+		goto error_free;
 	}
 
 	// Set the DTR line.
-	status = dc_serial_set_dtr (device->base.port, 1);
+	status = dc_iostream_set_dtr (device->base.iostream, 1);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the DTR line.");
-		goto error_close;
+		goto error_free;
 	}
 
 	// Set the RTS line.
-	status = dc_serial_set_rts (device->base.port, 1);
+	status = dc_iostream_set_rts (device->base.iostream, 1);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the RTS line.");
-		goto error_close;
+		goto error_free;
 	}
 
 	// Make sure everything is in a sane state.
-	dc_serial_sleep (device->base.port, 100);
-	dc_serial_purge (device->base.port, DC_DIRECTION_ALL);
+	dc_iostream_sleep (device->base.iostream, 100);
+	dc_iostream_purge (device->base.iostream, DC_DIRECTION_ALL);
 
 	// Override the base class values.
 	device->base.echo = 1;
@@ -171,29 +163,11 @@ mares_darwin_device_open (dc_device_t **out, dc_context_t *context, const char *
 
 	return DC_STATUS_SUCCESS;
 
-error_close:
-	dc_serial_close (device->base.port);
 error_free:
 	dc_device_deallocate ((dc_device_t *) device);
 	return status;
 }
 
-static dc_status_t
-mares_darwin_device_close (dc_device_t *abstract)
-{
-	dc_status_t status = DC_STATUS_SUCCESS;
-	mares_darwin_device_t *device = (mares_darwin_device_t *) abstract;
-	dc_status_t rc = DC_STATUS_SUCCESS;
-
-	// Close the device.
-	rc = dc_serial_close (device->base.port);
-	if (rc != DC_STATUS_SUCCESS) {
-		dc_status_set_error(&status, rc);
-	}
-
-	return status;
-}
-
 
 static dc_status_t
 mares_darwin_device_set_fingerprint (dc_device_t *abstract, const unsigned char data[], unsigned int size)
@@ -215,19 +189,33 @@ mares_darwin_device_set_fingerprint (dc_device_t *abstract, const unsigned char
 static dc_status_t
 mares_darwin_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 {
+	dc_status_t status = DC_STATUS_SUCCESS;
 	mares_darwin_device_t *device = (mares_darwin_device_t *) abstract;
 
 	assert (device->layout != NULL);
 
-	// Erase the current contents of the buffer and
-	// allocate the required amount of memory.
-	if (!dc_buffer_clear (buffer) || !dc_buffer_resize (buffer, device->layout->memsize)) {
+	// Allocate the required amount of memory.
+	if (!dc_buffer_resize (buffer, device->layout->memsize)) {
 		ERROR (abstract->context, "Insufficient buffer space available.");
 		return DC_STATUS_NOMEMORY;
 	}
 
-	return device_dump_read (abstract, dc_buffer_get_data (buffer),
+	// Download the memory dump.
+	status = device_dump_read (abstract, 0, dc_buffer_get_data (buffer),
 		dc_buffer_get_size (buffer), PACKETSIZE);
+	if (status != DC_STATUS_SUCCESS) {
+		return status;
+	}
+
+	// Emit a device info event.
+	unsigned char *data = dc_buffer_get_data (buffer);
+	dc_event_devinfo_t devinfo;
+	devinfo.model = device->model;
+	devinfo.firmware = 0;
+	devinfo.serial = array_uint16_be (data + 8);
+	device_event_emit (abstract, DC_EVENT_DEVINFO, &devinfo);
+
+	return status;
 }
 
 
@@ -248,14 +236,6 @@ mares_darwin_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback,
 		return rc;
 	}
 
-	// Emit a device info event.
-	unsigned char *data = dc_buffer_get_data (buffer);
-	dc_event_devinfo_t devinfo;
-	devinfo.model = device->model;
-	devinfo.firmware = 0;
-	devinfo.serial = array_uint16_be (data + 8);
-	device_event_emit (abstract, DC_EVENT_DEVINFO, &devinfo);
-
 	rc = mares_darwin_extract_dives (abstract, dc_buffer_get_data (buffer),
 		dc_buffer_get_size (buffer), callback, userdata);
 
@@ -331,7 +311,7 @@ mares_darwin_extract_dives (dc_device_t *abstract, const unsigned char data[], u
 			current -= length;
 		}
 
-		if (device && memcmp (buffer, device->fingerprint, sizeof (device->fingerprint)) == 0) {
+		if (memcmp (buffer, device->fingerprint, sizeof (device->fingerprint)) == 0) {
 			free (buffer);
 			return DC_STATUS_SUCCESS;
 		}
