@@ -26,7 +26,6 @@
 #include "mares_common.h"
 #include "context-private.h"
 #include "device-private.h"
-#include "serial.h"
 #include "checksum.h"
 #include "array.h"
 
@@ -45,14 +44,13 @@
 
 typedef struct mares_nemo_device_t {
 	dc_device_t base;
-	dc_serial_t *port;
+	dc_iostream_t *iostream;
 	unsigned char fingerprint[5];
 } mares_nemo_device_t;
 
 static dc_status_t mares_nemo_device_set_fingerprint (dc_device_t *abstract, const unsigned char data[], unsigned int size);
 static dc_status_t mares_nemo_device_dump (dc_device_t *abstract, dc_buffer_t *buffer);
 static dc_status_t mares_nemo_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback, void *userdata);
-static dc_status_t mares_nemo_device_close (dc_device_t *abstract);
 
 static const dc_device_vtable_t mares_nemo_device_vtable = {
 	sizeof(mares_nemo_device_t),
@@ -63,7 +61,7 @@ static const dc_device_vtable_t mares_nemo_device_vtable = {
 	mares_nemo_device_dump, /* dump */
 	mares_nemo_device_foreach, /* foreach */
 	NULL, /* timesync */
-	mares_nemo_device_close /* close */
+	NULL /* close */
 };
 
 static const mares_common_layout_t mares_nemo_layout = {
@@ -84,7 +82,7 @@ static const mares_common_layout_t mares_nemo_apneist_layout = {
 
 
 dc_status_t
-mares_nemo_device_open (dc_device_t **out, dc_context_t *context, const char *name)
+mares_nemo_device_open (dc_device_t **out, dc_context_t *context, dc_iostream_t *iostream)
 {
 	dc_status_t status = DC_STATUS_SUCCESS;
 	mares_nemo_device_t *device = NULL;
@@ -100,76 +98,50 @@ mares_nemo_device_open (dc_device_t **out, dc_context_t *context, const char *na
 	}
 
 	// Set the default values.
-	device->port = NULL;
+	device->iostream = iostream;
 	memset (device->fingerprint, 0, sizeof (device->fingerprint));
 
-	// Open the device.
-	status = dc_serial_open (&device->port, context, name);
+	// Set the serial communication protocol (9600 8N1).
+	status = dc_iostream_configure (device->iostream, 9600, 8, DC_PARITY_NONE, DC_STOPBITS_ONE, DC_FLOWCONTROL_NONE);
 	if (status != DC_STATUS_SUCCESS) {
-		ERROR (context, "Failed to open the serial port.");
+		ERROR (context, "Failed to set the terminal attributes.");
 		goto error_free;
 	}
 
-	// Set the serial communication protocol (9600 8N1).
-	status = dc_serial_configure (device->port, 9600, 8, DC_PARITY_NONE, DC_STOPBITS_ONE, DC_FLOWCONTROL_NONE);
-	if (status != DC_STATUS_SUCCESS) {
-		ERROR (context, "Failed to set the terminal attributes.");
-		goto error_close;
-	}
-
 	// Set the timeout for receiving data (1000 ms).
-	status = dc_serial_set_timeout (device->port, 1000);
+	status = dc_iostream_set_timeout (device->iostream, 1000);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the timeout.");
-		goto error_close;
+		goto error_free;
 	}
 
 	// Set the DTR line.
-	status = dc_serial_set_dtr (device->port, 1);
+	status = dc_iostream_set_dtr (device->iostream, 1);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the DTR line.");
-		goto error_close;
+		goto error_free;
 	}
 
 	// Set the RTS line.
-	status = dc_serial_set_rts (device->port, 1);
+	status = dc_iostream_set_rts (device->iostream, 1);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the RTS line.");
-		goto error_close;
+		goto error_free;
 	}
 
 	// Make sure everything is in a sane state.
-	dc_serial_purge (device->port, DC_DIRECTION_ALL);
+	dc_iostream_purge (device->iostream, DC_DIRECTION_ALL);
 
 	*out = (dc_device_t*) device;
 
 	return DC_STATUS_SUCCESS;
 
-error_close:
-	dc_serial_close (device->port);
 error_free:
 	dc_device_deallocate ((dc_device_t *) device);
 	return status;
 }
 
 
-static dc_status_t
-mares_nemo_device_close (dc_device_t *abstract)
-{
-	dc_status_t status = DC_STATUS_SUCCESS;
-	mares_nemo_device_t *device = (mares_nemo_device_t*) abstract;
-	dc_status_t rc = DC_STATUS_SUCCESS;
-
-	// Close the device.
-	rc = dc_serial_close (device->port);
-	if (rc != DC_STATUS_SUCCESS) {
-		dc_status_set_error(&status, rc);
-	}
-
-	return status;
-}
-
-
 static dc_status_t
 mares_nemo_device_set_fingerprint (dc_device_t *abstract, const unsigned char data[], unsigned int size)
 {
@@ -193,9 +165,8 @@ mares_nemo_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 	dc_status_t status = DC_STATUS_SUCCESS;
 	mares_nemo_device_t *device = (mares_nemo_device_t *) abstract;
 
-	// Erase the current contents of the buffer and
-	// pre-allocate the required amount of memory.
-	if (!dc_buffer_clear (buffer) || !dc_buffer_reserve (buffer, MEMORYSIZE)) {
+	// Pre-allocate the required amount of memory.
+	if (!dc_buffer_reserve (buffer, MEMORYSIZE)) {
 		ERROR (abstract->context, "Insufficient buffer space available.");
 		return DC_STATUS_NOMEMORY;
 	}
@@ -206,19 +177,17 @@ mares_nemo_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 	device_event_emit (abstract, DC_EVENT_PROGRESS, &progress);
 
 	// Wait until some data arrives.
-	size_t available = 0;
-	while (dc_serial_get_available (device->port, &available) == DC_STATUS_SUCCESS && available == 0) {
+	while (dc_iostream_poll (device->iostream, 100) == DC_STATUS_TIMEOUT) {
 		if (device_is_cancelled (abstract))
 			return DC_STATUS_CANCELLED;
 
 		device_event_emit (abstract, DC_EVENT_WAITING, NULL);
-		dc_serial_sleep (device->port, 100);
 	}
 
 	// Receive the header of the package.
 	unsigned char header = 0x00;
 	for (unsigned int i = 0; i < 20;) {
-		status = dc_serial_read (device->port, &header, 1, NULL);
+		status = dc_iostream_read (device->iostream, &header, 1, NULL);
 		if (status != DC_STATUS_SUCCESS) {
 			ERROR (abstract->context, "Failed to receive the header.");
 			return status;
@@ -238,7 +207,7 @@ mares_nemo_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 	while (nbytes < MEMORYSIZE) {
 		// Read the packet.
 		unsigned char packet[(PACKETSIZE + 1) * 2] = {0};
-		status = dc_serial_read (device->port, packet, sizeof (packet), NULL);
+		status = dc_iostream_read (device->iostream, packet, sizeof (packet), NULL);
 		if (status != DC_STATUS_SUCCESS) {
 			ERROR (abstract->context, "Failed to receive the answer.");
 			return status;
@@ -276,6 +245,14 @@ mares_nemo_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 		nbytes += PACKETSIZE;
 	}
 
+	// Emit a device info event.
+	unsigned char *data = dc_buffer_get_data (buffer);
+	dc_event_devinfo_t devinfo;
+	devinfo.model = data[1];
+	devinfo.firmware = 0;
+	devinfo.serial = array_uint16_be (data + 8);
+	device_event_emit (abstract, DC_EVENT_DEVINFO, &devinfo);
+
 	return DC_STATUS_SUCCESS;
 }
 
@@ -295,13 +272,7 @@ mares_nemo_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback, v
 		return rc;
 	}
 
-	// Emit a device info event.
 	unsigned char *data = dc_buffer_get_data (buffer);
-	dc_event_devinfo_t devinfo;
-	devinfo.model = data[1];
-	devinfo.firmware = 0;
-	devinfo.serial = array_uint16_be (data + 8);
-	device_event_emit (abstract, DC_EVENT_DEVINFO, &devinfo);
 
 	const mares_common_layout_t *layout = NULL;
 	switch (data[1]) {
