@@ -27,7 +27,6 @@
 #include "mares_common.h"
 #include "context-private.h"
 #include "device-private.h"
-#include "serial.h"
 #include "checksum.h"
 #include "array.h"
 
@@ -47,7 +46,6 @@ typedef struct mares_puck_device_t {
 static dc_status_t mares_puck_device_set_fingerprint (dc_device_t *abstract, const unsigned char data[], unsigned int size);
 static dc_status_t mares_puck_device_dump (dc_device_t *abstract, dc_buffer_t *buffer);
 static dc_status_t mares_puck_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback, void *userdata);
-static dc_status_t mares_puck_device_close (dc_device_t *abstract);
 
 static const dc_device_vtable_t mares_puck_device_vtable = {
 	sizeof(mares_puck_device_t),
@@ -58,7 +56,7 @@ static const dc_device_vtable_t mares_puck_device_vtable = {
 	mares_puck_device_dump, /* dump */
 	mares_puck_device_foreach, /* foreach */
 	NULL, /* timesync */
-	mares_puck_device_close /* close */
+	NULL /* close */
 };
 
 static const mares_common_layout_t mares_puck_layout = {
@@ -87,7 +85,7 @@ static const mares_common_layout_t mares_nemowide_layout = {
 
 
 dc_status_t
-mares_puck_device_open (dc_device_t **out, dc_context_t *context, const char *name)
+mares_puck_device_open (dc_device_t **out, dc_context_t *context, dc_iostream_t *iostream)
 {
 	dc_status_t status = DC_STATUS_SUCCESS;
 	mares_puck_device_t *device = NULL;
@@ -103,55 +101,48 @@ mares_puck_device_open (dc_device_t **out, dc_context_t *context, const char *na
 	}
 
 	// Initialize the base class.
-	mares_common_device_init (&device->base);
+	mares_common_device_init (&device->base, iostream);
 
 	// Set the default values.
 	device->layout = NULL;
 	memset (device->fingerprint, 0, sizeof (device->fingerprint));
 
-	// Open the device.
-	status = dc_serial_open (&device->base.port, context, name);
+	// Set the serial communication protocol (38400 8N1).
+	status = dc_iostream_configure (device->base.iostream, 38400, 8, DC_PARITY_NONE, DC_STOPBITS_ONE, DC_FLOWCONTROL_NONE);
 	if (status != DC_STATUS_SUCCESS) {
-		ERROR (context, "Failed to open the serial port.");
+		ERROR (context, "Failed to set the terminal attributes.");
 		goto error_free;
 	}
 
-	// Set the serial communication protocol (38400 8N1).
-	status = dc_serial_configure (device->base.port, 38400, 8, DC_PARITY_NONE, DC_STOPBITS_ONE, DC_FLOWCONTROL_NONE);
-	if (status != DC_STATUS_SUCCESS) {
-		ERROR (context, "Failed to set the terminal attributes.");
-		goto error_close;
-	}
-
 	// Set the timeout for receiving data (1000 ms).
-	status = dc_serial_set_timeout (device->base.port, 1000);
+	status = dc_iostream_set_timeout (device->base.iostream, 1000);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the timeout.");
-		goto error_close;
+		goto error_free;
 	}
 
 	// Clear the DTR line.
-	status = dc_serial_set_dtr (device->base.port, 0);
+	status = dc_iostream_set_dtr (device->base.iostream, 0);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to clear the DTR line.");
-		goto error_close;
+		goto error_free;
 	}
 
 	// Clear the RTS line.
-	status = dc_serial_set_rts (device->base.port, 0);
+	status = dc_iostream_set_rts (device->base.iostream, 0);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to clear the RTS line.");
-		goto error_close;
+		goto error_free;
 	}
 
 	// Make sure everything is in a sane state.
-	dc_serial_purge (device->base.port, DC_DIRECTION_ALL);
+	dc_iostream_purge (device->base.iostream, DC_DIRECTION_ALL);
 
 	// Identify the model number.
 	unsigned char header[PACKETSIZE] = {0};
 	status = mares_common_device_read ((dc_device_t *) device, 0, header, sizeof (header));
 	if (status != DC_STATUS_SUCCESS) {
-		goto error_close;
+		goto error_free;
 	}
 
 	// Override the base class values.
@@ -175,31 +166,12 @@ mares_puck_device_open (dc_device_t **out, dc_context_t *context, const char *na
 
 	return DC_STATUS_SUCCESS;
 
-error_close:
-	dc_serial_close (device->base.port);
 error_free:
 	dc_device_deallocate ((dc_device_t *) device);
 	return status;
 }
 
 
-static dc_status_t
-mares_puck_device_close (dc_device_t *abstract)
-{
-	dc_status_t status = DC_STATUS_SUCCESS;
-	mares_puck_device_t *device = (mares_puck_device_t*) abstract;
-	dc_status_t rc = DC_STATUS_SUCCESS;
-
-	// Close the device.
-	rc = dc_serial_close (device->base.port);
-	if (rc != DC_STATUS_SUCCESS) {
-		dc_status_set_error(&status, rc);
-	}
-
-	return status;
-}
-
-
 static dc_status_t
 mares_puck_device_set_fingerprint (dc_device_t *abstract, const unsigned char data[], unsigned int size)
 {
@@ -220,19 +192,33 @@ mares_puck_device_set_fingerprint (dc_device_t *abstract, const unsigned char da
 static dc_status_t
 mares_puck_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 {
+	dc_status_t status = DC_STATUS_SUCCESS;
 	mares_puck_device_t *device = (mares_puck_device_t *) abstract;
 
 	assert (device->layout != NULL);
 
-	// Erase the current contents of the buffer and
-	// allocate the required amount of memory.
-	if (!dc_buffer_clear (buffer) || !dc_buffer_resize (buffer, device->layout->memsize)) {
+	// Allocate the required amount of memory.
+	if (!dc_buffer_resize (buffer, device->layout->memsize)) {
 		ERROR (abstract->context, "Insufficient buffer space available.");
 		return DC_STATUS_NOMEMORY;
 	}
 
-	return device_dump_read (abstract, dc_buffer_get_data (buffer),
+	// Download the memory dump.
+	status = device_dump_read (abstract, 0, dc_buffer_get_data (buffer),
 		dc_buffer_get_size (buffer), PACKETSIZE);
+	if (status != DC_STATUS_SUCCESS) {
+		return status;
+	}
+
+	// Emit a device info event.
+	unsigned char *data = dc_buffer_get_data (buffer);
+	dc_event_devinfo_t devinfo;
+	devinfo.model = data[1];
+	devinfo.firmware = 0;
+	devinfo.serial = array_uint16_be (data + 8);
+	device_event_emit (abstract, DC_EVENT_DEVINFO, &devinfo);
+
+	return status;
 }
 
 
@@ -253,15 +239,8 @@ mares_puck_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback, v
 		return rc;
 	}
 
-	// Emit a device info event.
-	unsigned char *data = dc_buffer_get_data (buffer);
-	dc_event_devinfo_t devinfo;
-	devinfo.model = data[1];
-	devinfo.firmware = 0;
-	devinfo.serial = array_uint16_be (data + 8);
-	device_event_emit (abstract, DC_EVENT_DEVINFO, &devinfo);
-
-	rc = mares_common_extract_dives (abstract->context, device->layout, device->fingerprint, data, callback, userdata);
+	rc = mares_common_extract_dives (abstract->context, device->layout, device->fingerprint,
+		dc_buffer_get_data (buffer), callback, userdata);
 
 	dc_buffer_free (buffer);
 
