@@ -541,6 +541,8 @@ mclean_extreme_device_foreach(dc_device_t *abstract, dc_dive_callback_t callback
 		return status;
 	}
 
+	HEXDUMP (abstract->context, DC_LOGLEVEL_DEBUG, "Firmware", firmware, sizeof(firmware));
+
 	// Read the serial number.
 	size_t serial_len = 0;
 	unsigned char serial[SZ_PACKET] = {0};
@@ -550,6 +552,8 @@ mclean_extreme_device_foreach(dc_device_t *abstract, dc_dive_callback_t callback
 		return status;
 	}
 
+	HEXDUMP (abstract->context, DC_LOGLEVEL_DEBUG, "Serial", serial, serial_len);
+
 	// Emit a device info event.
 	dc_event_devinfo_t devinfo;
 	devinfo.model = 0;
@@ -565,6 +569,8 @@ mclean_extreme_device_foreach(dc_device_t *abstract, dc_dive_callback_t callback
 		return status;
 	}
 
+	HEXDUMP (abstract->context, DC_LOGLEVEL_DEBUG, "Config", computer, sizeof(computer));
+
 	// Verify the format version.
 	unsigned int format = computer[0x0000];
 	if (format != 0) {
