@@ -35,6 +35,12 @@
 #define GAUGE    1
 #define FREEDIVE 2
 
+#define DSX_CC        0
+#define DSX_OC        1
+#define DSX_SIDEMOUNT 2
+#define DSX_SIDEGAUGE 3
+#define DSX_GAUGE     4
+
 #define NGASMIXES 6
 
 #define HEADER  1
@@ -45,6 +51,7 @@ typedef struct oceanic_atom2_parser_t oceanic_atom2_parser_t;
 struct oceanic_atom2_parser_t {
 	dc_parser_t base;
 	unsigned int model;
+	unsigned int logbooksize;
 	unsigned int headersize;
 	unsigned int footersize;
 	// Cached fields.
@@ -59,7 +66,6 @@ struct oceanic_atom2_parser_t {
 	double maxdepth;
 };
 
-static dc_status_t oceanic_atom2_parser_set_data (dc_parser_t *abstract, const unsigned char *data, unsigned int size);
 static dc_status_t oceanic_atom2_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *datetime);
 static dc_status_t oceanic_atom2_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsigned int flags, void *value);
 static dc_status_t oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t callback, void *userdata);
@@ -67,7 +73,6 @@ static dc_status_t oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract,
 static const dc_parser_vtable_t oceanic_atom2_parser_vtable = {
 	sizeof(oceanic_atom2_parser_t),
 	DC_FAMILY_OCEANIC_ATOM2,
-	oceanic_atom2_parser_set_data, /* set_data */
 	NULL, /* set_clock */
 	NULL, /* set_atmospheric */
 	NULL, /* set_density */
@@ -77,9 +82,14 @@ static const dc_parser_vtable_t oceanic_atom2_parser_vtable = {
 	NULL /* destroy */
 };
 
+static unsigned int
+is_freedive (unsigned int divemode, unsigned int model)
+{
+	return divemode == FREEDIVE && model != DSX;
+}
 
 dc_status_t
-oceanic_atom2_parser_create (dc_parser_t **out, dc_context_t *context, unsigned int model)
+oceanic_atom2_parser_create (dc_parser_t **out, dc_context_t *context, const unsigned char data[], size_t size, unsigned int model)
 {
 	oceanic_atom2_parser_t *parser = NULL;
 
@@ -87,7 +97,7 @@ oceanic_atom2_parser_create (dc_parser_t **out, dc_context_t *context, unsigned
 		return DC_STATUS_INVALIDARGS;
 
 	// Allocate memory.
-	parser = (oceanic_atom2_parser_t *) dc_parser_allocate (context, &oceanic_atom2_parser_vtable);
+	parser = (oceanic_atom2_parser_t *) dc_parser_allocate (context, &oceanic_atom2_parser_vtable, data, size);
 	if (parser == NULL) {
 		ERROR (context, "Failed to allocate memory.");
 		return DC_STATUS_NOMEMORY;
@@ -95,6 +105,7 @@ oceanic_atom2_parser_create (dc_parser_t **out, dc_context_t *context, unsigned
 
 	// Set the default values.
 	parser->model = model;
+	parser->logbooksize = 0;
 	parser->headersize = 9 * PAGESIZE / 2;
 	parser->footersize = 2 * PAGESIZE / 2;
 	if (model == DATAMASK || model == COMPUMASK ||
@@ -131,8 +142,16 @@ oceanic_atom2_parser_create (dc_parser_t **out, dc_context_t *context, unsigned
 	} else if (model == PROPLUSX) {
 		parser->headersize = 3 * PAGESIZE;
 	} else if (model == I550C || model == WISDOM4 ||
-		model == I200CV2) {
+		model == I200CV2|| model == I100V2) {
 		parser->headersize = 5 * PAGESIZE / 2;
+	} else if (model == I330R || model == I330R_C) {
+		parser->logbooksize = 64;
+		parser->headersize = parser->logbooksize + 80;
+		parser->footersize = 48;
+	} else if (model == DSX) {
+		parser->logbooksize = 512;
+		parser->headersize = parser->logbooksize + 256;
+		parser->footersize = 64;
 	}
 
 	parser->cached = 0;
@@ -153,28 +172,6 @@ oceanic_atom2_parser_create (dc_parser_t **out, dc_context_t *context, unsigned
 }
 
 
-static dc_status_t
-oceanic_atom2_parser_set_data (dc_parser_t *abstract, const unsigned char *data, unsigned int size)
-{
-	oceanic_atom2_parser_t *parser = (oceanic_atom2_parser_t *) abstract;
-
-	// Reset the cache.
-	parser->cached = 0;
-	parser->header = 0;
-	parser->footer = 0;
-	parser->mode = NORMAL;
-	parser->ngasmixes = 0;
-	for (unsigned int i = 0; i < NGASMIXES; ++i) {
-		parser->oxygen[i] = 0;
-		parser->helium[i] = 0;
-	}
-	parser->divetime = 0;
-	parser->maxdepth = 0.0;
-
-	return DC_STATUS_SUCCESS;
-}
-
-
 static dc_status_t
 oceanic_atom2_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *datetime)
 {
@@ -194,8 +191,19 @@ oceanic_atom2_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *datetim
 	if (datetime) {
 		// AM/PM bit of the 12-hour clock.
 		unsigned int pm = p[1] & 0x80;
+		unsigned int have_ampm = 1;
 
 		switch (parser->model) {
+		case I330R:
+		case I330R_C:
+		case DSX:
+			datetime->year   = p[7] + 2000;
+			datetime->month  = p[6];
+			datetime->day    = p[5];
+			datetime->hour   = p[3];
+			datetime->minute = p[4];
+			have_ampm = 0;
+			break;
 		case OC1A:
 		case OC1B:
 		case OC1C:
@@ -214,6 +222,7 @@ oceanic_atom2_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *datetim
 		case I470TC:
 		case I200CV2:
 		case GEOAIR:
+		case I100V2:
 			datetime->year   = ((p[5] & 0xE0) >> 5) + ((p[7] & 0xE0) >> 2) + 2000;
 			datetime->month  = (p[3] & 0x0F);
 			datetime->day    = ((p[0] & 0x80) >> 3) + ((p[3] & 0xF0) >> 4);
@@ -304,9 +313,11 @@ oceanic_atom2_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *datetim
 		datetime->timezone = DC_TIMEZONE_NONE;
 
 		// Convert to a 24-hour clock.
-		datetime->hour %= 12;
-		if (pm)
-			datetime->hour += 12;
+		if (have_ampm) {
+			datetime->hour %= 12;
+			if (pm)
+				datetime->hour += 12;
+		}
 
 		/*
 		 * Workaround for the year 2010 problem.
@@ -381,6 +392,10 @@ oceanic_atom2_parser_cache (oceanic_atom2_parser_t *parser)
 	} else if (parser->model == VEO20 || parser->model == VEO30 ||
 		parser->model == OCS) {
 		mode = (data[1] & 0x60) >> 5;
+	} else if (parser->model == I330R || parser->model == I330R_C) {
+		mode = data[2];
+	} else if (parser->model == DSX) {
+		mode = data[45];
 	}
 
 	// Get the gas mixes.
@@ -388,7 +403,7 @@ oceanic_atom2_parser_cache (oceanic_atom2_parser_t *parser)
 	unsigned int o2_offset = 0;
 	unsigned int he_offset = 0;
 	unsigned int o2_step = 1;
-	if (mode == FREEDIVE) {
+	if (is_freedive (mode, parser->model)) {
 		ngasmixes = 0;
 	} else if (parser->model == DATAMASK || parser->model == COMPUMASK) {
 		ngasmixes = 1;
@@ -438,6 +453,17 @@ oceanic_atom2_parser_cache (oceanic_atom2_parser_t *parser)
 	} else if (parser->model == WISDOM4) {
 		o2_offset = header + 4;
 		ngasmixes = 1;
+	} else if (parser->model == I330R || parser->model == I330R_C) {
+		ngasmixes = 3;
+		o2_offset = parser->logbooksize + 16;
+	} else if (parser->model == DSX) {
+		if (mode < DSX_SIDEGAUGE) {
+			o2_offset = parser->logbooksize + 0x89 + mode * 16;
+			he_offset = parser->logbooksize + 0xB9 + mode * 16;
+			ngasmixes = 6;
+		} else {
+			ngasmixes = 0;
+		}
 	} else {
 		o2_offset = header + 4;
 		ngasmixes = 3;
@@ -451,6 +477,11 @@ oceanic_atom2_parser_cache (oceanic_atom2_parser_t *parser)
 	for (unsigned int i = 0; i < ngasmixes; ++i) {
 		if (data[o2_offset + i * o2_step]) {
 			parser->oxygen[i] = data[o2_offset + i * o2_step];
+			// The i330R uses 20 as "Air" and 21 as 21% Nitrox
+			if ((parser->model == I330R || parser->model == I330R_C) &&
+				parser->oxygen[i] == 20) {
+				parser->oxygen[i] = 21;
+			}
 		} else {
 			parser->oxygen[i] = 21;
 		}
@@ -509,13 +540,23 @@ oceanic_atom2_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, uns
 				parser->model == F11A || parser->model == F11B ||
 				parser->model == MUNDIAL2 || parser->model == MUNDIAL3)
 				*((double *) value) = array_uint16_le (data + 4) / 16.0 * FEET;
+			else if (parser->model == I330R || parser->model == I330R_C || parser->model == DSX)
+				*((double *) value) = array_uint16_le (data + parser->footer + 10) / 10.0 * FEET;
 			else
 				*((double *) value) = (array_uint16_le (data + parser->footer + 4) & 0x0FFF) / 16.0 * FEET;
 			break;
+		case DC_FIELD_AVGDEPTH:
+			if (parser->model == I330R || parser->model == I330R_C || parser->model == DSX) {
+				*((double *) value) = array_uint16_le (data + parser->footer + 12) / 10.0 * FEET;
+			} else {
+				return DC_STATUS_UNSUPPORTED;
+			}
+			break;
 		case DC_FIELD_GASMIX_COUNT:
 			*((unsigned int *) value) = parser->ngasmixes;
 			break;
 		case DC_FIELD_GASMIX:
+			gasmix->usage = DC_USAGE_NONE;
 			gasmix->oxygen = parser->oxygen[flags] / 100.0;
 			gasmix->helium = parser->helium[flags] / 100.0;
 			gasmix->nitrogen = 1.0 - gasmix->oxygen - gasmix->helium;
@@ -529,23 +570,49 @@ oceanic_atom2_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, uns
 					water->type = DC_WATER_SALT;
 				}
 				water->density = 0.0;
+			} else if (parser->model == I330R || parser->model == I330R_C || parser->model == DSX) {
+				unsigned int settings = array_uint32_le (data + parser->logbooksize + 12);
+				if (settings & 0x10000) {
+					water->type = DC_WATER_FRESH;
+				} else {
+					water->type = DC_WATER_SALT;
+				}
+				water->density = 0.0;
 			} else {
 				return DC_STATUS_UNSUPPORTED;
 			}
 			break;
 		case DC_FIELD_DIVEMODE:
-			switch (parser->mode) {
-			case NORMAL:
-				*((unsigned int *) value) = DC_DIVEMODE_OC;
-				break;
-			case GAUGE:
-				*((unsigned int *) value) = DC_DIVEMODE_GAUGE;
-				break;
-			case FREEDIVE:
-				*((unsigned int *) value) = DC_DIVEMODE_FREEDIVE;
-				break;
-			default:
-				return DC_STATUS_DATAFORMAT;
+			if (parser->model == DSX) {
+				switch (parser->mode) {
+				case DSX_OC:
+				case DSX_SIDEMOUNT:
+					*((unsigned int *) value) = DC_DIVEMODE_OC;
+					break;
+				case DSX_SIDEGAUGE:
+				case DSX_GAUGE:
+					*((unsigned int *) value) = DC_DIVEMODE_GAUGE;
+					break;
+				case DSX_CC:
+					*((unsigned int *) value) = DC_DIVEMODE_CCR;
+					break;
+				default:
+					return DC_STATUS_DATAFORMAT;
+				}
+			} else {
+				switch (parser->mode) {
+				case NORMAL:
+					*((unsigned int *) value) = DC_DIVEMODE_OC;
+					break;
+				case GAUGE:
+					*((unsigned int *) value) = DC_DIVEMODE_GAUGE;
+					break;
+				case FREEDIVE:
+					*((unsigned int *) value) = DC_DIVEMODE_FREEDIVE;
+					break;
+				default:
+					return DC_STATUS_DATAFORMAT;
+				}
 			}
 			break;
 		default:
@@ -564,7 +631,7 @@ oceanic_atom2_parser_vendor (oceanic_atom2_parser_t *parser, const unsigned char
 		dc_sample_value_t sample = {0};
 
 		// Ignore empty samples.
-		if ((parser->mode != FREEDIVE &&
+		if ((!is_freedive (parser->mode, parser->model) &&
 			array_isequal (data + offset, samplesize, 0x00)) ||
 			array_isequal (data + offset, samplesize, 0xFF)) {
 			offset += samplesize;
@@ -573,7 +640,7 @@ oceanic_atom2_parser_vendor (oceanic_atom2_parser_t *parser, const unsigned char
 
 		// Get the sample type.
 		unsigned int sampletype = data[offset + 0];
-		if (parser->mode == FREEDIVE)
+		if (is_freedive (parser->mode, parser->model))
 			sampletype = 0;
 
 		// Get the sample size.
@@ -586,7 +653,7 @@ oceanic_atom2_parser_vendor (oceanic_atom2_parser_t *parser, const unsigned char
 		sample.vendor.type = SAMPLE_VENDOR_OCEANIC_ATOM2;
 		sample.vendor.size = length;
 		sample.vendor.data = data + offset;
-		if (callback) callback (DC_SAMPLE_VENDOR, sample, userdata);
+		if (callback) callback (DC_SAMPLE_VENDOR, &sample, userdata);
 
 		offset += length;
 	}
@@ -608,34 +675,29 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 
 	unsigned int extratime = 0;
 	unsigned int time = 0;
-	unsigned int interval = 1;
-	unsigned int samplerate = 1;
-	if (parser->mode != FREEDIVE) {
-		unsigned int offset = 0x17;
-		if (parser->model == A300CS || parser->model == VTX ||
-			parser->model == I450T || parser->model == I750TC ||
-			parser->model == PROPLUSX || parser->model == I770R ||
-			parser->model == SAGE || parser->model == BEACON)
-			offset = 0x1f;
-		const unsigned int intervals[] = {2, 15, 30, 60};
-		unsigned int idx = data[offset] & 0x03;
-		interval = intervals[idx];
+	unsigned int interval = 1000;
+	if (!is_freedive (parser->mode, parser->model)) {
+		if (parser->model == I330R || parser->model == I330R_C || parser->model == DSX) {
+			interval = data[parser->logbooksize + 36] * 1000;
+		} else {
+			unsigned int offset = 0x17;
+			if (parser->model == A300CS || parser->model == VTX ||
+				parser->model == I450T || parser->model == I750TC ||
+				parser->model == PROPLUSX || parser->model == I770R ||
+				parser->model == SAGE || parser->model == BEACON)
+				offset = 0x1f;
+			const unsigned int intervals[] = {2000, 15000, 30000, 60000};
+			unsigned int idx = data[offset] & 0x03;
+			interval = intervals[idx];
+		}
 	} else if (parser->model == F11A || parser->model == F11B) {
-		const unsigned int intervals[] = {1, 1, 1, 2};
-		const unsigned int samplerates[] = {4, 2, 1, 1};
+		const unsigned int intervals[] = {250, 500, 1000, 2000};
 		unsigned int idx = data[0x29] & 0x03;
 		interval = intervals[idx];
-		samplerate = samplerates[idx];
-		if (samplerate > 1) {
-			// Some models supports multiple samples per second.
-			// Since our smallest unit of time is one second, we can't
-			// represent this, and the extra samples will get dropped.
-			WARNING(abstract->context, "Multiple samples per second are not supported!");
-		}
 	}
 
 	unsigned int samplesize = PAGESIZE / 2;
-	if (parser->mode == FREEDIVE) {
+	if (is_freedive (parser->mode, parser->model)) {
 		if (parser->model == F10A || parser->model == F10B ||
 			parser->model == F11A || parser->model == F11B ||
 			parser->model == MUNDIAL2 || parser->model == MUNDIAL3) {
@@ -650,12 +712,15 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 		parser->model == I750TC || parser->model == PROPLUSX ||
 		parser->model == I770R || parser->model == I470TC ||
 		parser->model == SAGE || parser->model == BEACON ||
-		parser->model == GEOAIR) {
+		parser->model == GEOAIR || parser->model == I330R ||
+		parser->model == I330R_C) {
 		samplesize = PAGESIZE;
+	} else if (parser->model == DSX) {
+		samplesize = 32;
 	}
 
 	unsigned int have_temperature = 1, have_pressure = 1;
-	if (parser->mode == FREEDIVE) {
+	if (is_freedive (parser->mode, parser->model)) {
 		have_temperature = 0;
 		have_pressure = 0;
 	} else if (parser->model == VEO30 || parser->model == OCS ||
@@ -666,7 +731,9 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 		parser->model == I200 || parser->model == I100 ||
 		parser->model == I300C || parser->model == TALIS ||
 		parser->model == I200C || parser->model == I200CV2 ||
-		parser->model == GEO40 || parser->model == VEO40) {
+		parser->model == GEO40 || parser->model == VEO40 ||
+		parser->model == I330R || parser->model == I330R_C ||
+		parser->model == I100V2) {
 		have_pressure = 0;
 	}
 
@@ -677,7 +744,7 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 	}
 
 	// Initial tank pressure.
-	unsigned int tank = 0;
+	unsigned int tank = 1;
 	unsigned int pressure = 0;
 	if (have_pressure) {
 		unsigned int idx = 2;
@@ -692,7 +759,6 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 	// Initial gas mix.
 	unsigned int gasmix_previous = 0xFFFFFFFF;
 
-	unsigned int count = 0;
 	unsigned int complete = 1;
 	unsigned int previous = 0;
 	unsigned int offset = parser->headersize;
@@ -700,7 +766,7 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 		dc_sample_value_t sample = {0};
 
 		// Ignore empty samples.
-		if ((parser->mode != FREEDIVE &&
+		if ((!is_freedive (parser->mode, parser->model) &&
 			array_isequal (data + offset, samplesize, 0x00)) ||
 			array_isequal (data + offset, samplesize, 0xFF)) {
 			offset += samplesize;
@@ -714,7 +780,7 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 
 		// Get the sample type.
 		unsigned int sampletype = data[offset + 0];
-		if (parser->mode == FREEDIVE)
+		if (is_freedive (parser->mode, parser->model))
 			sampletype = 0;
 
 		// The sample size is usually fixed, but some sample types have a
@@ -732,17 +798,17 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 		if (sampletype == 0xAA) {
 			if (parser->model == DATAMASK || parser->model == COMPUMASK) {
 				// Tank pressure (1 psi) and number
-				tank = 0;
+				tank = 1;
 				pressure = (((data[offset + 7] << 8) + data[offset + 6]) & 0x0FFF);
 			} else if (parser->model == A300CS || parser->model == VTX ||
 				parser->model == I750TC || parser->model == SAGE ||
 				parser->model == BEACON) {
 				// Tank pressure (1 psi) and number (one based index)
-				tank = (data[offset + 1] & 0x03) - 1;
+				tank = data[offset + 1] & 0x03;
 				pressure = ((data[offset + 7] << 8) + data[offset + 6]) & 0x0FFF;
 			} else {
 				// Tank pressure (2 psi) and number (one based index)
-				tank = (data[offset + 1] & 0x03) - 1;
+				tank = data[offset + 1] & 0x03;
 				if (parser->model == ATOM2 || parser->model == EPICA || parser->model == EPICB)
 					pressure = (((data[offset + 3] << 8) + data[offset + 4]) & 0x0FFF) * 2;
 				else
@@ -752,14 +818,14 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 			// The surface time is not always a nice multiple of the samplerate.
 			// The number of inserted surface samples is therefore rounded down
 			// to keep the timestamps aligned at multiples of the samplerate.
-			unsigned int surftime = 60 * bcd2dec (data[offset + 1]) + bcd2dec (data[offset + 2]);
+			unsigned int surftime = (60 * bcd2dec (data[offset + 1]) + bcd2dec (data[offset + 2])) * 1000;
 			unsigned int nsamples = surftime / interval;
 
 			for (unsigned int i = 0; i < nsamples; ++i) {
 				// Time
 				time += interval;
 				sample.time = time;
-				if (callback) callback (DC_SAMPLE_TIME, sample, userdata);
+				if (callback) callback (DC_SAMPLE_TIME, &sample, userdata);
 
 				// Vendor specific data
 				if (i == 0) {
@@ -771,25 +837,18 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 
 				// Depth
 				sample.depth = 0.0;
-				if (callback) callback (DC_SAMPLE_DEPTH, sample, userdata);
+				if (callback) callback (DC_SAMPLE_DEPTH, &sample, userdata);
 				complete = 1;
 			}
 
 			extratime += surftime;
 		} else {
-			// Skip the extra samples.
-			if ((count % samplerate) != 0) {
-				offset += samplesize;
-				count++;
-				continue;
-			}
-
 			// Time.
 			if (parser->model == I450T || parser->model == I470TC) {
 				unsigned int minute = bcd2dec(data[offset + 0]);
 				unsigned int hour   = bcd2dec(data[offset + 1] & 0x0F);
 				unsigned int second = bcd2dec(data[offset + 2]);
-				unsigned int timestamp = (hour * 3600) + (minute * 60 ) + second + extratime;
+				unsigned int timestamp = ((hour * 3600) + (minute * 60 ) + second) * 1000 + extratime;
 				if (timestamp < time) {
 					ERROR (abstract->context, "Timestamp moved backwards.");
 					return DC_STATUS_DATAFORMAT;
@@ -803,7 +862,7 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 				time += interval;
 			}
 			sample.time = time;
-			if (callback) callback (DC_SAMPLE_TIME, sample, userdata);
+			if (callback) callback (DC_SAMPLE_TIME, &sample, userdata);
 
 			// Vendor specific data
 			oceanic_atom2_parser_vendor (parser,
@@ -828,7 +887,7 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 					parser->model == I300C || parser->model == I200C ||
 					parser->model == GEO40 || parser->model == VEO40 ||
 					parser->model == I470TC || parser->model == I200CV2 ||
-					parser->model == GEOAIR) {
+					parser->model == GEOAIR || parser->model == I100V2) {
 					temperature = data[offset + 3];
 				} else if (parser->model == OCS || parser->model == TX1) {
 					temperature = data[offset + 1];
@@ -842,6 +901,8 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 					parser->model == I770R|| parser->model == SAGE ||
 					parser->model == BEACON) {
 					temperature = data[offset + 11];
+				} else if (parser->model == I330R || parser->model == I330R_C || parser->model == DSX) {
+					temperature = array_uint16_le(data + offset + 10);
 				} else {
 					unsigned int sign;
 					if (parser->model == DG03 || parser->model == PROPLUS3 ||
@@ -864,8 +925,12 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 					else
 						temperature += (data[offset + 7] & 0x0C) >> 2;
 				}
-				sample.temperature = (temperature - 32.0) * (5.0 / 9.0);
-				if (callback) callback (DC_SAMPLE_TEMPERATURE, sample, userdata);
+				if (parser->model == I330R || parser->model == I330R_C || parser->model == DSX) {
+					sample.temperature = ((temperature / 10.0) - 32.0) * (5.0 / 9.0);
+				} else {
+					sample.temperature = (temperature - 32.0) * (5.0 / 9.0);
+				}
+				if (callback) callback (DC_SAMPLE_TEMPERATURE, &sample, userdata);
 			}
 
 			// Tank Pressure (psi)
@@ -889,16 +954,22 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 					parser->model == PROPLUSX || parser->model == I770R ||
 					parser->model == SAGE || parser->model == BEACON)
 					pressure = array_uint16_le (data + offset + 4);
-				else
+				else if (parser->model == DSX) {
+					pressure = array_uint16_le (data + offset + 14);
+					tank = (data[offset] & 0xF0) >> 4;
+				} else {
 					pressure -= data[offset + 1];
-				sample.pressure.tank = tank;
-				sample.pressure.value = pressure * PSI / BAR;
-				if (callback) callback (DC_SAMPLE_PRESSURE, sample, userdata);
+				}
+				if (tank) {
+					sample.pressure.tank = tank - 1;
+					sample.pressure.value = pressure * PSI / BAR;
+					if (callback) callback (DC_SAMPLE_PRESSURE, &sample, userdata);
+				}
 			}
 
 			// Depth (1/16 ft)
 			unsigned int depth;
-			if (parser->mode == FREEDIVE)
+			if (is_freedive (parser->mode, parser->model))
 				depth = array_uint16_le (data + offset);
 			else if (parser->model == GEO20 || parser->model == VEO20 ||
 				parser->model == VEO30 || parser->model == OC1A ||
@@ -909,14 +980,20 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 				parser->model == I300C || parser->model == I200C ||
 				parser->model == GEO40 || parser->model == VEO40 ||
 				parser->model == I470TC || parser->model == I200CV2 ||
-				parser->model == GEOAIR)
+				parser->model == GEOAIR || parser->model == I100V2)
 				depth = (data[offset + 4] + (data[offset + 5] << 8)) & 0x0FFF;
+			else if (parser->model == I330R || parser->model == I330R_C || parser->model == DSX)
+				depth = array_uint16_le (data + offset + 2);
 			else if (parser->model == ATOM1)
 				depth = data[offset + 3] * 16;
 			else
 				depth = (data[offset + 2] + (data[offset + 3] << 8)) & 0x0FFF;
-			sample.depth = depth / 16.0 * FEET;
-			if (callback) callback (DC_SAMPLE_DEPTH, sample, userdata);
+			if (parser->model == I330R || parser->model == I330R_C || parser->model == DSX) {
+				sample.depth = depth / 10.0 * FEET;
+			} else {
+				sample.depth = depth / 16.0 * FEET;
+			}
+			if (callback) callback (DC_SAMPLE_DEPTH, &sample, userdata);
 
 			// Gas mix
 			unsigned int have_gasmix = 0;
@@ -924,14 +1001,17 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 			if (parser->model == TX1) {
 				gasmix = data[offset] & 0x07;
 				have_gasmix = 1;
+			} else if (parser->model == DSX) {
+				gasmix = (data[offset] & 0xF0) >> 4;
+				have_gasmix = 1;
 			}
-			if (have_gasmix && gasmix != gasmix_previous) {
+			if (have_gasmix && gasmix != gasmix_previous && parser->ngasmixes > 0) {
 				if (gasmix < 1 || gasmix > parser->ngasmixes) {
 					ERROR (abstract->context, "Invalid gas mix index (%u).", gasmix);
 					return DC_STATUS_DATAFORMAT;
 				}
 				sample.gasmix = gasmix - 1;
-				if (callback) callback (DC_SAMPLE_GASMIX, sample, userdata);
+				if (callback) callback (DC_SAMPLE_GASMIX, &sample, userdata);
 				gasmix_previous = gasmix;
 			}
 
@@ -955,7 +1035,8 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 				have_deco = 1;
 			} else if (parser->model == ATOM31 || parser->model == VISION ||
 				parser->model == XPAIR || parser->model == I550 ||
-				parser->model == I550C || parser->model == WISDOM4) {
+				parser->model == I550C || parser->model == WISDOM4 ||
+				parser->model == PROPLUS4 || parser->model == ATMOSAI2) {
 				decostop = (data[offset + 5] & 0xF0) >> 4;
 				decotime = array_uint16_le(data + offset + 4) & 0x03FF;
 				have_deco = 1;
@@ -966,21 +1047,36 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 				parser->model == I450T || parser->model == I200C ||
 				parser->model == GEO40 || parser->model == VEO40 ||
 				parser->model == I470TC || parser->model == I200CV2 ||
-				parser->model == GEOAIR) {
+				parser->model == GEOAIR || parser->model == I100V2) {
 				decostop = (data[offset + 7] & 0xF0) >> 4;
 				decotime = array_uint16_le(data + offset + 6) & 0x0FFF;
 				have_deco = 1;
+			} else if (parser->model == I330R || parser->model == I330R_C || parser->model == DSX) {
+				decostop = data[offset + 8];
+				if (decostop) {
+					// Deco time
+					decotime = array_uint16_le(data + offset + 6);
+				} else {
+					// NDL
+					decotime = array_uint16_le(data + offset + 4);
+				}
+				have_deco = 1;
 			}
 			if (have_deco) {
 				if (decostop) {
 					sample.deco.type = DC_DECO_DECOSTOP;
-					sample.deco.depth = decostop * 10 * FEET;
+					if (parser->model == I330R || parser->model == I330R_C || parser->model == DSX) {
+						sample.deco.depth = decostop * FEET;
+					} else {
+						sample.deco.depth = decostop * 10 * FEET;
+					}
 				} else {
 					sample.deco.type = DC_DECO_NDL;
 					sample.deco.depth = 0.0;
 				}
 				sample.deco.time = decotime * 60;
-				if (callback) callback (DC_SAMPLE_DECO, sample, userdata);
+				sample.deco.tts = 0;
+				if (callback) callback (DC_SAMPLE_DECO, &sample, userdata);
 			}
 
 			unsigned int have_rbt = 0;
@@ -997,13 +1093,21 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 				have_rbt = 1;
 			} else if (parser->model == VISION || parser->model == XPAIR ||
 				parser->model == I550 || parser->model == I550C ||
-				parser->model == WISDOM4) {
+				parser->model == WISDOM4 || parser->model == PROPLUS4 ||
+				parser->model == ATMOSAI2) {
 				rbt = array_uint16_le(data + offset + 6) & 0x03FF;
 				have_rbt = 1;
 			}
 			if (have_rbt) {
 				sample.rbt = rbt;
-				if (callback) callback (DC_SAMPLE_RBT, sample, userdata);
+				if (callback) callback (DC_SAMPLE_RBT, &sample, userdata);
+			}
+
+			// PPO2
+			if (parser->model == I330R || parser->model == I330R_C) {
+				sample.ppo2.sensor = DC_SENSOR_NONE;
+				sample.ppo2.value = data[offset + 9] / 100.0;
+				if (callback) callback (DC_SAMPLE_PPO2, &sample, userdata);
 			}
 
 			// Bookmarks
@@ -1018,10 +1122,9 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 				sample.event.time = 0;
 				sample.event.flags = 0;
 				sample.event.value = 0;
-				if (callback) callback (DC_SAMPLE_EVENT, sample, userdata);
+				if (callback) callback (DC_SAMPLE_EVENT, &sample, userdata);
 			}
 
-			count++;
 			complete = 1;
 		}
 
