@@ -37,7 +37,7 @@ dc_status_t
 oceanic_veo250_device_open (dc_device_t **device, dc_context_t *context, dc_iostream_t *iostream);
 
 dc_status_t
-oceanic_veo250_parser_create (dc_parser_t **parser, dc_context_t *context, unsigned int model);
+oceanic_veo250_parser_create (dc_parser_t **parser, dc_context_t *context, const unsigned char data[], size_t size, unsigned int model);
 
 #ifdef __cplusplus
 }
