@@ -41,7 +41,7 @@ typedef struct dc_parser_vtable_t dc_parser_vtable_t;
 struct dc_parser_t {
 	const dc_parser_vtable_t *vtable;
 	dc_context_t *context;
-	const unsigned char *data;
+	unsigned char *data;
 	unsigned int size;
 };
 
@@ -50,8 +50,6 @@ struct dc_parser_vtable_t {
 
 	dc_family_t type;
 
-	dc_status_t (*set_data) (dc_parser_t *parser, const unsigned char *data, unsigned int size);
-
 	dc_status_t (*set_clock) (dc_parser_t *parser, unsigned int devtime, dc_ticks_t systime);
 
 	dc_status_t (*set_atmospheric) (dc_parser_t *parser, double atmospheric);
@@ -68,7 +66,7 @@ struct dc_parser_vtable_t {
 };
 
 dc_parser_t *
-dc_parser_allocate (dc_context_t *context, const dc_parser_vtable_t *vtable);
+dc_parser_allocate (dc_context_t *context, const dc_parser_vtable_t *vtable, const unsigned char data[], size_t size);
 
 void
 dc_parser_deallocate (dc_parser_t *parser);
@@ -84,7 +82,7 @@ typedef struct sample_statistics_t {
 #define SAMPLE_STATISTICS_INITIALIZER {0, 0.0}
 
 void
-sample_statistics_cb (dc_sample_type_t type, dc_sample_value_t value, void *userdata);
+sample_statistics_cb (dc_sample_type_t type, const dc_sample_value_t *value, void *userdata);
 
 #ifdef __cplusplus
 }
