@@ -23,6 +23,7 @@
 #define REEFNET_SENSUS_H
 
 #include <libdivecomputer/context.h>
+#include <libdivecomputer/iostream.h>
 #include <libdivecomputer/device.h>
 #include <libdivecomputer/parser.h>
 #include <libdivecomputer/reefnet_sensus.h>
@@ -32,10 +33,10 @@ extern "C" {
 #endif /* __cplusplus */
 
 dc_status_t
-reefnet_sensus_device_open (dc_device_t **device, dc_context_t *context, const char *name);
+reefnet_sensus_device_open (dc_device_t **device, dc_context_t *context, dc_iostream_t *iostream);
 
 dc_status_t
-reefnet_sensus_parser_create (dc_parser_t **parser, dc_context_t *context, unsigned int devtime, dc_ticks_t systime);
+reefnet_sensus_parser_create (dc_parser_t **parser, dc_context_t *context, const unsigned char data[], size_t size);
 
 #ifdef __cplusplus
 }
