@@ -48,7 +48,6 @@ struct reefnet_sensus_parser_t {
 	unsigned int maxdepth;
 };
 
-static dc_status_t reefnet_sensus_parser_set_data (dc_parser_t *abstract, const unsigned char *data, unsigned int size);
 static dc_status_t reefnet_sensus_parser_set_clock (dc_parser_t *abstract, unsigned int devtime, dc_ticks_t systime);
 static dc_status_t reefnet_sensus_parser_set_atmospheric (dc_parser_t *abstract, double atmospheric);
 static dc_status_t reefnet_sensus_parser_set_density (dc_parser_t *abstract, double density);
@@ -59,7 +58,6 @@ static dc_status_t reefnet_sensus_parser_samples_foreach (dc_parser_t *abstract,
 static const dc_parser_vtable_t reefnet_sensus_parser_vtable = {
 	sizeof(reefnet_sensus_parser_t),
 	DC_FAMILY_REEFNET_SENSUS,
-	reefnet_sensus_parser_set_data, /* set_data */
 	reefnet_sensus_parser_set_clock, /* set_clock */
 	reefnet_sensus_parser_set_atmospheric, /* set_atmospheric */
 	reefnet_sensus_parser_set_density, /* set_density */
@@ -71,7 +69,7 @@ static const dc_parser_vtable_t reefnet_sensus_parser_vtable = {
 
 
 dc_status_t
-reefnet_sensus_parser_create (dc_parser_t **out, dc_context_t *context, unsigned int devtime, dc_ticks_t systime)
+reefnet_sensus_parser_create (dc_parser_t **out, dc_context_t *context, const unsigned char data[], size_t size)
 {
 	reefnet_sensus_parser_t *parser = NULL;
 
@@ -79,7 +77,7 @@ reefnet_sensus_parser_create (dc_parser_t **out, dc_context_t *context, unsigned
 		return DC_STATUS_INVALIDARGS;
 
 	// Allocate memory.
-	parser = (reefnet_sensus_parser_t *) dc_parser_allocate (context, &reefnet_sensus_parser_vtable);
+	parser = (reefnet_sensus_parser_t *) dc_parser_allocate (context, &reefnet_sensus_parser_vtable, data, size);
 	if (parser == NULL) {
 		ERROR (context, "Failed to allocate memory.");
 		return DC_STATUS_NOMEMORY;
@@ -88,8 +86,8 @@ reefnet_sensus_parser_create (dc_parser_t **out, dc_context_t *context, unsigned
 	// Set the default values.
 	parser->atmospheric = DEF_ATMOSPHERIC;
 	parser->hydrostatic = DEF_DENSITY_SALT * GRAVITY;
-	parser->devtime = devtime;
-	parser->systime = systime;
+	parser->devtime = 0;
+	parser->systime = 0;
 	parser->cached = 0;
 	parser->divetime = 0;
 	parser->maxdepth = 0;
@@ -100,35 +98,6 @@ reefnet_sensus_parser_create (dc_parser_t **out, dc_context_t *context, unsigned
 }
 
 
-static dc_status_t
-reefnet_sensus_parser_set_data (dc_parser_t *abstract, const unsigned char *data, unsigned int size)
-{
-	reefnet_sensus_parser_t *parser = (reefnet_sensus_parser_t*) abstract;
-
-	// Reset the cache.
-	parser->cached = 0;
-	parser->divetime = 0;
-	parser->maxdepth = 0;
-
-	return DC_STATUS_SUCCESS;
-}
-
-
-dc_status_t
-reefnet_sensus_parser_set_calibration (dc_parser_t *abstract, double atmospheric, double hydrostatic)
-{
-	reefnet_sensus_parser_t *parser = (reefnet_sensus_parser_t*) abstract;
-
-	if (!ISINSTANCE (abstract))
-		return DC_STATUS_INVALIDARGS;
-
-	parser->atmospheric = atmospheric;
-	parser->hydrostatic = hydrostatic;
-
-	return DC_STATUS_SUCCESS;
-}
-
-
 static dc_status_t
 reefnet_sensus_parser_set_clock (dc_parser_t *abstract, unsigned int devtime, dc_ticks_t systime)
 {
@@ -279,13 +248,13 @@ reefnet_sensus_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback
 
 				// Time (seconds)
 				time += interval;
-				sample.time = time;
-				if (callback) callback (DC_SAMPLE_TIME, sample, userdata);
+				sample.time = time * 1000;
+				if (callback) callback (DC_SAMPLE_TIME, &sample, userdata);
 
 				// Depth (adjusted feet of seawater).
 				unsigned int depth = data[offset++];
 				sample.depth = ((depth + 33.0 - (double) SAMPLE_DEPTH_ADJUST) * FSW - parser->atmospheric) / parser->hydrostatic;
-				if (callback) callback (DC_SAMPLE_DEPTH, sample, userdata);
+				if (callback) callback (DC_SAMPLE_DEPTH, &sample, userdata);
 
 				// Temperature (degrees Fahrenheit)
 				if ((nsamples % 6) == 0) {
@@ -293,7 +262,7 @@ reefnet_sensus_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback
 						return DC_STATUS_DATAFORMAT;
 					unsigned int temperature = data[offset++];
 					sample.temperature = (temperature - 32.0) * (5.0 / 9.0);
-					if (callback) callback (DC_SAMPLE_TEMPERATURE, sample, userdata);
+					if (callback) callback (DC_SAMPLE_TEMPERATURE, &sample, userdata);
 				}
 
 				// Current sample is complete.
