@@ -31,7 +31,6 @@
 #include "context-private.h"
 #include "iostream-private.h"
 #include "iterator-private.h"
-#include "descriptor-private.h"
 #include "platform.h"
 
 static dc_status_t dc_serial_iterator_next (dc_iterator_t *iterator, void *item);
@@ -228,7 +227,7 @@ dc_serial_iterator_next (dc_iterator_t *abstract, void *out)
 		// Null terminate the string.
 		data[data_len] = 0;
 
-		if (!dc_descriptor_filter (iterator->descriptor, DC_TRANSPORT_SERIAL, data, NULL)) {
+		if (!dc_descriptor_filter (iterator->descriptor, DC_TRANSPORT_SERIAL, data)) {
 			continue;
 		}
 
