@@ -28,6 +28,7 @@
 #include "shearwater_petrel.h"
 #include "context-private.h"
 #include "parser-private.h"
+#include "platform.h"
 #include "array.h"
 
 #define ISINSTANCE(parser)	( \
@@ -107,6 +108,8 @@ typedef struct shearwater_predator_gasmix_t {
 	unsigned int oxygen;
 	unsigned int helium;
 	unsigned int diluent;
+	unsigned int enabled;
+	unsigned int active;
 } shearwater_predator_gasmix_t;
 
 typedef struct shearwater_predator_tank_t {
@@ -118,6 +121,7 @@ typedef struct shearwater_predator_tank_t {
 	unsigned int pressure_reserve;
 	unsigned int serial;
 	char name[2];
+	dc_usage_t usage;
 } shearwater_predator_tank_t;
 
 struct shearwater_predator_parser_t {
@@ -140,6 +144,7 @@ struct shearwater_predator_parser_t {
 	shearwater_predator_tank_t tank[NTANKS];
 	unsigned int tankidx[NTANKS];
 	unsigned int aimode;
+	unsigned int hpccr;
 	unsigned int calibrated;
 	double calibration[3];
 	unsigned int divemode;
@@ -148,7 +153,6 @@ struct shearwater_predator_parser_t {
 	unsigned int density;
 };
 
-static dc_status_t shearwater_predator_parser_set_data (dc_parser_t *abstract, const unsigned char *data, unsigned int size);
 static dc_status_t shearwater_predator_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *datetime);
 static dc_status_t shearwater_predator_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsigned int flags, void *value);
 static dc_status_t shearwater_predator_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t callback, void *userdata);
@@ -158,7 +162,6 @@ static dc_status_t shearwater_predator_parser_cache (shearwater_predator_parser_
 static const dc_parser_vtable_t shearwater_predator_parser_vtable = {
 	sizeof(shearwater_predator_parser_t),
 	DC_FAMILY_SHEARWATER_PREDATOR,
-	shearwater_predator_parser_set_data, /* set_data */
 	NULL, /* set_clock */
 	NULL, /* set_atmospheric */
 	NULL, /* set_density */
@@ -171,7 +174,6 @@ static const dc_parser_vtable_t shearwater_predator_parser_vtable = {
 static const dc_parser_vtable_t shearwater_petrel_parser_vtable = {
 	sizeof(shearwater_predator_parser_t),
 	DC_FAMILY_SHEARWATER_PETREL,
-	shearwater_predator_parser_set_data, /* set_data */
 	NULL, /* set_clock */
 	NULL, /* set_atmospheric */
 	NULL, /* set_density */
@@ -182,6 +184,12 @@ static const dc_parser_vtable_t shearwater_petrel_parser_vtable = {
 };
 
 
+static unsigned int
+shearwater_predator_is_ccr (unsigned int divemode)
+{
+	return divemode == M_CC || divemode == M_CC2 || divemode == M_SC;
+}
+
 static unsigned int
 shearwater_predator_find_gasmix (shearwater_predator_parser_t *parser, unsigned int o2, unsigned int he, unsigned int dil)
 {
@@ -197,7 +205,7 @@ shearwater_predator_find_gasmix (shearwater_predator_parser_t *parser, unsigned
 
 
 static dc_status_t
-shearwater_common_parser_create (dc_parser_t **out, dc_context_t *context, unsigned int model, unsigned int petrel)
+shearwater_common_parser_create (dc_parser_t **out, dc_context_t *context, const unsigned char data[], size_t size, unsigned int model, unsigned int petrel)
 {
 	shearwater_predator_parser_t *parser = NULL;
 	const dc_parser_vtable_t *vtable = NULL;
@@ -215,7 +223,7 @@ shearwater_common_parser_create (dc_parser_t **out, dc_context_t *context, unsig
 	}
 
 	// Allocate memory.
-	parser = (shearwater_predator_parser_t *) dc_parser_allocate (context, vtable);
+	parser = (shearwater_predator_parser_t *) dc_parser_allocate (context, vtable, data, size);
 	if (parser == NULL) {
 		ERROR (context, "Failed to allocate memory.");
 		return DC_STATUS_NOMEMORY;
@@ -240,6 +248,8 @@ shearwater_common_parser_create (dc_parser_t **out, dc_context_t *context, unsig
 		parser->gasmix[i].oxygen = 0;
 		parser->gasmix[i].helium = 0;
 		parser->gasmix[i].diluent = 0;
+		parser->gasmix[i].enabled = 0;
+		parser->gasmix[i].active = 0;
 	}
 	parser->ntanks = 0;
 	for (unsigned int i = 0; i < NTANKS; ++i) {
@@ -251,9 +261,11 @@ shearwater_common_parser_create (dc_parser_t **out, dc_context_t *context, unsig
 		parser->tank[i].pressure_reserve = 0;
 		parser->tank[i].serial = 0;
 		memset (parser->tank[i].name, 0, sizeof (parser->tank[i].name));
+		parser->tank[i].usage = DC_USAGE_NONE;
 		parser->tankidx[i] = i;
 	}
 	parser->aimode = AI_OFF;
+	parser->hpccr = 0;
 	parser->calibrated = 0;
 	for (unsigned int i = 0; i < 3; ++i) {
 		parser->calibration[i] = 0.0;
@@ -270,64 +282,16 @@ shearwater_common_parser_create (dc_parser_t **out, dc_context_t *context, unsig
 
 
 dc_status_t
-shearwater_predator_parser_create (dc_parser_t **out, dc_context_t *context, unsigned int model)
+shearwater_predator_parser_create (dc_parser_t **out, dc_context_t *context, const unsigned char data[], size_t size, unsigned int model)
 {
-	return shearwater_common_parser_create (out, context, model, 0);
+	return shearwater_common_parser_create (out, context, data, size, model, 0);
 }
 
 
 dc_status_t
-shearwater_petrel_parser_create (dc_parser_t **out, dc_context_t *context, unsigned int model)
+shearwater_petrel_parser_create (dc_parser_t **out, dc_context_t *context, const unsigned char data[], size_t size, unsigned int model)
 {
-	return shearwater_common_parser_create (out, context, model, 1);
-}
-
-
-static dc_status_t
-shearwater_predator_parser_set_data (dc_parser_t *abstract, const unsigned char *data, unsigned int size)
-{
-	shearwater_predator_parser_t *parser = (shearwater_predator_parser_t *) abstract;
-
-	// Reset the cache.
-	parser->cached = 0;
-	parser->pnf = 0;
-	parser->logversion = 0;
-	parser->headersize = 0;
-	parser->footersize = 0;
-	for (unsigned int i = 0; i < NRECORDS; ++i) {
-		parser->opening[i] = UNDEFINED;
-		parser->closing[i] = UNDEFINED;
-	}
-	parser->final = UNDEFINED;
-	parser->ngasmixes = 0;
-	for (unsigned int i = 0; i < NGASMIXES; ++i) {
-		parser->gasmix[i].oxygen = 0;
-		parser->gasmix[i].helium = 0;
-		parser->gasmix[i].diluent = 0;
-	}
-	parser->ntanks = 0;
-	for (unsigned int i = 0; i < NTANKS; ++i) {
-		parser->tank[i].enabled = 0;
-		parser->tank[i].active = 0;
-		parser->tank[i].beginpressure = 0;
-		parser->tank[i].endpressure = 0;
-		parser->tank[i].pressure_max = 0;
-		parser->tank[i].pressure_reserve = 0;
-		parser->tank[i].serial = 0;
-		memset (parser->tank[i].name, 0, sizeof (parser->tank[i].name));
-		parser->tankidx[i] = i;
-	}
-	parser->aimode = AI_OFF;
-	parser->calibrated = 0;
-	for (unsigned int i = 0; i < 3; ++i) {
-		parser->calibration[i] = 0.0;
-	}
-	parser->divemode = M_OC_TEC;
-	parser->units = METRIC;
-	parser->density = DEF_DENSITY_SALT;
-	parser->atmospheric = DEF_ATMOSPHERIC / (BAR / 1000);
-
-	return DC_STATUS_SUCCESS;
+	return shearwater_common_parser_create (out, context, data, size, model, 1);
 }
 
 
@@ -430,11 +394,13 @@ shearwater_predator_parser_cache (shearwater_predator_parser_t *parser)
 	shearwater_predator_tank_t tank[NTANKS] = {0};
 	unsigned int o2_previous = UNDEFINED, he_previous = UNDEFINED, dil_previous = UNDEFINED;
 	unsigned int aimode = AI_OFF;
+	unsigned int hpccr = 0;
 	if (!pnf) {
 		for (unsigned int i = 0; i < NFIXED; ++i) {
 			gasmix[i].oxygen = data[20 + i];
 			gasmix[i].helium = data[30 + i];
 			gasmix[i].diluent = i >= 5;
+			gasmix[i].enabled = 1;
 		}
 	}
 
@@ -483,6 +449,8 @@ shearwater_predator_parser_cache (shearwater_predator_parser_t *parser)
 					ngasmixes = idx + 1;
 				}
 
+				gasmix[idx].active = 1;
+
 				o2_previous = o2;
 				he_previous = he;
 				dil_previous = ccr;
@@ -504,12 +472,14 @@ shearwater_predator_parser_cache (shearwater_predator_parser_t *parser)
 					unsigned int id = (aimode == AI_HPCCR ? 4 : 0) + i;
 					if (pressure < 0xFFF0) {
 						pressure &= 0x0FFF;
-						if (!tank[id].active) {
-							tank[id].active = 1;
-							tank[id].beginpressure = pressure;
+						if (pressure) {
+							if (!tank[id].active) {
+								tank[id].active = 1;
+								tank[id].beginpressure = pressure;
+								tank[id].endpressure = pressure;
+							}
 							tank[id].endpressure = pressure;
 						}
-						tank[id].endpressure = pressure;
 					}
 				}
 			}
@@ -521,12 +491,14 @@ shearwater_predator_parser_cache (shearwater_predator_parser_t *parser)
 					unsigned int id = 2 + i;
 					if (pressure < 0xFFF0) {
 						pressure &= 0x0FFF;
-						if (!tank[id].active) {
-							tank[id].active = 1;
-							tank[id].beginpressure = pressure;
+						if (pressure) {
+							if (!tank[id].active) {
+								tank[id].active = 1;
+								tank[id].beginpressure = pressure;
+								tank[id].endpressure = pressure;
+							}
 							tank[id].endpressure = pressure;
 						}
-						tank[id].endpressure = pressure;
 					}
 				}
 			}
@@ -541,8 +513,8 @@ shearwater_predator_parser_cache (shearwater_predator_parser_t *parser)
 							tank[id].enabled = 1;
 							tank[id].beginpressure = pressure;
 							tank[id].endpressure = pressure;
-							tank[id].name[0] = i == 0 ? 'D': 'O';
-							tank[id].name[1] = 0;
+							tank[id].usage = i == 0 ? DC_USAGE_DILUENT : DC_USAGE_OXYGEN;
+							hpccr = 1;
 						}
 						tank[id].endpressure = pressure;
 					}
@@ -586,9 +558,24 @@ shearwater_predator_parser_cache (shearwater_predator_parser_t *parser)
 						if (aimode == AI_HPCCR) {
 							for (unsigned int i = 0; i < 2; ++i) {
 								tank[4 + i].enabled = 1;
-								tank[4 + i].name[0] = i == 0 ? 'D': 'O';
-								tank[4 + i].name[1] = 0;
+								tank[4 + i].usage = i == 0 ? DC_USAGE_DILUENT : DC_USAGE_OXYGEN;
 							}
+							hpccr = 1;
+						}
+					}
+				}
+
+				// Gas mix on/off state.
+				unsigned int state = array_uint16_be (data + offset + 17);
+				for (unsigned int i = 0; i < NFIXED; ++i) {
+					gasmix[i].enabled = (state & (1 << i)) != 0;
+				}
+
+				unsigned int gtrmode = data[offset + 29];
+				if (popcount(gtrmode) >= 2) {
+					for (unsigned int i = 0; i < 4; ++i) {
+						if (gtrmode & (1 << i)) {
+							tank[i].usage = DC_USAGE_SIDEMOUNT;
 						}
 					}
 				}
@@ -688,6 +675,10 @@ shearwater_predator_parser_cache (shearwater_predator_parser_t *parser)
 	// Get the correct model number from the final block.
 	if (parser->final != UNDEFINED) {
 		parser->model = data[parser->final + 13];
+		DEBUG (abstract->context, "Device: model=%u, serial=%u, firmware=%u",
+			data[parser->final + 13],
+			array_uint32_be (data + parser->final + 2),
+			bcd2dec (data[parser->final + 10]));
 	}
 
 	// Fix the Teric tank serial number.
@@ -710,6 +701,10 @@ shearwater_predator_parser_cache (shearwater_predator_parser_t *parser)
 		for (unsigned int i = 0; i < ngasmixes; ++i) {
 			if (gasmix[i].oxygen == 0 && gasmix[i].helium == 0)
 				continue;
+			if (!gasmix[i].enabled && !gasmix[i].active)
+				continue;
+			if (gasmix[i].diluent && !shearwater_predator_is_ccr (divemode))
+				continue;
 			parser->gasmix[parser->ngasmixes] = gasmix[i];
 			parser->ngasmixes++;
 		}
@@ -725,6 +720,7 @@ shearwater_predator_parser_cache (shearwater_predator_parser_t *parser)
 		}
 	}
 	parser->aimode = aimode;
+	parser->hpccr = hpccr;
 	parser->divemode = divemode;
 	parser->units = data[parser->opening[0] + 8];
 	parser->atmospheric = array_uint16_be (data + parser->opening[1] + (parser->pnf ? 16 : 47));
@@ -774,6 +770,7 @@ shearwater_predator_parser_get_field (dc_parser_t *abstract, dc_field_type_t typ
 			*((unsigned int *) value) = parser->ngasmixes;
 			break;
 		case DC_FIELD_GASMIX:
+			gasmix->usage = parser->gasmix[flags].diluent ? DC_USAGE_DILUENT : DC_USAGE_NONE;
 			gasmix->oxygen = parser->gasmix[flags].oxygen / 100.0;
 			gasmix->helium = parser->gasmix[flags].helium / 100.0;
 			gasmix->nitrogen = 1.0 - gasmix->oxygen - gasmix->helium;
@@ -788,6 +785,21 @@ shearwater_predator_parser_get_field (dc_parser_t *abstract, dc_field_type_t typ
 			tank->beginpressure = parser->tank[flags].beginpressure * 2 * PSI / BAR;
 			tank->endpressure   = parser->tank[flags].endpressure   * 2 * PSI / BAR;
 			tank->gasmix = DC_GASMIX_UNKNOWN;
+			if (shearwater_predator_is_ccr (parser->divemode) && !parser->hpccr) {
+				switch (parser->tank[flags].name[0]) {
+				case 'O':
+					tank->usage = DC_USAGE_OXYGEN;
+					break;
+				case 'D':
+					tank->usage = DC_USAGE_DILUENT;
+					break;
+				default:
+					tank->usage = DC_USAGE_NONE;
+					break;
+				}
+			} else {
+				tank->usage = parser->tank[flags].usage;
+			}
 			break;
 		case DC_FIELD_SALINITY:
 			if (parser->density == 1000)
@@ -871,14 +883,9 @@ shearwater_predator_parser_samples_foreach (dc_parser_t *abstract, dc_sample_cal
 
 	// Sample interval.
 	unsigned int time = 0;
-	unsigned int interval = 10;
+	unsigned int interval = 10000;
 	if (parser->pnf && parser->logversion >= 9 && parser->opening[5] != UNDEFINED) {
 		interval = array_uint16_be (data + parser->opening[5] + 23);
-		if (interval % 1000 != 0) {
-			ERROR (abstract->context, "Unsupported sample interval (%u ms).", interval);
-			return DC_STATUS_DATAFORMAT;
-		}
-		interval /= 1000;
 	}
 
 	unsigned int pnf = parser->pnf;
@@ -900,7 +907,7 @@ shearwater_predator_parser_samples_foreach (dc_parser_t *abstract, dc_sample_cal
 			// Time (seconds).
 			time += interval;
 			sample.time = time;
-			if (callback) callback (DC_SAMPLE_TIME, sample, userdata);
+			if (callback) callback (DC_SAMPLE_TIME, &sample, userdata);
 
 			// Depth (1/10 m or ft).
 			unsigned int depth = array_uint16_be (data + pnf + offset);
@@ -908,7 +915,7 @@ shearwater_predator_parser_samples_foreach (dc_parser_t *abstract, dc_sample_cal
 				sample.depth = depth * FEET / 10.0;
 			else
 				sample.depth = depth / 10.0;
-			if (callback) callback (DC_SAMPLE_DEPTH, sample, userdata);
+			if (callback) callback (DC_SAMPLE_DEPTH, &sample, userdata);
 
 			// Temperature (°C or °F).
 			int temperature = (signed char) data[offset + pnf + 13];
@@ -923,7 +930,7 @@ shearwater_predator_parser_samples_foreach (dc_parser_t *abstract, dc_sample_cal
 				sample.temperature = (temperature - 32.0) * (5.0 / 9.0);
 			else
 				sample.temperature = temperature;
-			if (callback) callback (DC_SAMPLE_TEMPERATURE, sample, userdata);
+			if (callback) callback (DC_SAMPLE_TEMPERATURE, &sample, userdata);
 
 			// Status flags.
 			unsigned int status = data[offset + pnf + 11];
@@ -932,19 +939,21 @@ shearwater_predator_parser_samples_foreach (dc_parser_t *abstract, dc_sample_cal
 			if (ccr) {
 				// PPO2
 				if ((status & PPO2_EXTERNAL) == 0) {
-#ifdef SENSOR_AVERAGE
-					sample.ppo2 = data[offset + pnf + 6] / 100.0;
-					if (callback) callback (DC_SAMPLE_PPO2, sample, userdata);
-#else
-					sample.ppo2 = data[offset + pnf + 12] * parser->calibration[0];
-					if (callback && (parser->calibrated & 0x01)) callback (DC_SAMPLE_PPO2, sample, userdata);
+					sample.ppo2.sensor = DC_SENSOR_NONE;
+					sample.ppo2.value = data[offset + pnf + 6] / 100.0;
+					if (callback) callback (DC_SAMPLE_PPO2, &sample, userdata);
 
-					sample.ppo2 = data[offset + pnf + 14] * parser->calibration[1];
-					if (callback && (parser->calibrated & 0x02)) callback (DC_SAMPLE_PPO2, sample, userdata);
+					sample.ppo2.sensor = 0;
+					sample.ppo2.value = data[offset + pnf + 12] * parser->calibration[0];
+					if (callback && (parser->calibrated & 0x01)) callback (DC_SAMPLE_PPO2, &sample, userdata);
 
-					sample.ppo2 = data[offset + pnf + 15] * parser->calibration[2];
-					if (callback && (parser->calibrated & 0x04)) callback (DC_SAMPLE_PPO2, sample, userdata);
-#endif
+					sample.ppo2.sensor = 1;
+					sample.ppo2.value = data[offset + pnf + 14] * parser->calibration[1];
+					if (callback && (parser->calibrated & 0x02)) callback (DC_SAMPLE_PPO2, &sample, userdata);
+
+					sample.ppo2.sensor = 2;
+					sample.ppo2.value = data[offset + pnf + 15] * parser->calibration[2];
+					if (callback && (parser->calibrated & 0x04)) callback (DC_SAMPLE_PPO2, &sample, userdata);
 				}
 
 				// Setpoint
@@ -958,13 +967,13 @@ shearwater_predator_parser_samples_foreach (dc_parser_t *abstract, dc_sample_cal
 						sample.setpoint = data[17] / 100.0;
 					}
 				}
-				if (callback) callback (DC_SAMPLE_SETPOINT, sample, userdata);
+				if (callback) callback (DC_SAMPLE_SETPOINT, &sample, userdata);
 			}
 
 			// CNS
 			if (parser->petrel) {
 				sample.cns = data[offset + pnf + 22] / 100.0;
-				if (callback) callback (DC_SAMPLE_CNS, sample, userdata);
+				if (callback) callback (DC_SAMPLE_CNS, &sample, userdata);
 			}
 
 			// Gaschange.
@@ -979,7 +988,7 @@ shearwater_predator_parser_samples_foreach (dc_parser_t *abstract, dc_sample_cal
 				}
 
 				sample.gasmix = idx;
-				if (callback) callback (DC_SAMPLE_GASMIX, sample, userdata);
+				if (callback) callback (DC_SAMPLE_GASMIX, &sample, userdata);
 				o2_previous = o2;
 				he_previous = he;
 				dil_previous = ccr;
@@ -998,7 +1007,8 @@ shearwater_predator_parser_samples_foreach (dc_parser_t *abstract, dc_sample_cal
 				sample.deco.depth = 0.0;
 			}
 			sample.deco.time = data[offset + pnf + 9] * 60;
-			if (callback) callback (DC_SAMPLE_DECO, sample, userdata);
+			sample.deco.tts = array_uint16_be (data + offset + pnf + 4) * 60;
+			if (callback) callback (DC_SAMPLE_DECO, &sample, userdata);
 
 			// for logversion 7 and newer (introduced for Perdix AI)
 			// detect tank pressure
@@ -1018,9 +1028,11 @@ shearwater_predator_parser_samples_foreach (dc_parser_t *abstract, dc_sample_cal
 					unsigned int id = (parser->aimode == AI_HPCCR ? 4 : 0) + i;
 					if (pressure < 0xFFF0) {
 						pressure &= 0x0FFF;
-						sample.pressure.tank = parser->tankidx[id];
-						sample.pressure.value = pressure * 2 * PSI / BAR;
-						if (callback) callback (DC_SAMPLE_PRESSURE, sample, userdata);
+						if (pressure) {
+							sample.pressure.tank = parser->tankidx[id];
+							sample.pressure.value = pressure * 2 * PSI / BAR;
+							if (callback) callback (DC_SAMPLE_PRESSURE, &sample, userdata);
+						}
 					}
 				}
 
@@ -1033,7 +1045,7 @@ shearwater_predator_parser_samples_foreach (dc_parser_t *abstract, dc_sample_cal
 				//    0xFB Tank size or max pressure haven’t been set up
 				if (data[offset + pnf + 21] < 0xF0) {
 					sample.rbt = data[offset + pnf + 21];
-					if (callback) callback (DC_SAMPLE_RBT, sample, userdata);
+					if (callback) callback (DC_SAMPLE_RBT, &sample, userdata);
 				}
 			}
 		} else if (type == LOG_RECORD_DIVE_SAMPLE_EXT) {
@@ -1044,9 +1056,11 @@ shearwater_predator_parser_samples_foreach (dc_parser_t *abstract, dc_sample_cal
 					unsigned int id = 2 + i;
 					if (pressure < 0xFFF0) {
 						pressure &= 0x0FFF;
-						sample.pressure.tank = parser->tankidx[id];
-						sample.pressure.value = pressure * 2 * PSI / BAR;
-						if (callback) callback (DC_SAMPLE_PRESSURE, sample, userdata);
+						if (pressure) {
+							sample.pressure.tank = parser->tankidx[id];
+							sample.pressure.value = pressure * 2 * PSI / BAR;
+							if (callback) callback (DC_SAMPLE_PRESSURE, &sample, userdata);
+						}
 					}
 				}
 			}
@@ -1058,7 +1072,7 @@ shearwater_predator_parser_samples_foreach (dc_parser_t *abstract, dc_sample_cal
 					if (pressure) {
 						sample.pressure.tank = parser->tankidx[id];
 						sample.pressure.value = pressure * 2 * PSI / BAR;
-						if (callback) callback (DC_SAMPLE_PRESSURE, sample, userdata);
+						if (callback) callback (DC_SAMPLE_PRESSURE, &sample, userdata);
 					}
 				}
 			}
@@ -1077,21 +1091,21 @@ shearwater_predator_parser_samples_foreach (dc_parser_t *abstract, dc_sample_cal
 				// Time (seconds).
 				time += interval;
 				sample.time = time;
-				if (callback) callback (DC_SAMPLE_TIME, sample, userdata);
+				if (callback) callback (DC_SAMPLE_TIME, &sample, userdata);
 
 				// Depth (absolute pressure in millibar)
 				unsigned int depth = array_uint16_be (data + idx + 1);
 				sample.depth = (signed int)(depth - parser->atmospheric) * (BAR / 1000.0) / (parser->density * GRAVITY);
-				if (callback) callback (DC_SAMPLE_DEPTH, sample, userdata);
+				if (callback) callback (DC_SAMPLE_DEPTH, &sample, userdata);
 
 				// Temperature (1/10 °C).
 				int temperature = (signed short) array_uint16_be (data + idx + 3);
 				sample.temperature = temperature / 10.0;
-				if (callback) callback (DC_SAMPLE_TEMPERATURE, sample, userdata);
+				if (callback) callback (DC_SAMPLE_TEMPERATURE, &sample, userdata);
 			}
 		} else if (type == LOG_RECORD_INFO_EVENT) {
 			unsigned int event = data[offset + 1];
-			unsigned int timestamp = array_uint32_be (data + offset + 4);
+			unsigned int DC_ATTR_UNUSED timestamp = array_uint32_be (data + offset + 4);
 			unsigned int w1 = array_uint32_be (data + offset + 8);
 			unsigned int w2 = array_uint32_be (data + offset + 12);
 
@@ -1099,7 +1113,7 @@ shearwater_predator_parser_samples_foreach (dc_parser_t *abstract, dc_sample_cal
 				// Compass heading
 				if (w1 != 0xFFFFFFFF) {
 					sample.bearing = w1;
-					if (callback) callback (DC_SAMPLE_BEARING, sample, userdata);
+					if (callback) callback (DC_SAMPLE_BEARING, &sample, userdata);
 				}
 
 				// Tag
@@ -1107,7 +1121,7 @@ shearwater_predator_parser_samples_foreach (dc_parser_t *abstract, dc_sample_cal
 				sample.event.time = 0;
 				sample.event.flags = 0;
 				sample.event.value = w2;
-				if (callback) callback (DC_SAMPLE_EVENT, sample, userdata);
+				if (callback) callback (DC_SAMPLE_EVENT, &sample, userdata);
 			}
 		}
 
