@@ -25,6 +25,7 @@
 #include "suunto_d9.h"
 #include "context-private.h"
 #include "parser-private.h"
+#include "platform.h"
 #include "array.h"
 
 #define ISINSTANCE(parser) dc_parser_isinstance((parser), &suunto_d9_parser_vtable)
@@ -92,7 +93,6 @@ typedef struct sample_info_t {
 	unsigned int divisor;
 } sample_info_t;
 
-static dc_status_t suunto_d9_parser_set_data (dc_parser_t *abstract, const unsigned char *data, unsigned int size);
 static dc_status_t suunto_d9_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *datetime);
 static dc_status_t suunto_d9_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsigned int flags, void *value);
 static dc_status_t suunto_d9_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t callback, void *userdata);
@@ -100,7 +100,6 @@ static dc_status_t suunto_d9_parser_samples_foreach (dc_parser_t *abstract, dc_s
 static const dc_parser_vtable_t suunto_d9_parser_vtable = {
 	sizeof(suunto_d9_parser_t),
 	DC_FAMILY_SUUNTO_D9,
-	suunto_d9_parser_set_data, /* set_data */
 	NULL, /* set_clock */
 	NULL, /* set_atmospheric */
 	NULL, /* set_density */
@@ -251,7 +250,7 @@ suunto_d9_parser_cache (suunto_d9_parser_t *parser)
 }
 
 dc_status_t
-suunto_d9_parser_create (dc_parser_t **out, dc_context_t *context, unsigned int model)
+suunto_d9_parser_create (dc_parser_t **out, dc_context_t *context, const unsigned char data[], size_t size, unsigned int model)
 {
 	suunto_d9_parser_t *parser = NULL;
 
@@ -259,7 +258,7 @@ suunto_d9_parser_create (dc_parser_t **out, dc_context_t *context, unsigned int
 		return DC_STATUS_INVALIDARGS;
 
 	// Allocate memory.
-	parser = (suunto_d9_parser_t *) dc_parser_allocate (context, &suunto_d9_parser_vtable);
+	parser = (suunto_d9_parser_t *) dc_parser_allocate (context, &suunto_d9_parser_vtable, data, size);
 	if (parser == NULL) {
 		ERROR (context, "Failed to allocate memory.");
 		return DC_STATUS_NOMEMORY;
@@ -285,28 +284,6 @@ suunto_d9_parser_create (dc_parser_t **out, dc_context_t *context, unsigned int
 }
 
 
-static dc_status_t
-suunto_d9_parser_set_data (dc_parser_t *abstract, const unsigned char *data, unsigned int size)
-{
-	suunto_d9_parser_t *parser = (suunto_d9_parser_t *) abstract;
-
-	// Reset the cache.
-	parser->cached = 0;
-	parser->id = 0;
-	parser->mode = AIR;
-	parser->ngasmixes = 0;
-	parser->nccr = 0;
-	for (unsigned int i = 0; i < NGASMIXES; ++i) {
-		parser->oxygen[i] = 0;
-		parser->helium[i] = 0;
-	}
-	parser->gasmix = 0;
-	parser->config = 0;
-
-	return DC_STATUS_SUCCESS;
-}
-
-
 static dc_status_t
 suunto_d9_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *datetime)
 {
@@ -388,6 +365,7 @@ suunto_d9_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsigne
 			*((unsigned int *) value) = parser->ngasmixes;
 			break;
 		case DC_FIELD_GASMIX:
+			gasmix->usage = DC_USAGE_NONE;
 			gasmix->helium = parser->helium[flags] / 100.0;
 			gasmix->oxygen = parser->oxygen[flags] / 100.0;
 			gasmix->nitrogen = 1.0 - gasmix->oxygen - gasmix->helium;
@@ -521,8 +499,8 @@ suunto_d9_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t ca
 		dc_sample_value_t sample = {0};
 
 		// Time (seconds).
-		sample.time = time;
-		if (callback) callback (DC_SAMPLE_TIME, sample, userdata);
+		sample.time = time * 1000;
+		if (callback) callback (DC_SAMPLE_TIME, &sample, userdata);
 
 		// Sample data.
 		for (unsigned int i = 0; i < nparams; ++i) {
@@ -537,19 +515,19 @@ suunto_d9_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t ca
 				case 0x64: // Depth
 					value = array_uint16_le (data + offset);
 					sample.depth = value / (double) info[i].divisor;
-					if (callback) callback (DC_SAMPLE_DEPTH, sample, userdata);
+					if (callback) callback (DC_SAMPLE_DEPTH, &sample, userdata);
 					break;
 				case 0x68: // Pressure
 					value = array_uint16_le (data + offset);
 					if (value != 0xFFFF) {
 						sample.pressure.tank = 0;
 						sample.pressure.value = value / (double) info[i].divisor;
-						if (callback) callback (DC_SAMPLE_PRESSURE, sample, userdata);
+						if (callback) callback (DC_SAMPLE_PRESSURE, &sample, userdata);
 					}
 					break;
 				case 0x74: // Temperature
 					sample.temperature = (signed char) data[offset] / (double) info[i].divisor;
-					if (callback) callback (DC_SAMPLE_TEMPERATURE, sample, userdata);
+					if (callback) callback (DC_SAMPLE_TEMPERATURE, &sample, userdata);
 					break;
 				default: // Unknown sample type
 					ERROR (abstract->context, "Unknown sample type 0x%02x.", info[i].type);
@@ -567,14 +545,14 @@ suunto_d9_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t ca
 				return DC_STATUS_DATAFORMAT;
 			}
 			sample.gasmix = parser->gasmix;
-			if (callback) callback (DC_SAMPLE_GASMIX, sample, userdata);
+			if (callback) callback (DC_SAMPLE_GASMIX, &sample, userdata);
 		}
 
 		// Events
 		if ((nsamples + 1) == marker) {
 			while (offset < size) {
 				unsigned int event = data[offset++];
-				unsigned int seconds, type, state, number, heading;
+				unsigned int seconds, type, DC_ATTR_UNUSED state, DC_ATTR_UNUSED number, heading;
 				unsigned int current, next;
 				unsigned int he, o2, ppo2, idx;
 				unsigned int length;
@@ -607,7 +585,7 @@ suunto_d9_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t ca
 					seconds = data[offset + 1];
 					sample.event.type = SAMPLE_EVENT_SURFACE;
 					sample.event.time = seconds;
-					if (callback) callback (DC_SAMPLE_EVENT, sample, userdata);
+					if (callback) callback (DC_SAMPLE_EVENT, &sample, userdata);
 					offset += 2;
 					break;
 				case 0x03: // Event
@@ -713,7 +691,7 @@ suunto_d9_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t ca
 						sample.event.flags = SAMPLE_FLAGS_BEGIN;
 					sample.event.time = seconds;
 					if (sample.event.type != SAMPLE_EVENT_NONE) {
-						if (callback) callback (DC_SAMPLE_EVENT, sample, userdata);
+						if (callback) callback (DC_SAMPLE_EVENT, &sample, userdata);
 					}
 					offset += 2;
 					break;
@@ -733,7 +711,7 @@ suunto_d9_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t ca
 						sample.event.value = heading / 2;
 					}
 					sample.event.time = seconds;
-					if (callback) callback (DC_SAMPLE_EVENT, sample, userdata);
+					if (callback) callback (DC_SAMPLE_EVENT, &sample, userdata);
 					offset += 4;
 					break;
 				case 0x05: // Gas Change
@@ -749,7 +727,7 @@ suunto_d9_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t ca
 						return DC_STATUS_DATAFORMAT;
 					}
 					sample.gasmix = idx;
-					if (callback) callback (DC_SAMPLE_GASMIX, sample, userdata);
+					if (callback) callback (DC_SAMPLE_GASMIX, &sample, userdata);
 					offset += 2;
 					break;
 				case 0x06: // Gas Change
@@ -783,10 +761,10 @@ suunto_d9_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t ca
 						return DC_STATUS_DATAFORMAT;
 					}
 					sample.gasmix = idx;
-					if (callback) callback (DC_SAMPLE_GASMIX, sample, userdata);
+					if (callback) callback (DC_SAMPLE_GASMIX, &sample, userdata);
 					if (type & 0x80) {
 						sample.setpoint = ppo2 / 10.0;
-						if (callback) callback (DC_SAMPLE_SETPOINT, sample, userdata);
+						if (callback) callback (DC_SAMPLE_SETPOINT, &sample, userdata);
 					}
 					offset += length;
 					break;
@@ -811,7 +789,8 @@ suunto_d9_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t ca
 		}
 		sample.deco.time = 0;
 		sample.deco.depth = 0.0;
-		if (callback) callback (DC_SAMPLE_DECO, sample, userdata);
+		sample.deco.tts = 0;
+		if (callback) callback (DC_SAMPLE_DECO, &sample, userdata);
 
 		time += interval_sample;
 		nsamples++;
