@@ -23,6 +23,7 @@
 #define SUUNTO_EON_H
 
 #include <libdivecomputer/context.h>
+#include <libdivecomputer/iostream.h>
 #include <libdivecomputer/device.h>
 #include <libdivecomputer/parser.h>
 #include <libdivecomputer/suunto_eon.h>
@@ -32,10 +33,10 @@ extern "C" {
 #endif /* __cplusplus */
 
 dc_status_t
-suunto_eon_device_open (dc_device_t **device, dc_context_t *context, const char *name);
+suunto_eon_device_open (dc_device_t **device, dc_context_t *context, dc_iostream_t *iostream);
 
 dc_status_t
-suunto_eon_parser_create (dc_parser_t **parser, dc_context_t *context, int spyder);
+suunto_eon_parser_create (dc_parser_t **parser, dc_context_t *context, const unsigned char data[], size_t size, int spyder);
 
 #ifdef __cplusplus
 }
