@@ -88,6 +88,7 @@ typedef struct suunto_eonsteel_parser_t {
 		double highsetpoint;
 		double customsetpoint;
 		dc_tankvolume_t tankinfo[MAXGASES];
+		dc_usage_t tankusage[MAXGASES];
 		double tanksize[MAXGASES];
 		double tankworkingpressure[MAXGASES];
 		dc_decomodel_t decomodel;
@@ -474,8 +475,8 @@ static void sample_time(struct sample_data *info, unsigned short time_delta)
 	dc_sample_value_t sample = {0};
 
 	info->time += time_delta;
-	sample.time = info->time / 1000;
-	if (info->callback) info->callback(DC_SAMPLE_TIME, sample, info->userdata);
+	sample.time = info->time;
+	if (info->callback) info->callback(DC_SAMPLE_TIME, &sample, info->userdata);
 }
 
 static void sample_depth(struct sample_data *info, unsigned short depth)
@@ -486,7 +487,7 @@ static void sample_depth(struct sample_data *info, unsigned short depth)
 		return;
 
 	sample.depth = depth / 100.0;
-	if (info->callback) info->callback(DC_SAMPLE_DEPTH, sample, info->userdata);
+	if (info->callback) info->callback(DC_SAMPLE_DEPTH, &sample, info->userdata);
 }
 
 static void sample_temp(struct sample_data *info, short temp)
@@ -497,7 +498,7 @@ static void sample_temp(struct sample_data *info, short temp)
 		return;
 
 	sample.temperature = temp / 10.0;
-	if (info->callback) info->callback(DC_SAMPLE_TEMPERATURE, sample, info->userdata);
+	if (info->callback) info->callback(DC_SAMPLE_TEMPERATURE, &sample, info->userdata);
 }
 
 static void sample_ndl(struct sample_data *info, short ndl)
@@ -510,7 +511,8 @@ static void sample_ndl(struct sample_data *info, short ndl)
 
 	sample.deco.type = DC_DECO_NDL;
 	sample.deco.time = ndl;
-	if (info->callback) info->callback(DC_SAMPLE_DECO, sample, info->userdata);
+	sample.deco.tts = 0;
+	if (info->callback) info->callback(DC_SAMPLE_DECO, &sample, info->userdata);
 }
 
 static void sample_tts(struct sample_data *info, unsigned short tts)
@@ -534,7 +536,7 @@ static void sample_heading(struct sample_data *info, unsigned short heading)
 
 	sample.event.type = SAMPLE_EVENT_HEADING;
 	sample.event.value = heading;
-	if (info->callback) info->callback(DC_SAMPLE_EVENT, sample, info->userdata);
+	if (info->callback) info->callback(DC_SAMPLE_EVENT, &sample, info->userdata);
 }
 
 static void sample_abspressure(struct sample_data *info, unsigned short pressure)
@@ -549,7 +551,7 @@ static void sample_gastime(struct sample_data *info, short gastime)
 		return;
 
 	sample.rbt = gastime / 60;
-	if (info->callback) info->callback (DC_SAMPLE_RBT, sample, info->userdata);
+	if (info->callback) info->callback (DC_SAMPLE_RBT, &sample, info->userdata);
 }
 
 /*
@@ -577,7 +579,7 @@ static void sample_pressure(struct sample_data *info, unsigned short pressure)
 
 	sample.pressure.tank = info->gasnr-1;
 	sample.pressure.value = pressure / 100.0;
-	if (info->callback) info->callback(DC_SAMPLE_PRESSURE, sample, info->userdata);
+	if (info->callback) info->callback(DC_SAMPLE_PRESSURE, &sample, info->userdata);
 }
 
 static void sample_bookmark_event(struct sample_data *info, unsigned short idx)
@@ -587,7 +589,7 @@ static void sample_bookmark_event(struct sample_data *info, unsigned short idx)
 	sample.event.type = SAMPLE_EVENT_BOOKMARK;
 	sample.event.value = idx;
 
-	if (info->callback) info->callback(DC_SAMPLE_EVENT, sample, info->userdata);
+	if (info->callback) info->callback(DC_SAMPLE_EVENT, &sample, info->userdata);
 }
 
 static void sample_gas_switch_event(struct sample_data *info, unsigned short idx)
@@ -599,7 +601,7 @@ static void sample_gas_switch_event(struct sample_data *info, unsigned short idx
 		return;
 
 	sample.gasmix = idx - 1;
-	if (info->callback) info->callback(DC_SAMPLE_GASMIX, sample, info->userdata);
+	if (info->callback) info->callback(DC_SAMPLE_GASMIX, &sample, info->userdata);
 }
 
 /*
@@ -699,7 +701,7 @@ static void sample_event_state_value(const struct type_desc *desc, struct sample
 		return;
 
 	sample.event.flags = value ? SAMPLE_FLAGS_BEGIN : SAMPLE_FLAGS_END;
-	if (info->callback) info->callback(DC_SAMPLE_EVENT, sample, info->userdata);
+	if (info->callback) info->callback(DC_SAMPLE_EVENT, &sample, info->userdata);
 }
 
 static void sample_event_notify_type(const struct type_desc *desc, struct sample_data *info, unsigned char type)
@@ -741,7 +743,7 @@ static void sample_event_notify_value(const struct type_desc *desc, struct sampl
 		return;
 
 	sample.event.flags = value ? SAMPLE_FLAGS_BEGIN : SAMPLE_FLAGS_END;
-	if (info->callback) info->callback(DC_SAMPLE_EVENT, sample, info->userdata);
+	if (info->callback) info->callback(DC_SAMPLE_EVENT, &sample, info->userdata);
 }
 
 
@@ -781,7 +783,7 @@ static void sample_event_warning_value(const struct type_desc *desc, struct samp
 		return;
 
 	sample.event.flags = value ? SAMPLE_FLAGS_BEGIN : SAMPLE_FLAGS_END;
-	if (info->callback) info->callback(DC_SAMPLE_EVENT, sample, info->userdata);
+	if (info->callback) info->callback(DC_SAMPLE_EVENT, &sample, info->userdata);
 }
 
 static void sample_event_alarm_type(const struct type_desc *desc, struct sample_data *info, unsigned char type)
@@ -814,7 +816,7 @@ static void sample_event_alarm_value(const struct type_desc *desc, struct sample
 		return;
 
 	sample.event.flags = value ? SAMPLE_FLAGS_BEGIN : SAMPLE_FLAGS_END;
-	if (info->callback) info->callback(DC_SAMPLE_EVENT, sample, info->userdata);
+	if (info->callback) info->callback(DC_SAMPLE_EVENT, &sample, info->userdata);
 }
 
 // enum:0=Low,1=High,2=Custom
@@ -840,7 +842,7 @@ static void sample_setpoint_type(const struct type_desc *desc, struct sample_dat
 		return;
 	}
 
-	if (info->callback) info->callback(DC_SAMPLE_SETPOINT, sample, info->userdata);
+	if (info->callback) info->callback(DC_SAMPLE_SETPOINT, &sample, info->userdata);
 	free(type);
 }
 
@@ -996,9 +998,10 @@ static int traverse_samples(unsigned short type, const struct type_desc *desc, c
 		dc_sample_value_t sample = {0};
 
 		sample.deco.type = DC_DECO_DECOSTOP;
-		sample.deco.time = info->tts;
+		sample.deco.time = 0;
 		sample.deco.depth = info->ceiling;
-		if (info->callback) info->callback(DC_SAMPLE_DECO, sample, info->userdata);
+		sample.deco.tts = info->tts;
+		if (info->callback) info->callback(DC_SAMPLE_DECO, &sample, info->userdata);
 	}
 
 	// Warn if there are left-over bytes for something we did use part of
@@ -1095,6 +1098,7 @@ suunto_eonsteel_parser_get_field(dc_parser_t *parser, dc_field_type_t type, unsi
 			if (fabs(tank->volume - rint(tank->volume)) > 0.001)
 				tank->type = DC_TANKVOLUME_IMPERIAL;
 		}
+		tank->usage = eon->cache.tankusage[flags];
 		break;
 	case DC_FIELD_DECOMODEL:
 		field_value(value, eon->cache.decomodel);
@@ -1158,6 +1162,7 @@ static int add_gas_type(suunto_eonsteel_parser_t *eon, const struct type_desc *d
 {
 	int idx = eon->cache.ngases;
 	dc_tankvolume_t tankinfo = DC_TANKVOLUME_METRIC;
+	dc_usage_t usage = DC_USAGE_NONE;
 	char *name;
 
 	if (idx >= MAXGASES)
@@ -1168,15 +1173,17 @@ static int add_gas_type(suunto_eonsteel_parser_t *eon, const struct type_desc *d
 	if (!name)
 		DEBUG(eon->base.context, "Unable to look up gas type %u in %s", type, desc->format);
 	else if (!strcasecmp(name, "Diluent"))
-		;
+		usage = DC_USAGE_DILUENT;
 	else if (!strcasecmp(name, "Oxygen"))
-		;
+		usage = DC_USAGE_OXYGEN;
 	else if (!strcasecmp(name, "None"))
 		tankinfo = DC_TANKVOLUME_NONE;
 	else if (strcasecmp(name, "Primary"))
 		DEBUG(eon->base.context, "Unknown gas type %u (%s)", type, name);
 
 	eon->cache.tankinfo[idx] = tankinfo;
+	eon->cache.tankusage[idx] = usage;
+	eon->cache.gasmix[idx].usage = usage;
 
 	eon->cache.initialized |= 1 << DC_FIELD_GASMIX_COUNT;
 	eon->cache.initialized |= 1 << DC_FIELD_TANK_COUNT;
@@ -1502,18 +1509,6 @@ static void show_all_descriptors(suunto_eonsteel_parser_t *eon)
 		show_descriptor(eon, i, eon->type_desc+i);
 }
 
-static dc_status_t
-suunto_eonsteel_parser_set_data(dc_parser_t *parser, const unsigned char *data, unsigned int size)
-{
-	suunto_eonsteel_parser_t *eon = (suunto_eonsteel_parser_t *) parser;
-
-	desc_free(eon->type_desc, MAXTYPE);
-	memset(eon->type_desc, 0, sizeof(eon->type_desc));
-	initialize_field_caches(eon);
-	show_all_descriptors(eon);
-	return DC_STATUS_SUCCESS;
-}
-
 static dc_status_t
 suunto_eonsteel_parser_destroy(dc_parser_t *parser)
 {
@@ -1527,7 +1522,6 @@ suunto_eonsteel_parser_destroy(dc_parser_t *parser)
 static const dc_parser_vtable_t suunto_eonsteel_parser_vtable = {
 	sizeof(suunto_eonsteel_parser_t),
 	DC_FAMILY_SUUNTO_EONSTEEL,
-	suunto_eonsteel_parser_set_data, /* set_data */
 	NULL, /* set_clock */
 	NULL, /* set_atmospheric */
 	NULL, /* set_density */
@@ -1538,14 +1532,14 @@ static const dc_parser_vtable_t suunto_eonsteel_parser_vtable = {
 };
 
 dc_status_t
-suunto_eonsteel_parser_create(dc_parser_t **out, dc_context_t *context, unsigned int model)
+suunto_eonsteel_parser_create(dc_parser_t **out, dc_context_t *context, const unsigned char data[], size_t size, unsigned int model)
 {
 	suunto_eonsteel_parser_t *parser = NULL;
 
 	if (out == NULL)
 		return DC_STATUS_INVALIDARGS;
 
-	parser = (suunto_eonsteel_parser_t *) dc_parser_allocate (context, &suunto_eonsteel_parser_vtable);
+	parser = (suunto_eonsteel_parser_t *) dc_parser_allocate (context, &suunto_eonsteel_parser_vtable, data, size);
 	if (parser == NULL) {
 		ERROR (context, "Failed to allocate memory.");
 		return DC_STATUS_NOMEMORY;
@@ -1554,6 +1548,9 @@ suunto_eonsteel_parser_create(dc_parser_t **out, dc_context_t *context, unsigned
 	memset(&parser->type_desc, 0, sizeof(parser->type_desc));
 	memset(&parser->cache, 0, sizeof(parser->cache));
 
+	initialize_field_caches(parser);
+	show_all_descriptors(parser);
+
 	*out = (dc_parser_t *) parser;
 
 	return DC_STATUS_SUCCESS;
