@@ -23,6 +23,7 @@
 #define SUUNTO_SOLUTION_H
 
 #include <libdivecomputer/context.h>
+#include <libdivecomputer/iostream.h>
 #include <libdivecomputer/device.h>
 #include <libdivecomputer/parser.h>
 
@@ -31,10 +32,10 @@ extern "C" {
 #endif /* __cplusplus */
 
 dc_status_t
-suunto_solution_device_open (dc_device_t **device, dc_context_t *context, const char *name);
+suunto_solution_device_open (dc_device_t **device, dc_context_t *context, dc_iostream_t *iostream);
 
 dc_status_t
-suunto_solution_parser_create (dc_parser_t **parser, dc_context_t *context);
+suunto_solution_parser_create (dc_parser_t **parser, dc_context_t *context, const unsigned char data[], size_t size);
 
 #ifdef __cplusplus
 }
