@@ -25,9 +25,9 @@
 #include "suunto_vyper2.h"
 #include "suunto_common2.h"
 #include "context-private.h"
-#include "serial.h"
 #include "checksum.h"
 #include "array.h"
+#include "timer.h"
 
 #define ISINSTANCE(device) dc_device_isinstance((device), (const dc_device_vtable_t *) &suunto_vyper2_device_vtable)
 
@@ -35,7 +35,8 @@
 
 typedef struct suunto_vyper2_device_t {
 	suunto_common2_device_t base;
-	dc_serial_t *port;
+	dc_iostream_t *iostream;
+	dc_timer_t *timer;
 } suunto_vyper2_device_t;
 
 static dc_status_t suunto_vyper2_device_packet (dc_device_t *abstract, const unsigned char command[], unsigned int csize, unsigned char answer[], unsigned int asize, unsigned int size);
@@ -50,7 +51,7 @@ static const suunto_common2_device_vtable_t suunto_vyper2_device_vtable = {
 		suunto_common2_device_write, /* write */
 		suunto_common2_device_dump, /* dump */
 		suunto_common2_device_foreach, /* foreach */
-		NULL, /* timesync */
+		suunto_common2_device_timesync, /* timesync */
 		suunto_vyper2_device_close /* close */
 	},
 	suunto_vyper2_device_packet
@@ -74,7 +75,7 @@ static const suunto_common2_layout_t suunto_helo2_layout = {
 
 
 dc_status_t
-suunto_vyper2_device_open (dc_device_t **out, dc_context_t *context, const char *name)
+suunto_vyper2_device_open (dc_device_t **out, dc_context_t *context, dc_iostream_t *iostream)
 {
 	dc_status_t status = DC_STATUS_SUCCESS;
 	suunto_vyper2_device_t *device = NULL;
@@ -93,52 +94,55 @@ suunto_vyper2_device_open (dc_device_t **out, dc_context_t *context, const char
 	suunto_common2_device_init (&device->base);
 
 	// Set the default values.
-	device->port = NULL;
+	device->iostream = iostream;
 
-	// Open the device.
-	status = dc_serial_open (&device->port, context, name);
+	// Create a high resolution timer.
+	status = dc_timer_new (&device->timer);
 	if (status != DC_STATUS_SUCCESS) {
-		ERROR (context, "Failed to open the serial port.");
+		ERROR (context, "Failed to create a high resolution timer.");
 		goto error_free;
 	}
 
 	// Set the serial communication protocol (9600 8N1).
-	status = dc_serial_configure (device->port, 9600, 8, DC_PARITY_NONE, DC_STOPBITS_ONE, DC_FLOWCONTROL_NONE);
+	status = dc_iostream_configure (device->iostream, 9600, 8, DC_PARITY_NONE, DC_STOPBITS_ONE, DC_FLOWCONTROL_NONE);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the terminal attributes.");
-		goto error_close;
+		goto error_timer_free;
 	}
 
 	// Set the timeout for receiving data (3000 ms).
-	status = dc_serial_set_timeout (device->port, 3000);
+	status = dc_iostream_set_timeout (device->iostream, 3000);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the timeout.");
-		goto error_close;
+		goto error_timer_free;
 	}
 
 	// Set the DTR line (power supply for the interface).
-	status = dc_serial_set_dtr (device->port, 1);
+	status = dc_iostream_set_dtr (device->iostream, 1);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the DTR line.");
-		goto error_close;
+		goto error_timer_free;
 	}
 
 	// Give the interface 100 ms to settle and draw power up.
-	dc_serial_sleep (device->port, 100);
+	dc_iostream_sleep (device->iostream, 100);
 
 	// Make sure everything is in a sane state.
-	dc_serial_purge (device->port, DC_DIRECTION_ALL);
-
-	// Enable half-duplex emulation.
-	dc_serial_set_halfduplex (device->port, 1);
+	status = dc_iostream_purge (device->iostream, DC_DIRECTION_ALL);
+	if (status != DC_STATUS_SUCCESS) {
+		ERROR (context, "Failed to reset IO state.");
+		goto error_timer_free;
+	}
 
 	// Read the version info.
 	status = suunto_common2_device_version ((dc_device_t *) device, device->base.version, sizeof (device->base.version));
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to read the version info.");
-		goto error_close;
+		goto error_timer_free;
 	}
 
+	HEXDUMP (context, DC_LOGLEVEL_DEBUG, "Version", device->base.version, sizeof (device->base.version));
+
 	// Override the base class values.
 	unsigned int model = device->base.version[0];
 	if (model == HELO2)
@@ -150,8 +154,8 @@ suunto_vyper2_device_open (dc_device_t **out, dc_context_t *context, const char
 
 	return DC_STATUS_SUCCESS;
 
-error_close:
-	dc_serial_close (device->port);
+error_timer_free:
+	dc_timer_free (device->timer);
 error_free:
 	dc_device_deallocate ((dc_device_t *) device);
 	return status;
@@ -161,17 +165,11 @@ error_free:
 static dc_status_t
 suunto_vyper2_device_close (dc_device_t *abstract)
 {
-	dc_status_t status = DC_STATUS_SUCCESS;
 	suunto_vyper2_device_t *device = (suunto_vyper2_device_t*) abstract;
-	dc_status_t rc = DC_STATUS_SUCCESS;
 
-	// Close the device.
-	rc = dc_serial_close (device->port);
-	if (rc != DC_STATUS_SUCCESS) {
-		dc_status_set_error(&status, rc);
-	}
+	dc_timer_free (device->timer);
 
-	return status;
+	return DC_STATUS_SUCCESS;
 }
 
 
@@ -184,23 +182,66 @@ suunto_vyper2_device_packet (dc_device_t *abstract, const unsigned char command[
 	if (device_is_cancelled (abstract))
 		return DC_STATUS_CANCELLED;
 
-	dc_serial_sleep (device->port, 600);
+	dc_iostream_sleep (device->iostream, 600);
 
 	// Set RTS to send the command.
-	dc_serial_set_rts (device->port, 1);
+	status = dc_iostream_set_rts (device->iostream, 1);
+	if (status != DC_STATUS_SUCCESS) {
+		ERROR (abstract->context, "Failed to set the RTS line.");
+		return status;
+	}
+
+	// Get the current timestamp.
+	dc_usecs_t begin = 0;
+	status = dc_timer_now (device->timer, &begin);
+	if (status != DC_STATUS_SUCCESS) {
+		return status;
+	}
 
 	// Send the command to the dive computer.
-	status = dc_serial_write (device->port, command, csize, NULL);
+	status = dc_iostream_write (device->iostream, command, csize, NULL);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to send the command.");
 		return status;
 	}
 
+	// Get the current timestamp.
+	dc_usecs_t end = 0;
+	status = dc_timer_now (device->timer, &end);
+	if (status != DC_STATUS_SUCCESS) {
+		return status;
+	}
+
+	// Calculate the elapsed time.
+	dc_usecs_t elapsed = end - begin;
+
+	// Calculate the expected duration. A 2 millisecond fudge factor is added
+	// because it improves the success rate significantly.
+	unsigned int baudrate = 9600;
+	unsigned int nbits = 1 + 8 /* databits */ + 1 /* stopbits */ + 0 /* parity */;
+	dc_usecs_t expected = (dc_usecs_t) csize * 1000000 * nbits / baudrate + 2000;
+
+	// Wait for the remaining time.
+	if (elapsed < expected) {
+		dc_usecs_t remaining = expected - elapsed;
+
+		// The remaining time is rounded up to the nearest millisecond
+		// because the sleep function doesn't support a higher
+		// resolution on all platforms. The higher resolution is
+		// pointless anyway, since we already added a fudge factor
+		// above.
+		dc_iostream_sleep (device->iostream, (remaining + 999) / 1000);
+	}
+
 	// Clear RTS to receive the reply.
-	dc_serial_set_rts (device->port, 0);
+	status = dc_iostream_set_rts (device->iostream, 0);
+	if (status != DC_STATUS_SUCCESS) {
+		ERROR (abstract->context, "Failed to set the RTS line.");
+		return status;
+	}
 
 	// Receive the answer of the dive computer.
-	status = dc_serial_read (device->port, answer, asize, NULL);
+	status = dc_iostream_read (device->iostream, answer, asize, NULL);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to receive the answer.");
 		return status;
@@ -213,7 +254,8 @@ suunto_vyper2_device_packet (dc_device_t *abstract, const unsigned char command[
 	}
 
 	// Verify the size of the package.
-	if (array_uint16_be (answer + 1) + 4 != asize) {
+	unsigned int len = array_uint16_be (answer + 1);
+	if (len + 4 != asize) {
 		ERROR (abstract->context, "Unexpected answer size.");
 		return DC_STATUS_PROTOCOL;
 	}
