@@ -57,7 +57,6 @@
 #include "common-private.h"
 #include "context-private.h"
 #include "iostream-private.h"
-#include "descriptor-private.h"
 #include "iterator-private.h"
 #include "platform.h"
 
@@ -438,8 +437,8 @@ dc_usbhid_iterator_next (dc_iterator_t *abstract, void *out)
 			return syserror (rc);
 		}
 
-		dc_usb_desc_t usb = {dev.idVendor, dev.idProduct};
-		if (!dc_descriptor_filter (iterator->descriptor, DC_TRANSPORT_USBHID, &usb, NULL)) {
+		dc_usbhid_desc_t usb = {dev.idVendor, dev.idProduct};
+		if (!dc_descriptor_filter (iterator->descriptor, DC_TRANSPORT_USBHID, &usb)) {
 			continue;
 		}
 
@@ -522,8 +521,8 @@ dc_usbhid_iterator_next (dc_iterator_t *abstract, void *out)
 		struct hid_device_info *current = iterator->current;
 		iterator->current = current->next;
 
-		dc_usb_desc_t usb = {current->vendor_id, current->product_id};
-		if (!dc_descriptor_filter (iterator->descriptor, DC_TRANSPORT_USBHID, &usb, NULL)) {
+		dc_usbhid_desc_t usb = {current->vendor_id, current->product_id};
+		if (!dc_descriptor_filter (iterator->descriptor, DC_TRANSPORT_USBHID, &usb)) {
 			continue;
 		}
 
