@@ -26,6 +26,7 @@
 #include "uwatec_memomouse.h"
 #include "context-private.h"
 #include "parser-private.h"
+#include "platform.h"
 #include "array.h"
 
 #define ISINSTANCE(parser) dc_parser_isinstance((parser), &uwatec_memomouse_parser_vtable)
@@ -38,7 +39,6 @@ struct uwatec_memomouse_parser_t {
 	dc_ticks_t systime;
 };
 
-static dc_status_t uwatec_memomouse_parser_set_data (dc_parser_t *abstract, const unsigned char *data, unsigned int size);
 static dc_status_t uwatec_memomouse_parser_set_clock (dc_parser_t *abstract, unsigned int devtime, dc_ticks_t systime);
 static dc_status_t uwatec_memomouse_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *datetime);
 static dc_status_t uwatec_memomouse_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsigned int flags, void *value);
@@ -47,7 +47,6 @@ static dc_status_t uwatec_memomouse_parser_samples_foreach (dc_parser_t *abstrac
 static const dc_parser_vtable_t uwatec_memomouse_parser_vtable = {
 	sizeof(uwatec_memomouse_parser_t),
 	DC_FAMILY_UWATEC_MEMOMOUSE,
-	uwatec_memomouse_parser_set_data, /* set_data */
 	uwatec_memomouse_parser_set_clock, /* set_clock */
 	NULL, /* set_atmospheric */
 	NULL, /* set_density */
@@ -59,7 +58,7 @@ static const dc_parser_vtable_t uwatec_memomouse_parser_vtable = {
 
 
 dc_status_t
-uwatec_memomouse_parser_create (dc_parser_t **out, dc_context_t *context, unsigned int devtime, dc_ticks_t systime)
+uwatec_memomouse_parser_create (dc_parser_t **out, dc_context_t *context, const unsigned char data[], size_t size)
 {
 	uwatec_memomouse_parser_t *parser = NULL;
 
@@ -67,15 +66,15 @@ uwatec_memomouse_parser_create (dc_parser_t **out, dc_context_t *context, unsign
 		return DC_STATUS_INVALIDARGS;
 
 	// Allocate memory.
-	parser = (uwatec_memomouse_parser_t *) dc_parser_allocate (context, &uwatec_memomouse_parser_vtable);
+	parser = (uwatec_memomouse_parser_t *) dc_parser_allocate (context, &uwatec_memomouse_parser_vtable, data, size);
 	if (parser == NULL) {
 		ERROR (context, "Failed to allocate memory.");
 		return DC_STATUS_NOMEMORY;
 	}
 
 	// Set the default values.
-	parser->devtime = devtime;
-	parser->systime = systime;
+	parser->devtime = 0;
+	parser->systime = 0;
 
 	*out = (dc_parser_t*) parser;
 
@@ -83,13 +82,6 @@ uwatec_memomouse_parser_create (dc_parser_t **out, dc_context_t *context, unsign
 }
 
 
-static dc_status_t
-uwatec_memomouse_parser_set_data (dc_parser_t *abstract, const unsigned char *data, unsigned int size)
-{
-	return DC_STATUS_SUCCESS;
-}
-
-
 static dc_status_t
 uwatec_memomouse_parser_set_clock (dc_parser_t *abstract, unsigned int devtime, dc_ticks_t systime)
 {
@@ -137,7 +129,7 @@ uwatec_memomouse_parser_get_field (dc_parser_t *abstract, dc_field_type_t type,
 
 	unsigned int model = data[3];
 
-	int is_nitrox = 0, is_oxygen = 0, is_air = 0;
+	int is_nitrox = 0, is_oxygen = 0, DC_ATTR_UNUSED is_air = 0;
 	if ((model & 0xF0) == 0xF0)
 		is_nitrox = 1;
 	if ((model & 0xF0) == 0xA0)
@@ -166,6 +158,7 @@ uwatec_memomouse_parser_get_field (dc_parser_t *abstract, dc_field_type_t type,
 			*((unsigned int *) value) = 1;
 			break;
 		case DC_FIELD_GASMIX:
+			gasmix->usage = DC_USAGE_NONE;
 			gasmix->helium = 0.0;
 			if (size >= header + 18) {
 				if (is_oxygen)
@@ -197,6 +190,7 @@ uwatec_memomouse_parser_get_field (dc_parser_t *abstract, dc_field_type_t type,
 			}
 			tank->endpressure = 0.0;
 			tank->gasmix = 0;
+			tank->usage = DC_USAGE_NONE;
 			break;
 		case DC_FIELD_TEMPERATURE_MINIMUM:
 			*((double *) value) = (signed char) data[15] / 4.0;
@@ -221,7 +215,7 @@ uwatec_memomouse_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callba
 
 	unsigned int model = data[3];
 
-	int is_nitrox = 0, is_oxygen = 0, is_air = 0;
+	int is_nitrox = 0, is_oxygen = 0, DC_ATTR_UNUSED is_air = 0;
 	if ((model & 0xF0) == 0xF0)
 		is_nitrox = 1;
 	if ((model & 0xF0) == 0xA0)
@@ -250,17 +244,17 @@ uwatec_memomouse_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callba
 		offset += 2;
 
 		// Time (seconds)
-		sample.time = time;
-		if (callback) callback (DC_SAMPLE_TIME, sample, userdata);
+		sample.time = time * 1000;
+		if (callback) callback (DC_SAMPLE_TIME, &sample, userdata);
 
 		// Depth (meters)
 		sample.depth = depth * 10.0 / 64.0;
-		if (callback) callback (DC_SAMPLE_DEPTH, sample, userdata);
+		if (callback) callback (DC_SAMPLE_DEPTH, &sample, userdata);
 
 		// Gas change.
 		if (gasmix != gasmix_previous) {
 			sample.gasmix = gasmix;
-			if (callback) callback (DC_SAMPLE_GASMIX, sample, userdata);
+			if (callback) callback (DC_SAMPLE_GASMIX, &sample, userdata);
 			gasmix_previous = gasmix;
 		}
 
@@ -272,7 +266,8 @@ uwatec_memomouse_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callba
 		}
 		sample.deco.time = 0;
 		sample.deco.depth = 0.0;
-		if (callback) callback (DC_SAMPLE_DECO, sample, userdata);
+		sample.deco.tts = 0;
+		if (callback) callback (DC_SAMPLE_DECO, &sample, userdata);
 
 		// Warnings
 		for (unsigned int i = 0; i < 6; ++i) {
@@ -301,7 +296,7 @@ uwatec_memomouse_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callba
 					break;
 				}
 				if (sample.event.type != SAMPLE_EVENT_NONE) {
-					if (callback) callback (DC_SAMPLE_EVENT, sample, userdata);
+					if (callback) callback (DC_SAMPLE_EVENT, &sample, userdata);
 				}
 			}
 		}
@@ -325,7 +320,7 @@ uwatec_memomouse_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callba
 				offset++;
 			}
 
-			if (callback) callback (DC_SAMPLE_VENDOR, sample, userdata);
+			if (callback) callback (DC_SAMPLE_VENDOR, &sample, userdata);
 		}
 
 		time += 20;
