@@ -23,6 +23,7 @@
 #define UWATEC_SMART_H
 
 #include <libdivecomputer/context.h>
+#include <libdivecomputer/iostream.h>
 #include <libdivecomputer/device.h>
 #include <libdivecomputer/parser.h>
 
@@ -31,10 +32,10 @@ extern "C" {
 #endif /* __cplusplus */
 
 dc_status_t
-uwatec_smart_device_open (dc_device_t **device, dc_context_t *context);
+uwatec_smart_device_open (dc_device_t **device, dc_context_t *context, dc_iostream_t *iostream);
 
 dc_status_t
-uwatec_smart_parser_create (dc_parser_t **parser, dc_context_t *context, unsigned int model, unsigned int devtime, dc_ticks_t systime);
+uwatec_smart_parser_create (dc_parser_t **parser, dc_context_t *context, const unsigned char data[], size_t size, unsigned int model);
 
 #ifdef __cplusplus
 }
